# EMEP DATABASE FUNCTIONS
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @date 2015-12-04
#' @version 13.0


# FAQ ----
## NEW_v12_001
#' Some information regarding the database.
#'
#' Q: How do I find out which data is present (substance and time)?
#' A: Do it look up in 'sgl_files' and 'f_substances'. This goes fast.
#'    If you exactly know, which data you want get the data from 'data'.
#'    It could be usefull to use file_id or sf_id to have short queries.
#'
#' Q: Was does 'complete' indicate?
#' A: To each entry in sgl_file one or more entries in f_substances exist.
#'    And to each entry in f_substances one or more entries in data exist.
#'    The boolean 'complete' just indicates whether all entries in the 
#'    hierarchie below are inserted correctly.
#'    More detailed: Sometimes, entries cannot be added into data (e.g. 
#'    bad numflag). In such a case 'complete' of the entry in f_substance, 
#'    to which the faulty entry in data belongs to, is set to FALSE. 
#'    Otherwise it is set to TRUE. If at least one entry of f_substances
#'    belonging to one entry of sgl_files is not complete, sgl_files'
#'    entry's complete is also set to FALSE.
#'
#' Q: Why do we not have just a table 'data' but four tables 'files', 
#' 'sgl_files', 'f_substances' and 'data'?
#' A: In a SQL database one wants to avoid redundant data and arrays.
#'    For the last reason, each measurement needs its own entry in one
#'    table. This table we call 'data'.
#'    Each times series of measurements has its metadata. Some metadata is 
#'    equal for all times series in one sourde file (emep data file) such 
#'    as start and end time, station_id or measurement interval. 'sgl_files'
#'    was introduced to contain this data.
#'    However, there is some meta information which is constant over several
#'    years (= for several files). Therefore the table 'files' was introduced
#'    to aggregate this metadata.
#'    When one inserts times series it is interesting to know whether the
#'    time series was already inserted. One could test 'data' for each value.
#'    This may take quite long. Also one could want to have a list of measured
#'    substances or could want to know when certain substances were measured.
#'    These tasks could be solved by lists/arrays in 'sgl_files' and by 
#'    querying 'data'. However, this takes time and does not fit into our 
#'    database concept. Thereore, 'f_substance' was introduced as layer 
#'    between 'sgl_files' and 'data'.
#'    


# load libraries and source R scripts ----
library("RODBC")


# define functions ----
# GET_DATA
#' TODO
#' 
#' Not fully implemented, yet. Please use the ctmeval package.
#' 
#' @param 
#' @return  
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note ... 
#' @format R Code
get_data <- function(measurement_id=-1,
                     file_id=-1,
                     station_id="",
                     substance_short="",
                     substance_long="",
                     substance="",
                     matrix="",
                     start_time_yyyyddd=-1,
                     start_time_hh=-1,
                     end_time_yyyyddd=-1,
                     end_time_hh=-1,
                     numflag=-1
                     ) {
  .NotYetImplemented()
  
  # buildCondition4Query
  #' TODO
  #' 
  #' @param varName ; 
  #' @param varValue ; 
  #' @param firstCondition ; 
  #' @return part of a condition string for a PostGreSQL SELECT call;
  #' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
  #'    \email{daniel.neumann@hzg.de}
  #' @format R Code
  buildCondition4Query <- function(varName, varValue, firstCondition) {
    return(paste(ifelse(firstCondition, " WHERE ", " AND "), varName, "=", varValue, sep=""))
  }
  
  #if (length()) {
    # if two substance strings are parse we throw an error.
  #}
  
  outColumns=c("start_time_yyyyddd",
               "start_time_hh",
               "end_time_yyyyddd",
               "end_time_hh",
               "file_id", 
               "substance_short", 
               "matrix", 
               "unit", 
               "value",
               "numflag")
  # ToDo: Modifiy if one condition =="" or more than one value
  
  queryString = paste("SELECT ",
                      paste(outColumns, collapse=", "),
                      " FROM data", sep="")
  
  nDataSets = 1
  firstCondition=TRUE
  selectParams = list(
    "measurement_id"=c(measurement_id, -1),
    "file_id"=c(file_id, -1),
    "substance_short"=c(substance_short, ""),
    "station_id"=c(station_id, ""),
    "matrix"=c(matrix,""),
    "start_time_yyyyddd"=c(start_time_yyyyddd,-1),
    "start_time_hh"=c(start_time_hh,-1),
    "end_time_yyyyddd"=c(end_time_yyyyddd,-1),
    "end_time_hh"=c(end_time_hh,-1),
    "numflag"=c(numflag,-1)
    )
  
  
  for (i1 in 1:length(selectParams)) {
    if (selectParams[[i1]][1] != selectParams[[i1]][2]) {
      queryString = paste(queryString, 
                          buildCondition4Query(names(selectParams)[i1], 
                                               selectParams[[i1]][1],
                                               firstCondition), 
                          sep="")
      firstCondition=FALSE
    }
  }
  
  queryString = paste(queryString, " ORDER BY ", paste(outColumns[1:6], collapse=", "), ";", sep="")
  
  dbOut = sqlQuery(myCon, queryString)
  
  # ToDo: 
  # - Split by file_id into different emep_data-lists/objects
  # - Get addFileInfo for each file_id ones
  # - put everything in a list or an array:
  #      list(emep_data_1, emep_data_2, ...)
  
  dummySet = empty_emep_data()
  
  return(dbOut)
}

# fill_data
#' TODO
#' 
#' @return TODO
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @format R Code
fill_data <- function() { .NotYetImplemented() }


# GET_ID
#' TODO
#' 
#' @param TODO
#' @return  TODO
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @format R Code
get_id <- function() { .NotYetImplemented() }

# GET_NAME
#' TODO
#' 
#' @param TODO
#' @return TODO
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @format R Code
get_name <- function() { .NotYetImplemented() }

# ID_EXISTS
#' TODO
#' 
#' @param TODO
#' @return TODO
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @format R Code
id_exists <- function() { .NotYetImplemented() }


## +--------------------------------------------+
## | FUNCTION: ADD_FILE
## +--------------------------------------------+
#'
#' 
#' TODO
#' 
#' @param 
#' @return  
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note ... 
#' @format R Code
add_file <- function(dbCon, fName, fDir, fast = FALSE) {
  err = 0
  err_file = 0
  noInsert = FALSE
  file_id = -1
  # file_name = paste("'", fName, "'", sep="")
  # general_file_name = paste("'", substr(fName, start=1, stop=8), "XXXXXXXX", substr(fName, start=17, stop=200), "'", sep="")
  ## NEW_v13_000
  fName_old = paste(substr(fName,1,16), substr(fName,23,31), substr(fName,38,256), sep = '')
  file_name = paste("'", substr(fName,1,16), substr(fName,23,31), substr(fName,38,256), "'", sep="")
  general_file_name = paste("'", substr(fName, start=1, stop=8), "XXXXXXXX", substr(fName,23,31), substr(fName, start=38, stop=200), "'", sep="")
  #print(general_file_name)
  
  if (file.exists(paste(fDir, fName ,sep="/"))) {
    # Read EMEP input file and remove duplicate variables
    
    ## NEW_v12_003
    emep_data <- emep_removeDuplicatesB(emep_file_parser(fDir, fName))$new_data
    ## NEW_v13_000
    emep_data$filename = paste(substr(fName,1,16), substr(fName,23,31), substr(fName,38,256), sep = '')
    tmpDim = dim(emep_data$flags$numflags) 
    ## NEW_v13_000
    ## DO NOT DO THIS. The correct dimension might be c(tmpDim[1], 1, tmpDim[2]) or
    ## c(1, tmpDim[1], tmpDim[2]). 
    if ( length(tmpDim) == 2 ) {
      dim(emep_data$flags$numflags) = c(emep_data$nRowData, emep_data$nVars,
                                        length(emep_data$flags$numflags)/emep_data$nRowData/emep_data$nVars)
    }
    ## NEW_v12_003
    
    ## NEW_v13_000
    tmpDim = dim(emep_data$measurements$variables)
    if (is.null(tmpDim)) tmpDim = 0
    if (length(tmpDim) == 1) {
      dim(emep_data$measurements$variables) = c(emep_data$nRowData, emep_data$nVars)
    }
    ## NEW_v13_000
    
    # OLD: emep_data <- emep_removeDuplicates(emep_file_parser(fDir, fName))$new_data
    
    #print(names(emep_data))
    
    nVars = emep_data$nVars
    nTimes = emep_data$nRowData
    substances_short = emep_data$measurements$titleVarsShort
    
    station_id = paste("'", emep_data$addDataInfo[["Station code"]], "'", sep="")
    start_time = emep_data$timing$times[1,1]
    end_time = emep_data$timing$times[emep_data$nRowData,1]
    data_definition = paste("'", emep_data$addDataInfo[["Data definition"]], "'", sep="")
    set_type_code = paste("'", emep_data$addDataInfo[["Set type code"]], "'", sep="")
    regime = paste("'", emep_data$addDataInfo[["Regime"]], "'", sep="")
    station_code = paste("'", emep_data$addDataInfo[["Station code"]], "'", sep="")
    platform_code = paste("'", emep_data$addDataInfo[["Platform code"]], "'", sep="")
    tmp_startdate = emep_data$addDataInfo[["Startdate"]]
    startdate = paste("'", substr(tmp_startdate, 1, 4), "-", substr(tmp_startdate, 5, 6), "-", substr(tmp_startdate, 7, 8), "'", sep="")
    tmp_timeref = emep_data$addDataInfo[["Timeref"]]
    timeref = ifelse(is.null(tmp_timeref), "'00:00:00'", paste("'", substr(tmp_timeref, 1, 2), ":", substr(tmp_timeref, 4, 5), ":00'", sep=""))
    tmp_revision_date = emep_data$addDataInfo[["Revision date"]]
    revision_date = paste("'", substr(tmp_revision_date, 1, 4), "-", substr(tmp_revision_date, 5, 6), "-", substr(tmp_revision_date, 7, 8), "'", sep="")
    component = paste("'", emep_data$addDataInfo[["Component"]], "'", sep="")
    unit = paste("'", emep_data$addDataInfo[["Unit"]], "'", sep="")
    matrix = paste("'", emep_data$addDataInfo[["Matrix"]], "'", sep="")
    period_code = paste("'", emep_data$addDataInfo[["Period code"]], "'", sep="")
    resolution_code = paste("'", emep_data$addDataInfo[["Resolution code"]], "'", sep="")
    laboratory_code = paste("'", emep_data$addDataInfo[["Laboratory code"]], "'", sep="")
    instrument_type = paste("'", emep_data$addDataInfo[["Instrument type"]], "'", sep="")
    instrument_name = paste("'", emep_data$addDataInfo[["Instrument name"]], "'", sep="")
    method_ref = paste("'", emep_data$addDataInfo[["Method ref"]], "'", sep="")
    ext_lab_code = paste("'", emep_data$addDataInfo[["Ext. lab. code"]], "'", sep="")
    ext_meth_ref = paste("'", emep_data$addDataInfo[["Ext. meth. ref"]], "'", sep="")
    add_qualifier = paste("'", emep_data$addDataInfo[["Add. qualifier"]], "'", sep="")
    
    start_time_YYYYDDD = emep_data$timing$times[1,1]
    start_time_HH = emep_data$timing$times[1,2]
    end_time_YYYYDDD = emep_data$timing$times[nTimes,3]
    end_time_HH = emep_data$timing$times[nTimes,4]
    
    duplicateString_sgl_files = paste("SELECT sgl_id, file_id, complete FROM sgl_files WHERE ",
                            "file_name=", file_name, ";", sep="")
    queryDuplicate_sgl_files = sqlQuery(myCon, query=duplicateString_sgl_files)
    #print(duplicateString_sgl_files)
    #print(queryDuplicate_sgl_files)
    
    if (dim(queryDuplicate_sgl_files)[1] > 0) {
      file_id = as.integer(queryDuplicate_sgl_files[1,"file_id"])
      sgl_id = as.integer(queryDuplicate_sgl_files[1,"sgl_id"])
      if(queryDuplicate_sgl_files[1,"complete"]) noInsert = TRUE
      err_file=as.integer(-1)
      queryInsert <- ""
    } else {
      like_file_name = paste("'", substr(fName_old, start=1, stop=8), '%', substr(fName_old, start=17, stop=200), "'", sep="")
      duplicateString_files = paste("SELECT file_id FROM files WHERE ",
                              "file_name LIKE ", like_file_name, ";", sep="")
      queryDuplicate_files = sqlQuery(myCon, query=duplicateString_files)
      #print(duplicateString_files)
      #print(queryDuplicate_files)
      if (dim(queryDuplicate_files)[1] > 0) {
        file_id = as.integer(queryDuplicate_files[1,"file_id"])
        queryInsert <- ""
      } else {
        # insert file ... 
        insertString_files = paste("INSERT INTO files",
                             " (file_name, station_id, ",
                             "data_definition, set_type_code, regime, station_code, ",
                             "platform_code, timeref, revision_date, ",
                             "component, unit, matrix, period_code, resolution_code, ",
                             "laboratory_code, instrument_type, instrument_name, ",
                             "method_ref, ext_lab_code, ext_meth_ref, add_qualifier)",
                             " VALUES(",
                             " ", general_file_name, 
                             ", ", station_id, 
                             ", ", data_definition, 
                             ", ", set_type_code, 
                             ", ", regime, 
                             ", ", station_code, 
                             ", ", platform_code, 
                             ", ", timeref, 
                             ", ", revision_date, 
                             ", ", component, 
                             ", ", unit, 
                             ", ", matrix, 
                             ", ", period_code, 
                             ", ", resolution_code, 
                             ", ", laboratory_code, 
                             ", ", instrument_type, 
                             ", ", instrument_name, 
                             ", ", method_ref, 
                             ", ", ext_lab_code, 
                             ", ", ext_meth_ref, 
                             ", ", add_qualifier,
                             ");", 
                             sep="")
        queryInsert_files <- sqlQuery(dbCon, query=insertString_files)
        #print(queryInsert_files)
        if (length(grep("ERROR", queryInsert_files)) == 0) { #NO ERROR
          queryGetID <- sqlQuery(dbCon, query=duplicateString_files)
          file_id = as.integer(queryGetID[1,"file_id"])
        } else { 
          err_file = as.integer(2)
          noInsert = TRUE
          #print(queryInsert)
        }
      }
      
      # browser()
      
      if (!noInsert) {
        insertString_sgl_files = paste("INSERT INTO sgl_files",
                                     "(file_name, start_time_yyyyddd, start_time_hh, ",
                                     "end_time_yyyyddd, end_time_hh, startdate,",
                                     "file_id, n_timesteps, special_comment_present)", # NEW_v12_004
# OLD:                               "file_id, n_timesteps)",
                                     " VALUES(",
                                     " ", file_name, 
                                     ", ", start_time_YYYYDDD,
                                     ", ", start_time_HH, 
                                     ", ", end_time_YYYYDDD,
                                     ", ", end_time_HH, 
                                     ", ", startdate, 
                                     ", ", file_id,
                                     ", ", emep_data$nRowData,
                                     ", ", as.character(!(emep_data$specialInfo=="")), # NEW_v12_004
                                     ");", 
                                     sep="")
        queryInsert_sgl_files <- sqlQuery(dbCon, query=insertString_sgl_files)
        #print(insertString_sgl_files)
        #print(queryInsert_sgl_files)
        if (length(grep("ERROR", queryInsert_sgl_files)) == 0) { #NO ERROR
          # everything o.k.
          print('test')
          get_sgl_id = sqlQuery(dbCon, query=duplicateString_sgl_files)
          #print(duplicateString_sgl_files)
          print(get_sgl_id)
          # browser()
          sgl_id = get_sgl_id[1,"sgl_id"]
        } else { 
          err_file = as.integer(2)
          noInsert = TRUE
          sgl_id = -1
          #print(queryInsert)
        }
      }
    }
  } else {
    err_file = 4
    sgl_id = -1
    noInsert = TRUE
    nVars = 0
    nTimes = 0
    substances_short = c(" ")
    queryInsert <- ""
  }
  #print(err_file)
  
  if (!noInsert) {
    print("adding Data ...")
    #print(file_id)
    #print(sgl_id)
    #print(fast)
    #print(emep_data$flags$numflags)
    dOut <- add_data(myCon, file_id, sgl_id, emep_data, fast)
  } else {
    dOut <- list("err"=as.integer(0), "id"=as.integer(-1), "n"=as.integer(nVars*nTimes), 
                 "dims"=c(as.integer(0), as.integer(0)), "errN"=as.integer(0), 
                 "errs"=as.integer(0), "new"=as.integer(0))
  }
  
  tmp_err = length(which(dOut$errs > 0)) + dOut$err
  if (tmp_err > 0 || err_file > 0) {
    err = err + as.integer(1)
  } 
  
  if (err <= 0) {
    completeQuery_f = paste("UPDATE sgl_files SET ",
                          "complete=TRUE",
                          " WHERE ",
                          "file_name=", file_name,
                          ";", sep="")
    queryUpdate = sqlQuery(myCon, query=completeQuery_f)
    if (length(grep("ERROR", queryUpdate)) > 0) err = err + 8
  }
  
  
  if (err_file == 0) {
    read_file = "1"
    new_file = as.integer(1)
  } else {
    new_file = as.integer(0)
    if (err_file==4) { # ToDo: Change
      read_file="No"
    } else {
      read_file="1"
    }
  }
  
  outString = paste(read_file, " file was read in containing ", nVars, " variables over ", nTimes, " time steps each. \n",
                    new_file , " of ", 1, " files and meta data sets were INSERTed into table FILES. \n",
                    length(which(dOut$complete==TRUE)) , " of ", nVars, " variables were INSERTed into table MEASUREMENTS. \n",
                    dOut$new, " of ", nTimes*nVars, " data sets (/measurement values) were INSERTed into table DATA. \n", sep="")
  if (err <= 0) {
    outString = paste(outString, "No error occured. \n", sep = "")
  } else {
    outString = paste(outString, "Error(s) occured. \n", sep = "")
  }
  
  cat(outString)
  
  return(list(
    "file_id"=file_id,
    "sgl_id"=sgl_id,
    "file_name"=file_name,
    "substances_short"=substances_short,
    "data_ids"=dOut$ids,
    "n_file"=as.integer(1),
    "n_measurement"=nVars,
    "n_data"=dOut$n,
    "new_file"=new_file,
    "new_measurement"=length(which(dOut$complete==TRUE)),
    "new_data"=dOut$new,
    "dims_data"=dOut$dims,
    "err"=err,
    "err_file"=err_file,
    "err_data"=dOut$err,
    "errN_data"=dOut$err,
    "errs_data"=dOut$errs,
    "text"=outString
    )
         )
  
  # err=-1 => data is already present
  # err=+1 => some unknown error occured
  # err=+2 => INSERT did not work properly
  # data_err=+4 => (length(measurement_id's) != nVars) in add_data
  # file_err=+4 => file does not exist

}



## +--------------------------------------------+
## | FUNCTION: ADD_DATA
## +--------------------------------------------+
#'
#' 
#' TODO
#' 
#' @param 
#' @return  
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note ... 
#' @format R Code
add_data <- function(dbCon, file_id_IN, sgl_id_IN, emep_data, fast=FALSE) {
  #print(emep_data)
  nVars = emep_data$nVars
  nRows = emep_data$nRowData
  complete = rep(FALSE, nVars)
  
  errs = as.integer(rep(-1, nVars*nRows))
  dim(errs) = c(nRows, nVars)
  
  ids = as.integer(rep(0, nVars*nRows))
  dim(ids) = c(nRows, nVars)
  
  if (length(sgl_id_IN) != nVars && length(sgl_id_IN) != 1) {
    err = as.integer(4)
  } else {
    err = as.integer(0)
    
    file_id = format(as.integer(file_id_IN))
    if (length(sgl_id_IN) == 1 && nVars > 1) {
      sgl_ids = as.integer(rep(sgl_id_IN, nVars))
    } else {
      sgl_ids = sgl_id_IN
    }
      
    #
    tmp_id=as.integer(0)
    tmp_err=as.integer(0)
        
    noInsert=FALSE
    station_id = paste("'", emep_data$addDataInfo[["Station code"]], "'", sep="")
    # OLD: matrix = paste("'", emep_data$addDataInfo$Matrix, "'", sep="")
    ## see NEW_v12_0 for replacement; now each time series has its own matrix
    
    # Go through all variables except of those which measurements were not
    # inserted correctly into the measurements table. If varibale i was not
    # inserted correctly measurement_id_IN is set to -1.
    if(length(which(sgl_id_IN==-1))>0) {
      cols = (1:nVars)[-which(sgl_id_IN==-1)]
    } else {
      cols = 1:nVars
    }
        
    for (iCol in cols) {
      sgl_id = format(as.integer(sgl_ids[iCol]))
      substance_short = paste("'", emep_data$measurements$titleVarsShort[iCol], "'", sep="")
      unit = paste("'", emep_data$measurements$units[iCol], "'", sep="")
      matrix = paste("'", emep_data$measurements$matrices[iCol], "'", sep="") # NEW_v12_005
      method_ref = paste("'", emep_data$measurements$method_refs[iCol], "'", sep="") # NEW_v12_005
      
      sf_exists = exists_f_substance(dbCon, sgl_id, substance_short, matrix, method_ref) # NEW_v12_006
      # OLD: sf_exists = exists_f_substance(dbCon, sgl_id, substance_short)
      #print(sf_exists$exists)
      if (!sf_exists$exists) sf_exists = add_f_substance(dbCon, sgl_id, substance_short, matrix, method_ref) # NEW_v12_007
      # OLD: if (!sf_exists$exists) sf_exists = add_f_substance(dbCon, sgl_id, substance_short)
      #print(sf_exists$exists)
      #print(sf_exists$complete)
      
      if (!sf_exists$complete) {
        sf_id = sf_exists$sf_id
        #print(sf_id)
        for (iRow in 1:nRows) {
        # for (iRow in 1:1) {
          start_time_YYYYDDD = emep_data$timing$times[iRow,1]
          start_time_HH = emep_data$timing$times[iRow,2]
          end_time_YYYYDDD = emep_data$timing$times[iRow,3]
          end_time_HH = emep_data$timing$times[iRow,4]
          # browser()
          value = format(emep_data$measurements$variables[iRow,iCol], scientific=TRUE)
          #print(dim(emep_data$flags$numflags))
          numflag = as.integer(emep_data$flags$numflags[iRow,iCol,]) # NEW_v12_007
          # OLD: numflag = as.integer(emep_data$flags$numflags[iRow,iCol])
          if (fast) {
            noInsert=FALSE
          } else {
            duplicateString = paste("SELECT data_id, value, numflag FROM data WHERE ",
                                    "file_id=", file_id,
                                    " AND sf_id=", sf_id, 
                                    " AND start_time_YYYYDDD=", start_time_YYYYDDD, 
                                    " AND start_time_HH=", start_time_HH, 
                                    " AND end_time_YYYYDDD=", end_time_YYYYDDD, 
                                    " AND end_time_HH=", end_time_HH, ";", 
                                    sep="")
            queryDuplicate <- sqlQuery(dbCon, query=duplicateString)
            #print(queryDuplicate)
            if (dim(queryDuplicate)[1] > 0) {
              noInsert=TRUE
              tmp_id = as.integer(queryDuplicate[1,1])
              tmp_err = as.integer(-1)
            }
          }
          
          if (!noInsert)  {
            # ToDo
            insertString = paste("INSERT INTO data",
                                 "(sf_id, station_id, substance_short, matrix, unit, ", 
                                 "start_time_YYYYDDD, start_time_HH, end_time_YYYYDDD, end_time_HH, ",
                                 "value, numflag1, numflag2, numflag3, file_id)", #NEW_v12_008
                                 # OLD: "value, numflag, file_id)",
                                 " VALUES(",
                                 sf_id, ", ", station_id, ", ", substance_short, ", ",
                                 matrix, ", ", unit, ", ", 
                                 start_time_YYYYDDD, ", ", start_time_HH, ", ", end_time_YYYYDDD, ", ", end_time_HH,  ", ",
                                 value, ", ", numflag[1], ", ", numflag[2], ", ", numflag[3], ", ", file_id, ");", # NEW_v12_008
                                 # OLD: value, ", ", numflag, ", ", file_id, ");", 
                                 sep="")
            #print(insertString)
            queryInsert <- sqlQuery(dbCon, query=insertString)
            #print(queryInsert)
            if (length(grep("ERROR", queryInsert)) == 0) { #NO ERROR
              if (fast) {
                tmp_id = as.integer(-1)
              } else {
                queryGetID <- sqlQuery(dbCon, query=duplicateString)
                tmp_id = as.integer(queryGetID[1,1])
              }
              tmp_err = 0
            } else { 
              tmp_id = as.integer(-1)
              tmp_err = as.integer(2)
            }
          }
          errs[iRow, iCol] = tmp_err
          ids[iRow, iCol] = tmp_id
        }
      }
      if (length(which(errs[,iCol]>0))==0) {
        complete[iCol] = TRUE
        sf_exists = complete_f_substance_B(dbCon, sf_exists$sf_id)
      }
    }
    
    if (length(which(errs>0))>0) err = err + 1
  }
  
  return(list(
    "err"=err, "id"=ids, "n"=nRows*nVars, "dims"=c(nRows, nVars), 
    "errN"=length(which(errs>0)), "errs"=errs, "new"=length(which(errs==0)),
    "complete"=complete))
}


# EXISTS_F_SUBSTANCE
#'
#' 
#' TODO
#' 
#' @param 
#' @return  
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note ... 
#' @format R Code
exists_f_substance <- function(dbCon, sgl_id, substance_short, matrix, method_ref) { # NEW_v12_009
# OLD: exists_f_substance <- function(dbCon, sgl_id, substance_short) {
  queryString = paste("SELECT sf_id, complete FROM f_substances WHERE ", 
                      "sgl_id=", sgl_id, 
                      " AND substance_short=", substance_short,
                      " AND matrix=", matrix, # NEW_v12_009
                      " AND method_ref=", method_ref, # NEW_v12_009
                      ";", sep="")
  
  queryOut = sqlQuery(dbCon, query=queryString)
  #print(queryString)
  #print(queryOut)
  if (length(grep("ERROR", queryOut)) == 0) {
    err = 0 
    if (dim(queryOut)[1] > 0) {
      exists = TRUE
      sf_id = queryOut[1, "sf_id"]
      if (queryOut[1, "complete"]) complete = TRUE else complete = FALSE
    } else {
      exists = FALSE
      complete = FALSE
      sf_id = -1
    }
  } else {
    err = 1
    exists = FALSE
    complete = FALSE
    sf_id = -1
  }
  
  return(list("err"=err, "exists"=exists, "complete"=complete, "sf_id"=sf_id))
}


# ADD_F_SUBSTANCE
#'
#' 
#' TODO
#' 
#' @param 
#' @return  
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note ... 
#' @format R Code
add_f_substance <- function(dbCon, sgl_id, substance_short, matrix, method_ref) {  # NEW_v12_010
# OLD: add_f_substance <- function(dbCon, sgl_id, substance_short) {  
  queryString = paste("INSERT INTO f_substances (sgl_id, substance_short, matrix, method_ref)",  # NEW_v12_010
  # OLD: queryString = paste("INSERT INTO f_substances (sgl_id, substance_short)", 
                      " VALUES(", sgl_id, ", ", 
                                  substance_short, ", ",  
                                  matrix, ", ",          # NEW_v12_010
                                  method_ref,            # NEW_v12_010
                      ");", sep="")
  queryOut = sqlQuery(dbCon, query=queryString)
  #print(queryString)
  #print(queryOut)
  
  if (length(grep("ERROR", queryOut)) == 0) {
    queryResult = exists_f_substance(dbCon, sgl_id, substance_short, matrix, method_ref)
    # OLD: queryResult = exists_f_substance(dbCon, sgl_id, substance_short) # NEW_v12_0
    #print(queryResult)
    if (queryResult$err != 0) {
      err = queryResult$err
      sf_id = -1
      exists=FALSE
      complete=FALSE
    } else {
      if (queryResult$exists) {
        err = 0
        sf_id = queryResult$sf_id
        exists=TRUE
        complete=FALSE
      } else {
        err = 2
        sf_id = -1
        exists=FALSE
        complete=FALSE
      }
    }
  } else {
    err = 1
    sf_id = -1
    exists=FALSE
    complete=FALSE
  }
  
  return(list("err"=err, "exists"=exists, "complete"=complete, "sf_id"=sf_id))
}


# complete_f_substance_A
#'
#' Sets the 'complete' boolean of one entry in table 'f_substances'
#' to TRUE. If an entry does not exist it is created.
#' 
#' Used functions:
#'  complete_f_substance_B
#'  exists_f_substance
#'  add_f_substance
#'  
#' 
#' @param dbCon ; handle to existing database connection
#' @param sgl_id ; id of an entry in the sgl_files table
#' @param substance_short ; 
#' @param matrix ; 
#' @param method_ref ; 
#' @return list(err, exists, complete, sf_id) ; list containing one integer
#'          - indicating if no error (==0) or an error (!=0) occured -, three 
#'          booleans - indicating if the endtry already exists and if it is
#'          complete - and the sf_id which was parsed at the beginning.
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note ... 
#' @format R Code
complete_f_substance_A <- function(dbCon, sgl_id, substance_short, matrix, method_ref) { # NEW_v12_011
# OLD: complete_f_substance_A <- function(dbCon, sgl_id, substance_short) {
  queryExists = exists_f_substance(dbCon, sgl_id, substance_short, matrix, method_ref) # NEW_v12_011
  # OLD: queryExists = exists_f_substance(dbCon, sgl_id, substance_short) 
  
  if (queryExists$err != 0) {
    err = queryExists$err
    sf_id = -1
    exists=FALSE
    complete=FALSE
  } else {
    if (!queryExists$exists) {
      addCall = add_f_substance(dbCon, sgl_id, substance_short, matrix, method_ref) # NEW_v12_011
      # OLD: addCall = add_f_substance(dbCon, sgl_id, substance_short)
      sf_id = addCall$sf_id
    } else {
      sf_id = queryExists$sf_id
    }
    exists=TRUE
    queryString = paste("UPDATE f_substances SET complete=TRUE WHERE ",
                        "sf_id=", sf_id, 
                        ";", sep="")
    queryOut = sqlQuery(dbCon, query=queryString)
    if (length(grep("ERROR", queryOut)) != 0) {
      err=1
      complete=FALSE
    } else {
      err=0
      complete=TRUE
    }
  }
  
  return(list("err"=err, "exists"=exists, "complete"=complete, "sf_id"=sf_id))
}


# complete_f_substance_B
#'
#' Sets the 'complete' boolean of one entry in table 'f_substances'
#' to TRUE. Returns whether it was successful or not. Does not test
#' whether an entry exists, in advance.
#' 
#' 
#' @param dbCon ; handle to existing database connection
#' @param sf_id ; id of an entry in the f_substances table
#' @return list(err, exists, complete, sf_id) ; list containing one integer
#'          - indicating if no error (==0) or an error (!=0) occured -, three 
#'          booleans - indicating if the endtry already exists and if it is
#'          complete - and the sf_id which was parsed at the beginning.
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note ... 
#' @format R Code
complete_f_substance_B <- function(dbCon, sf_id) {
  queryString = paste("UPDATE f_substances SET complete=TRUE WHERE ",
                        "sf_id=", sf_id, 
                        ";", sep="")
  queryOut = sqlQuery(dbCon, query=queryString)
  if (length(grep("ERROR", queryOut)) != 0) {
    err=1
    exists=FALSE
    complete=FALSE
  } else {
    err=0
    exists=TRUE
    complete=TRUE
  }
  
  return(list("err"=err, "exists"=exists, "complete"=complete, "sf_id"=sf_id))
}