#!/bin/bash

# The script creates an ocean file based on a GRIDCRO2D file. The surf zone 
# is set to 0 everywhere (=> no surf zone emis).
#
# @author: Daniel Neumann, daniel.neumann@hzg.de
# @date: 2016-03-30
# @version: 1.1
#
# Preconditions / Needed Data
#  GRIDCRO2D file $METEO_IN_NAME in $METEO_DIR
#
# Note 1: Works only for the CN06 grid because inland water bodies need to
#          be masked out.


## change into the appropriate directory
cd .


## create calculated script file for a call of 'ncap2':
echo 'defdim("LAY",1);' > addLAY_dim.nco
echo '' >> addLAY_dim.nco
echo 'XMASK[$time,$LAY,$y,$x]=MASK;' >> addLAY_dim.nco
echo 'YSURF[$time,$LAY,$y,$x]=SURF;' >> addLAY_dim.nco
echo 'ZOPEN[$time,$LAY,$y,$x]=OPEN;' >> addLAY_dim.nco


## grid name
GRID_NAME='CN06'
grid_name='cn06'


## File Names
METEO_IN_NAME=GRIDCRO2D_${grid_name}_2013001
OCEAN_OUT_NAME=OCEAN_${GRID_NAME}_sf000m.nc

## Directories
TMP_DIR='./tmp'
METEO_DIR="../../GRIDCRO"
OUT_DIR="../../OCEAN_basic"

## Copy the GRIDCRO2D file and make TSTEP to the record dimension
ncks -O --mk_rec_dmn TSTEP ${METEO_DIR}/${METEO_IN_NAME} ${TMP_DIR}/cn06.nc

## Calculate the OPEN varianle from the LUFRAC_16 and LWMASK variables and
## write everything into a tmp file.
# OPEN = LUFRAC_16 * ((-1) * (LWMASK - 1))
cdo chname,LUFRAC_16,OPEN -mul -selname,LUFRAC_16 ${TMP_DIR}/cn06.nc -mulc,-1 -subc,1 -selname,LWMASK ${TMP_DIR}/cn06.nc ${TMP_DIR}/tmp01.nc

## Remove some water grind cells from the new OPEN variable
# Otherwise, large lakes and river deltas who we open ocean regions.
cdo setcindexbox,0,1,90,55,100 -setcindexbox,0,1,68,0,100 -setcindexbox,0,90,100,1,10 -setcindexbox,0,110,115,10,24 -setcindexbox,0,125,140,84,100 ${TMP_DIR}/tmp01.nc ${TMP_DIR}/tmp02.nc

## Create SURF and MASK variables from the OPEN variables and write them into
## another tmp file.
# SURF = 0.0 * OPEN
# MASK(i,j) = IF( OPEN(i,j) == 0 )      0.0 * OPEN(i,j)
#             ELSE IF( OPEN(i,j) == 1 ) 2.0 + 0.0 * OPEN(i,j)
#             ELSE                      1.0 + 0.0 * OPEN(i,j)
cdo -O merge -chname,OPEN,SURF -mulc,0 -selname,OPEN ${TMP_DIR}/tmp02.nc -selname,OPEN ${TMP_DIR}/tmp02.nc -chname,OPEN,MASK -ifthenelse -eqc,0 -selname,OPEN ${TMP_DIR}/tmp02.nc -mulc,0 -selname,OPEN ${TMP_DIR}/tmp02.nc -ifthenelse -eqc,1 -selname,OPEN ${TMP_DIR}/tmp02.nc -addc,2 -mulc,0 -selname,OPEN ${TMP_DIR}/tmp02.nc -addc,1 -mulc,0 -selname,OPEN ${TMP_DIR}/tmp02.nc ${TMP_DIR}/tmp03.nc

## Copy the TFLAG variable from the modified GRIDCRO2D file into an extra
## tmp files and duplicate that file.
ncks -O -v TFLAG -d VAR,0,2 ${TMP_DIR}/cn06.nc ${TMP_DIR}/tmp04.nc
cp ${TMP_DIR}/tmp04.nc ${TMP_DIR}/tmp08.nc

## Copy MASK, OPEN, and SURF into new variables which have the additional 
## dimension LAY (=1). Then remove the old variables and rename some dimensions.
ncap2 -O -S addLAY_dim.nco ${TMP_DIR}/tmp03.nc ${TMP_DIR}/tmp05.nc
ncks -O -x -v MASK,OPEN,SURF ${TMP_DIR}/tmp05.nc ${TMP_DIR}/tmp06.nc
ncrename -O -d x,COL -d y,ROW -d time,TSTEP ${TMP_DIR}/tmp06.nc ${TMP_DIR}/tmp07.nc

## Copy the copied variables with the extra dimension into the file with the
## TFLAG variable and rename the MASK, SURF, and OPEN variables to their old
## names.
ncks -A -v XMASK,YSURF,ZOPEN ${TMP_DIR}/tmp07.nc ${TMP_DIR}/tmp08.nc
ncrename -O -v XMASK,MASK -v YSURF,SURF -v ZOPEN,OPEN ${TMP_DIR}/tmp08.nc ${TMP_DIR}/tmp09.nc

## Finally, add some global attributes.
ncatted -O -a 'VAR-LIST',global,m,c,'MASK            SURF            OPEN            ' -a FILEDESC,global,m,c,'OCEAN file' -a units,MASK,o,c,'none            ' -a long_name,MASK,o,c,"MASK            " -a var_desc,MASK,o,c,"2=open ocean, 1=coastline, 0=other" -a units,OPEN,o,c,'none            ' -a long_name,OPEN,o,c,"OPEN            " -a var_desc,OPEN,o,c,"open ocean area / total area" -a units,SURF,o,c,'none            ' -a long_name,SURF,o,c,"SURF            " -a var_desc,SURF,o,c,"surf zone area / total area" -a GDNAM,global,m,c,"CN06" -a UPNAM,global,o,c,"OCEAN" -a NVARS,global,o,i,3 ${TMP_DIR}/tmp09.nc OCEAN_CN06_sf000m.nc


echo -e "\n  Move Ocean file to output directory."
mv ${OCEAN_OUT_NAME} ${OUT_DIR}/${OCEAN_OUT_NAME}

echo -e "\n  Remove temporary files."
rm ${TMP_DIR}/tmp*.nc ${TMP_DIR}/cn06.nc
