#!/bin/bash

# The script creates an ocean file based on salinity data and based on a
# GRIDCRO2D file. The surf zone is set to 0 everywhere (=> no surf zone emis).
#
# @author: Daniel Neumann, daniel.neumann@hzg.de
# @date: 2016-03-31
# @version: 1.1
#
# Preconditions / Needed Data
#  Salinity data file $SALT_IN_NAME in $SALT_DIR
#   - same grid resolution as GRIDCRO2D and target grid
#   - salinity variable is denoted as SALINITY
#   - set date of the salinity file in $YEAR
#  GRIDCRO2D file $METEO_IN_NAME in $METEO_DIR
#   - set date of the GRIDCRO2D file in $DATE
#   - the GRIDCRO2D file needs to contain the ocean fraction per grid cell
#
# Applicable for which grids?
#  - theoretically for each grid for which the salinity and gridcro2d files
#     are available
#  - practically:
#     -- all regions that lay within the provided salinity data field are 
#         fine; the current salinity comes from the BSH. Thus, the CD04a, CD04b
#         and CD06 grids make no problems
#     -- all other regions need to be individually treated by manually adding
#         salinity in regions in which the BSH model makes no predictions;
#         regions that are currently implemented are the CD16, CD24, and CD64 
#         grids;
#     -- all other regions will not be correctly processed 
#
# Usage
#  - set the GRID_NAME and the grid_name variables;
#  - if necessary set the YEAR variable; the ocean files are not named according
#     to the year; you would need to do that on your own
#  - set the 'control variables' further below


## grid name
GRID_NAME='CD64'
grid_name='cd64'


## dates of source files
YEAR='2012'
case ${GRID_NAME} in
  "CD16"|"CD04a"|"CD04b"|"CD64") DATE='2011335';;
  "CD06"|"CD24"|"CD72") DATE="${YEAR}001";;
  *) DATE="${YEAR}001"  ;;
esac


## control variables:
SAL_DEP=1  # 1 for YES/TRUE and 0 for FALSE/NO
           #  (values != 1 are considered as FALSE)
DO_CORR_SALT=1  # 1 for YES/TRUE and 0 for FALSE/NO
                #  (values != 1 are considered as FALSE)
# currently not used:
DO_CORR_LAND=0  # 1 for YES/TRUE and 0 for FALSE/NO
                #  (values != 1 are considered as FALSE)


## File Names
METEO_IN_NAME=GRIDCRO2D_${grid_name}_${DATE}
SALT_IN_NAME=SAL_BSH_RAW_${GRID_NAME}_${YEAR}.nc
OCEAN_OUT_NAME=OCEAN_${GRID_NAME}_sf000m_sal.nc


## Directories
TOOL_DIR='../..'
# TOOL_DIR='/home/neumannd/TMP/ocean/tools'
TMP_DIR='./tmp'
METEO_DIR="${TOOL_DIR}/GRIDCRO"
SALT_DIR="${TOOL_DIR}/SALINITY/BSH"
OUT_DIR="${TOOL_DIR}/OCEAN_basic"


## create calculated script file for a call of 'ncap2':
echo 'defdim("LAY",1);' > addLAY_dim.nco
echo '' > addLAY_dim.nco
echo 'XMASK[$time,$LAY,$y,$x]=MASK;' > addLAY_dim.nco
echo 'YSURF[$time,$LAY,$y,$x]=SURF;' > addLAY_dim.nco
echo 'ZOPEN[$time,$LAY,$y,$x]=OPEN;' > addLAY_dim.nco


## START
echo "Start processing"

# prepare GRIDCRO2D file and extract TFLAG variable
echo -e "\n  Copying GRIDCRO2D and setting TSTEP to record dimension (UINLIMITED)"
ncks -O --mk_rec_dmn TSTEP ${METEO_DIR}/${METEO_IN_NAME} ${TMP_DIR}/tmp_gridcro2d_${grid_name}.nc

echo -e "\n  Extract TFLAG variable and correction VAR dimension [B]."
ncks -O -v TFLAG -d VAR,0,2 ${TMP_DIR}/tmp_gridcro2d_${grid_name}.nc ${TMP_DIR}/tmp_TFLAG.nc


# The salinity field does not necessarily extend over the whole domain. 
# Therefore, one needs to set salinities in some parts of the domain manually.
# Later we multiply the OPEN variable with the salinity. Thus, the salinity can
# be set to values >0 on land grid cells because OPEN is 0 there.
echo -e "\n  Correct salinity if necessary."
if [ ${DO_CORR_SALT} -eq 1 ]
then
  case ${GRID_NAME} in
    "CD24")
        cdo -O -s setcindexbox,0,67,67,101,101 -setcindexbox,35,1,27,73,106 -setcindexbox,35,28,48,77,106 -setcindexbox,35,40,48,76,76 -setcindexbox,35,1,24,1,23 -setcindexbox,35,1,24,34,106 -setcindexbox,35,1,17,23,35 -setcindexbox,35,24,27,70,71 -setcindexbox,35,49,58,83,106 -setcindexbox,35,59,65,92,106 -setcindexbox,35,65,68,99,106 -setcindexbox,10,107,112,102,106 -setcindexbox,30,66,74,2,8 -setcindexbox,37,53,76,1,8 ${SALT_DIR}/${SALT_IN_NAME} ${TMP_DIR}/tmp_sal.nc
        # remove lakes:
        # setcindexbox,0,67,67,101,101
        # 
        # add Atlantic Ocean:
        # -setcindexbox,35,1,27,73,106 -setcindexbox,35,28,48,77,106 -setcindexbox,35,40,48,76,76 -setcindexbox,35,1,24,1,23 -setcindexbox,35,1,24,34,106 -setcindexbox,35,1,17,23,35 -setcindexbox,35,24,27,70,71 -setcindexbox,35,49,58,83,106 -setcindexbox,35,59,65,92,106 -setcindexbox,35,65,68,99,106
        #
        # northern Russian coast:
        # -setcindexbox,10,107,112,102,106
        #
        # add Mediterranean Sea:
        # -setcindexbox,30,66,74,2,8 -setcindexbox,37,53,76,1,8
        ;;
    "CD16")
        cdo -O -s setcindexbox,0,64,102,79,99 -setcindexbox,0,87,103,103,128 -setcindexbox,0,79,84,105,112 -setcindexbox,0,97,100,139,145 -setcindexbox,0,93,100,133,140 -setcindexbox,0,59,62,32,35 -setcindexbox,35,1,27,1,70 -setcindexbox,35,1,26,71,145 -setcindexbox,35,27,31,76,87 -setcindexbox,35,27,75,88,145 -setcindexbox,35,76,96,104,145 -setcindexbox,10,140,161,126,145 ${SALT_DIR}/${SALT_IN_NAME} ${TMP_DIR}/tmp_sal.nc
        # remove lakes:
        # setcindexbox,0,64,102,79,99 -setcindexbox,0,87,103,103,128 -setcindexbox,0,79,84,105,112 -setcindexbox,0,97,100,139,145 -setcindexbox,0,93,100,133,140 -setcindexbox,0,59,62,32,35
        # 
        # add Atlantic Ocean
        # -setcindexbox,35,1,27,1,70 -setcindexbox,35,1,26,71,145 -setcindexbox,35,27,31,76,87 -setcindexbox,35,27,75,88,145 -setcindexbox,35,76,96,104,145
        #
        # northern Russian coast
        # -setcindexbox,10,140,161,126,145
        ;;
    "CD64")
        # echo 'todo'
        # TODO: Black sea = 10?
        # cdo -O -s copy ${TOOL_DIR}/BSH/SAL_BSH_RAW_${GRID_NAME}_${YEAR}.nc ${TMP_DIR}/tmp_sal.nc
        cdo -O -s setcindexbox,0,42,60,54,58 -setcindexbox,0,33,34,47,49 -setcindexbox,0,55,55,36,36 -setcindexbox,0,61,61,19,19 -setcindexbox,30,34,37,24,27 -setcindexbox,36,9,18,1,18 -setcindexbox,10,52,62,20,34 -setcindexbox,37,18,62,1,26 -setcindexbox,35,1,17,1,78 -setcindexbox,35,34,40,64,69 -setcindexbox,35,18,18,49,51 -setcindexbox,35,18,34,52,69 -setcindexbox,10,43,56,59,66 -setcindexbox,25,43,56,67,68 -setcindexbox,30,43,56,69,70 -setcindexbox,35,1,62,70,78 ${TOOL_DIR}/BSH/SAL_BSH_RAW_${GRID_NAME}_${YEAR}.nc ${TMP_DIR}/tmp_sal.nc
        # remove lakes:
        # setcindexbox,0,42,60,54,58 -setcindexbox,0,33,34,47,49 -setcindexbox,0,55,55,36,36 -setcindexbox,0,61,61,19,19
        #
        # add Mediterranean Sea and Black Sea:
        # -setcindexbox,30,34,37,24,27 -setcindexbox,36,9,18,1,18 -setcindexbox,10,52,62,20,34 -setcindexbox,37,18,62,1,26
        #
        # add Atlantic Ocean
        # -setcindexbox,35,1,17,1,78 -setcindexbox,35,34,40,64,69 -setcindexbox,35,18,18,49,51 -setcindexbox,35,18,34,52,69
        #
        # add northern Atlantic Ocean and northern Russian coast
        # -setcindexbox,10,43,56,59,66 -setcindexbox,25,43,56,67,68 -setcindexbox,30,43,56,69,70 -setcindexbox,35,1,62,70,78
        ;;
    *)
        cdo -O -s copy ${TOOL_DIR}/BSH/SAL_BSH_RAW_${GRID_NAME}_${YEAR}.nc ${TMP_DIR}/tmp_sal.nc
        ;;
  esac
else
  cdo -O -s ${TOOL_DIR}/BSH/SAL_BSH_RAW_${GRID_NAME}_${YEAR}.nc ${TMP_DIR}/tmp_sal.nc
fi


# create OPEN variable and scale it optionally by the salinity
echo -e "\n  Make first version of OPEN variables by merging LUFRAC_16 and LWMASK"
echo -e "    from the GRIDCRO2D. Add annual average SALINITY from the BSH data [A]."
cdo -O -s merge -chname,LUFRAC_16,OPEN -mul -selname,LUFRAC_16 ${TMP_DIR}/tmp_gridcro2d_${grid_name}.nc -mulc,-1 -subc,1 -selname,LWMASK ${TMP_DIR}/tmp_gridcro2d_${grid_name}.nc -setmisstoc,0 -selname,SALINITY ${TMP_DIR}/tmp_sal.nc ${TMP_DIR}/tmp_OPEN01.nc

echo -e "\n  Set gridcells of OPEN to 0 where SALINITY is 0. These are lakes [A]."
cdo -O -s merge -setmisstoc,0 -ifthen -nec,0 -selname,SALINITY ${TMP_DIR}/tmp_OPEN01.nc -selname,OPEN ${TMP_DIR}/tmp_OPEN01.nc -selname,SALINITY ${TMP_DIR}/tmp_OPEN01.nc ${TMP_DIR}/tmp_OPEN02.nc

echo -e "\n  Scale OPEN with SALINITY if requested by the user [A]."
if [ ${SAL_DEP} -eq 1 ]
then
    cdo -O -s chname,NEW_OPEN,OPEN -expr,'NEW_OPEN=OPEN*SALINITY/35;OLD_OPEN=OPEN' ${TMP_DIR}/tmp_OPEN02.nc ${TMP_DIR}/tmp_OPEN03.nc
else
    cdo -O -s chname,NEW_OPEN,OPEN -expr,'NEW_OPEN=OPEN;OLD_OPEN=OPEN' ${TMP_DIR}/tmp_OPEN02.nc ${TMP_DIR}/tmp_OPEN03.nc
fi


# Create SURF and MASK variables from the OPEN variable. We need to keep an 
# non-salinity-scaled open variable denoted as OLD_OPEN in order to create a
# correct MASK variable. If we would take the salinity-scaled OPEN variable
# we would get a lot of 1 (ones) in the MASK variable in the Baltic Sea and 
# other regions.
echo -e "\n  Create SURF and MASK variables [A]."
cdo -O -s merge -chname,OLD_OPEN,SURF -mulc,0 -selname,OLD_OPEN ${TMP_DIR}/tmp_OPEN03.nc -selname,OPEN ${TMP_DIR}/tmp_OPEN03.nc -chname,OLD_OPEN,MASK -ifthenelse -eqc,0 -selname,OLD_OPEN ${TMP_DIR}/tmp_OPEN03.nc -mulc,0 -selname,OLD_OPEN ${TMP_DIR}/tmp_OPEN03.nc -ifthenelse -eqc,1 -selname,OLD_OPEN ${TMP_DIR}/tmp_OPEN03.nc -addc,2 -mulc,0 -selname,OPEN ${TMP_DIR}/tmp_OPEN03.nc -addc,1 -mulc,0 -selname,OLD_OPEN ${TMP_DIR}/tmp_OPEN03.nc ${TMP_DIR}/tmp_MASK_SURF_OPEN01.nc

echo -e "\n  Add LAY dimensions which was removed by cdo and close the MASK, OPEN and"
echo -e "    SURF variables into new variables for reordering by ncks [A]."
ncap2 -O -S addLAY_dim.nco ${TMP_DIR}/tmp_MASK_SURF_OPEN01.nc ${TMP_DIR}/tmp_MASK_SURF_OPEN02.nc

echo -e "\n  Remove original MASK, OPEN and SURF [A]."
ncks -O -x -v MASK,OPEN,SURF ${TMP_DIR}/tmp_MASK_SURF_OPEN02.nc ${TMP_DIR}/tmp_MASK_SURF_OPEN03.nc

echo -e "\n  Rename some dimensions [A]."
ncrename -O -d x,COL -d y,ROW -d time,TSTEP ${TMP_DIR}/tmp_MASK_SURF_OPEN03.nc ${TMP_DIR}/tmp_MASK_SURF_OPEN04.nc


# Merge TFLAG together with MASK, OPEN and SURF.
# The variables need to be in a proper order. At least TFLAG needs to be the
# first variable and MASK, OPEN and SURF need to be in the same order as
# written down in the global netCDF-variable VAR-LIST. To achieve this, we
# rename MASK, OPEN und SURF now. 'ncks' without '-a' automatically sorts
# the variables alphabetically. After that we rename MASK, OPEN and SURF back 
# to their old names.
echo -e "\n  Copy one temp file into another (for debugging) [B]."
cp ${TMP_DIR}/tmp_TFLAG.nc ${TMP_DIR}/tmp_OCEAN01.nc

echo -e "\n  Add cloned MASK, SURF and OPEN variables to the TFLAG-containing file."
echo -e "    The variables are automatically reordered because not -a option is set"
echo -e "    in the ncks call [A+B=C]."
ncks -A -v XMASK,YSURF,ZOPEN ${TMP_DIR}/tmp_MASK_SURF_OPEN04.nc ${TMP_DIR}/tmp_OCEAN01.nc

echo -e "\n  Rename cloned variables to their original names [C]."
ncrename -O -v XMASK,MASK -v YSURF,SURF -v ZOPEN,OPEN ${TMP_DIR}/tmp_OCEAN01.nc ${TMP_DIR}/tmp_OCEAN02.nc


# Do final modifications to attributes.
echo -e "\n  Modify attributes in order to get an IOAPI and CMAQ conformal file and"
echo -e "    write the output into the final file [C]."
ncatted -O -a 'VAR-LIST',global,m,c,'MASK            SURF            OPEN            ' -a FILEDESC,global,m,c,'OCEAN file' -a units,MASK,o,c,'none            ' -a long_name,MASK,o,c,"MASK            " -a var_desc,MASK,o,c,"2=open ocean, 1=coastline, 0=other" -a units,OPEN,o,c,'none            ' -a long_name,OPEN,o,c,"OPEN            " -a var_desc,OPEN,o,c,"open ocean area / total area" -a units,SURF,o,c,'none            ' -a long_name,SURF,o,c,"SURF            " -a var_desc,SURF,o,c,"surf zone area / total area" -a GDNAM,global,m,c,"CN06" -a UPNAM,global,o,c,"OCEAN" -a NVARS,global,o,i,3 ${TMP_DIR}/tmp_OCEAN02.nc ${OCEAN_OUT_NAME}

echo -e "\n  Move Ocean file to output directory."
mv ${OCEAN_OUT_NAME} ${OUT_DIR}/${OCEAN_OUT_NAME}

echo -e "\n  Remove temporary files."
rm ${TMP_DIR}/tmp*.nc

echo "Finished"
