#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'      \email{daniel.neumann@hzg.de}
#' @description R script to create a new oceanFile from a Excel file
#' @keywords R, 
#' @note 
#'  
#' Date: 2016/03/31
#' 
#' Changes in this Version:
#'  - naming of the output file automatized
#'  - changed directories
#'  - renamed the scripts from create_OCEAN_files_from_raw_data_anna_2016-03-30.R
#'     to make_ocean_from_coastlineData_2016-03-31.R

#' EXCEL FILE
#' Needed columns (exact titles!!!) in the excel file:
#'  Id, Shape_Area_ocean, Shape_Area_land, Shape_Area_surfzone, Shape_Area
#' 
#' Id: identifies the grid cell;
#'        id == 1 is in the top left corner of the map
#'        id == nCols is in the top right corner of the map
#'        id == (nRows-1)*nCols+1 is in the bottom left corner of the map
#'        id == nRows * nCols is in the bottom right corner of the map
#' Shape_Area_ocean:    area of the open ocean surface in the grid cell
#' Shape_Area_land:     area of the surf zone surface in the grid cell
#' Shape_Area_surfzone: area of the land surface in the grid cell
#' Shape_Area:          complete area of the grid cell
#' 
#' NOTE 1: Some Id's exists two or more times. For each closed surf zone
#'          ( == for each island without connection to the mainland) a
#'          new row with the same 'id' is added to the excel file. Up till
#'          now, the 'ocean' and 'land' were equal for duplicate 'id's.
#'          To deal with this, we take 'ocean' and 'land' from that row in
#'          which each 'id' appeared the first time and we add up all 
#'          'surfzone's of each duplicate 'id'.
#'
#' NOTE 2: Shape_Area is the expected area of the grid cell (e.g. 24*24 km2 
#'        in the CD24 grid). However, the sum of the three columns 'ocean'
#'        'land' and 'surfzone' is not always 'Shape_Area' (even when NOTE1
#'        is considered). Therefore, 'ocean', 'land' and 'surfzone' is 
#'        normalised by 'ocean + land + surfzone'.
#'        
#'  USAGE: Modify the parameters in section (3) and run the script.
#'  
#'  NEEDED DATA:
#'     - Excel file (see some lines earlier)
#'     - saliniy data (netCDF file; same grid format like the target Ocean File)
#'        
#' 
#' OceanFILE (netCDF file)
#'  This file has the structure as needed for CMAQ.


#' +~~~~~~~~~~~~~~~~~~~~~
#  | Content -----
#' +~~~~~~~~~~~~~~~~~~~~~
#' 1) load libraries and source R scripts
#' 2) define functions
#' 3) set parameters
#' 4) get data (if 'readData' set to TRUE)
#' 5) write out data (if 'writeFile' set to TRUE)


# +~~~~~~~~~~~~~~~~~~~~~
# | 1) load libraries and source R scripts ----
# +~~~~~~~~~~~~~~~~~~~~~
library('ncdf4')
library('cmaqtools')
library('gdata')


# +~~~~~~~~~~~~~~~~~~~~~
# | 2) define function ----
# +~~~~~~~~~~~~~~~~~~~~~
# Needed to simplify the code below
catchNA <- function(inVal) {
  inVal[is.na(inVal)] <- 0.0
  return(inVal)
}


# +~~~~~~~~~~~~~~~~~~~~~
# | 3) set parameters ----
# +~~~~~~~~~~~~~~~~~~~~~
print('SETTING PARAMETERS')
writeFile = TRUE
readData = TRUE
dummyData = FALSE # Does only have an effect if 'readData' is FALSE; default/recommended: FALSE
bounded = TRUE # if true: 'uBound' is set as upper bound for the surf zone; default/recommended: TRUE
considerSalinity = TRUE # default/recommended option: TRUE
manualModification = TRUE # default/recommended option: TRUE

inDir = "../../COASTLINES/GIS_EXPORT"
outDir = "../../OCEAN_gis"
saltDir = "../../SALINITY/ECOSMO"
metDir = "../../GRIDCRO"

# for (gridType in c('AQ24')) {
for (gridType in c('CD06', 'CD24', 'CD72', 'AQ24')) {
  for (surfWidth in c(0, 25, 50, 100)) {

# Choose type of the grid
# gridType = "CD72"
# gridType = "CD24"
# gridType = "CD06"
# gridType = 'AQ24'

# width of the surf zone in m
# (I assume that the surf zone width in the Excel file is 50 m.)
# surfWidth = 50

# upper bound for the proportion of surf zone in the grid cell
uBound = sqrt(5) * surfWidth*10^(-3) / as.numeric(substr(gridType, 3, 4))

# get salinity data which is modified by hand
source('./salinityData_hand_2014-10-17.R')

# create file names
# fileOut = paste("OCEAN_", gridType, ".nc", sep="")
fileOut = paste(paste("OCEAN_", gridType, 
                      paste("_sf", formatC(surfWidth, format = 'd', width = 3, flag = 0), "m", sep = ''),
                      "_GIS", 
                      if(bounded) "_ubound" else "", 
                      if(considerSalinity) "_sal" else "",
                      sep = ""),
                "nc", sep=".")
saltFile = paste("SALINI_ANNUAL_", gridType, ".NCF", sep = "")
# saltFile = ""

print('STARTING')
print(paste('    GRID_NAME:        ', gridType, sep = ' '))
print(paste('    surf width:       ', surfWidth, sep = ' '))
print(paste('    consider salinity:', considerSalinity, sep = ' '))
print(paste('    bounded surf zone:', bounded, sep = ' '))
# +~~~~~~~~~~~~~~~~~~~~~
# | 4) get data ----
# +~~~~~~~~~~~~~~~~~~~~~
if ( readData || dummyData ) {
  # choose ...
  myGrid <- griddesc(gridType)  
  
  # Initialise variables
  if ( gridType == 'AQ24' ) {
    nCols = 112
    nRows = 106
  } else {
    nCols = myGrid$ncol
    nRows = myGrid$nrow
  }
  
  cellArea = myGrid$xcell * myGrid$ycell * 10^6
  
  # initialise MASK array
  valMASK = rep(as.integer(0), nCols * nRows)
  dim(valMASK) = c(nCols, nRows)
  
}

if ( readData ) {
  # Open files
  if ( gridType == 'AQ24' ) {
    valsIn <- read.xls(paste(inDir, "CD24_ocean_land_surfzone_TableToExcel.xls", sep="/"))
  } else {
    #ncIn <- nc_open(paste(inDir, paste(gridType, "_ocean_land_surfzone_Tab.nc", sep=""), sep="/"), write=FALSE)
    valsIn <- read.xls(paste(inDir, paste(gridType, "_ocean_land_surfzone_TableToExcel.xls", sep=""), sep="/"))
  }
  #valsIn <- TMP_XLS
  orderIn <- order(valsIn$Id)
  
  # number of lines in the excel file
  #nRecs = ncIn$dim$RecordID$len
  nRecs = length(valsIn$Id)
  
  # look for duplicate 'id's
  duplicates1 = duplicated(valsIn$Id)
  # number of duplicate 'id's
  nDup = length(valsIn$Id[duplicates1])
  
  #' create an alternative order in which the duplicated 'id's are
  #' dropped out  
  if ( nDup > 0 ) {
    orderTmp <- order(valsIn$Id[!duplicates1])
    orderIn <- (1:nRecs)[!duplicates1][orderTmp]
  } 
  
  #' Proceed only if the is at least one line in the excel file for
  #' each grid cell on the target grid.
  if ( nRecs - nDup == nCols * nRows ) {
    
    #' Copy values from 'Shape_Area_surfzone' to valSURF - but not the
    #' duplicates (if an 'id' exists twice, a value is copied at the first
    #' 'id's appeareance but not at the second). 
    valSURF = as.numeric(valsIn$Shape_Area_surfzone[orderIn])
    
    #' Here we add up those values which's id's appeared a second, third
    #' or more often times.
    if ( nDup > 0 ) {
#       valSURF = rep(0.0, nCols*nRows)
#       duplicates2 = (1:nRecs)[duplicated2(valsIn$Id)]
      duplicates2 = (1:nRecs)[duplicates1]
      for ( iS in duplicates2) {
        # iS = index source array
        # iT = index target array
        iT = valsIn$Id[iS]
        valSURF[iT] = valSURF[iT] + catchNA(as.numeric(valsIn$Shape_Area_surfzone[iS]))
      }
      
      nRecs = nCols*nRows
    } 
  
    #' 'Shape_Area_ocean', 'Shape_Area_land', 'Shape_Area' and 'Id' are 
    #' copied in the same way independent of the number of duplicates.
    #' Just 'orderIn' was adjusted beforehand.
    valOPEN = as.numeric(valsIn$Shape_Area_ocean[orderIn])
    valLAND = as.numeric(valsIn$Shape_Area_land[orderIn])
    # valGRID = as.numeric(valsIn$Shape_Area[orderIn])
    valId = valsIn$Id[orderIn]
    

    if (gridType == "CD72" || gridType == "AQ24") valOPEN[as.logical(is.na(valOPEN)*is.na(valSURF)*is.na(valLAND))] = 7.2^2*10^8
    valSURF[is.na(valSURF)] = 0.0
    valOPEN[is.na(valOPEN)] = 0.0
    valLAND[is.na(valLAND)] = 0.0
    # valGRID[is.na(valGRID)] = 0.0

    dim(valSURF) = dim(valMASK)
    dim(valOPEN) = dim(valMASK)
    dim(valLAND) = dim(valMASK)
    # dim(valGRID) = dim(valMASK)
    dim(valId) = dim(valMASK)
    
    valSURF = valSURF[, nRows:1]
    valOPEN = valOPEN[, nRows:1]
    valLAND = valLAND[, nRows:1]
    # valGRID = valGRID[, nRows:1]
    valId = valId[, nRows:1]
    
    valAREA = valSURF + valOPEN + valLAND
    
    # Fill the matrix MASK
    for ( iRec in 1:nRecs) {
      if ( valSURF[iRec] > 0.0 ) {
        valMASK[iRec] = as.integer(1)
      } else if ( valOPEN[iRec] > 0.0 ) {
        valMASK[iRec] = as.integer(2)
      }
    }

    # if no surf zone should exist ...
    if ( surfWidth == 0) {
      valOPEN = valOPEN + valSURF
      valSURF = matrix(0.0, nrow = nCols, ncol = nRows)
    }
    
    if ( surfWidth != 50 || surfWidth != 0 ) {
      for ( iRec in 1:nRecs ) {
        if( valMASK[iRec] == 1 ) {
          if( valOPEN[iRec] >= valSURF[iRec] * (surfWidth-50)/50 || surfWidth < 50 ){
            valSURF[iRec] = surfWidth/50 * valSURF[iRec]
            valOPEN[iRec] = valOPEN[iRec] - valSURF[iRec] * (surfWidth-50)/50 
          } else {
            valSURF[iRec] = valSURF[iRec] + valOPEN[iRec]
            valOPEN[iRec] = 0.0
          }
        }
      }
    } 

    valSURF = valSURF / valAREA
    valOPEN = valOPEN / valAREA
    valLAND = valLAND / valAREA
    
    if ( gridType == 'AQ24') {
      COLoffset = 22
      ROWoffset = 61+1
      
      nColsCD24 = nCols
      nRowsCD24 = nRows
      
      nColsAQ24 = myGrid$ncol
      nRowsAQ24 = myGrid$nrow
      
      # set nCols, nRows and nRecs to AQ24 values
      nCols = nColsAQ24
      nRows = nRowsAQ24
      nRecs = nCols * nRows
      
      # read AQ24 MASK and OPEN arrays from met file
      ncMet <- nc_open(paste(metDir, 'GRIDCRO2D_aq24_2008275', sep = "/"))
      metMASK <- abs(ncvar_get(ncMet, 'LWMASK') - 1) * 2
      metOPEN <- ncvar_get(ncMet, 'LUFRAC_16')
      
      # Remove lakes
      metMASK[68:110,70:80] = 0
      metMASK[109:157,86:110] = 0
      metMASK[116:165,120:134] = 0
      metMASK[128:165,130:161] = 0
      metMASK[111:160,142:160] = 0
      metMASK[95:100,170:177] = 0
      metMASK[105:127,174:182] = 0
      metMASK[115:120,45:52] = 0
      metMASK[150:165,40:52] = 0
      metMASK[170:171,4:11] = 0
      
      # Set OPEN to 0 where MASK is zero (lakes etc.)
      metOPEN[metMASK == 0] = 0
      
      # set SURF zone values in MASK
      metMASK[abs(abs(metOPEN-0.5)-0.5) != 0] = 1
      
      # set SURF to coast line (surfWidth [m] 10^(-3) [km/m] X gridSize [km] / gridSize^2 [m^2])
      metSURF = array(surfWidth * 10^(-3) / as.numeric(substr(gridType, 3, 4)), dim = dim(metMASK))*abs(abs(metMASK-1)-1)
      for (i1 in 1:dim(metSURF)[1]) {
        for (i2 in 1:dim(metSURF)[2]) {
          metSURF[i1, i2] = min(metSURF[i1, i2],(metOPEN - abs(abs(metMASK-0.5)-0.5))[i1,i2])
        }
      }
      
      # subtract surf zone fraction from open fraction
      metOPEN = metOPEN - metSURF
      
      # replace CD24 section of the AQ24 data by CD24 data
      metMASK[(COLoffset+1):(COLoffset+nColsCD24), (ROWoffset+1):(ROWoffset+nRowsCD24)] = valMASK
      metOPEN[(COLoffset+1):(COLoffset+nColsCD24), (ROWoffset+1):(ROWoffset+nRowsCD24)] = valOPEN
      metSURF[(COLoffset+1):(COLoffset+nColsCD24), (ROWoffset+1):(ROWoffset+nRowsCD24)] = valSURF
      
      # replace old val*arrays
      valMASK = metMASK
      valOPEN = metOPEN
      valSURF = metSURF
      valArea = array(1.0, dim = dim(valMASK))
    }
  
    if ( surfWidth != 0 && bounded ) {
      for ( iRec in 1:nRecs ) {
        if( valSURF[iRec] > uBound ) {
          valOPEN[iRec] = valOPEN[iRec] + valSURF[iRec] - uBound
          valSURF[iRec] = uBound
        }
      }
    }

    if ( considerSalinity ) {
      ncId <- nc_open( paste(saltDir, saltFile, sep = "/") )
      valSALT = ncvar_get(ncId, 'SALINI')
      saveSALT = valSALT
      
      if ( manualModification) {
        if ( gridType == "CD06") {
          valSALT[cbind(sal18xCD06, sal18yCD06)] = 18
          valSALT[cbind(sal20xCD06, sal20yCD06)] = 20
          valSALT[cbind(sal22xCD06, sal22yCD06)] = 22
          valSALT[cbind(sal24xCD06, sal24yCD06)] = 24
          valSALT[cbind(sal26xCD06, sal26yCD06)] = 26
          valSALT[cbind(sal28xCD06, sal28yCD06)] = 28
          valSALT[cbind(sal30xCD06, sal30yCD06)] = 30
        }
        if ( gridType == "CD24") {
          valSALT[cbind(sal16xCD24, sal16yCD24)] = 16
          valSALT[cbind(sal18xCD24, sal18yCD24)] = 18
          valSALT[cbind(sal20xCD24, sal20yCD24)] = 20
          #           valSALT[cbind(sal22xCD24, sal22yCD24)] = 22
          valSALT[cbind(sal24xCD24, sal24yCD24)] = 24
          valSALT[cbind(sal26xCD24, sal26yCD24)] = 26
          valSALT[cbind(sal28xCD24, sal28yCD24)] = 28
          valSALT[cbind(sal30xCD24, sal30yCD24)] = 30
        }
        if ( gridType == "CD72" ) {
          
          valSALT[cbind(sal00xCD72, sal00yCD72)] = 0.035
          valSALT[cbind(sal30xCD72, sal30yCD72)] = 30
          valSALT[cbind(sal32xCD72, sal32yCD72)] = 32
          valSALT[cbind(sal33xCD72, sal33yCD72)] = 33
          valSALT[cbind(sal34xCD72, sal34yCD72)] = 34
          valSALT[cbind(sal35xCD72, sal35yCD72)] = 35
          valSALT[cbind(sal36xCD72, sal36yCD72)] = 36
          valSALT[cbind(sal37xCD72, sal37yCD72)] = 37
          
        }
        if ( gridType == "AQ24" ) {
          
          # valSALT[cbind(sal00xAQ24, sal00yAQ24)] = 0.035
          valSALT[cbind(sal16xAQ24, sal16yAQ24)] = 16
          valSALT[cbind(sal18xAQ24, sal18yAQ24)] = 18
          valSALT[cbind(sal20xAQ24, sal20yAQ24)] = 20
          valSALT[cbind(sal24xAQ24, sal24yAQ24)] = 24
          valSALT[cbind(sal26xAQ24, sal26yAQ24)] = 26
          valSALT[cbind(sal28xAQ24, sal28yAQ24)] = 28
          valSALT[cbind(sal30xAQ24, sal30yAQ24)] = 30
          valSALT[cbind(sal32xAQ24, sal32yAQ24)] = 32
          valSALT[cbind(sal33xAQ24, sal33yAQ24)] = 33
          valSALT[cbind(sal34xAQ24, sal34yAQ24)] = 34
          valSALT[cbind(sal35xAQ24, sal35yAQ24)] = 35
          valSALT[cbind(sal36xAQ24, sal36yAQ24)] = 36
          valSALT[cbind(sal37xAQ24, sal37yAQ24)] = 37
          
        }
      } else {
        valSALT = matrix(35, dim(valMASK))
      }
      
      for ( i1 in 1:5 ) {
        if ( i1 == 1 || i1 == 2 || i1 == 5) lRows = 1:nRows else lRows = nRows:1
        if ( i1 == 1 || i1 == 4 || i1 == 5 ) lCols = 1:nCols else lCols = nCols:1
        for ( iRow in lRows ) {
          dxN = 1
          dxS = 1
          if ( iRow == 1 ) dxS = 0
          if ( iRow == nRows ) dxN = 0
          
          for ( iCol in lCols ) {
            dyW = 1
            dyE = 1
            if ( iCol == 1 ) dyW = 0
            if ( iCol == nCols) dyE = 0
            
            if ( valMASK[iCol, iRow] > 0 ) {
              if ( valSALT[iCol, iRow] == 0 ) {
                tmpSALT = valSALT[(iCol-dyW):(iCol+dyE),(iRow-dxS):(iRow+dxN)]
                mySALT = if(all(tmpSALT==0)) 0 else mean(tmpSALT[tmpSALT!=0])
                
                if ( mySALT != 0 ) valSALT[iCol, iRow] = mySALT
                if ( i1 == 5 && mySALT == 0) valSALT[iCol, iRow] = 35
              }
            }
          }
        }
      }
    }
    
    if ( considerSalinity ) {
      valSURF = valSURF * valSALT / 35
      valOPEN = valOPEN * valSALT / 35
    }

  } else {
  #   cell_ids = ncvar_get(ncIn, varid="gridcode")
#     cell_ids = valsIn$Id
#     for( i1 in 1:ncOpen$dim$RecordID$len ) {
#       cell_id = cell_ids[i1]
#       iCol = mod(cell_id - 1, nCols) + 1
#       iRow = (cell_id - iCol) / nCols + 1
#       # TODO
#     }
    # TODO
    stop('BAD INPUT EXCEL/CSV FILE!')
  }
}

# if (dim(valOPEN) == c(112,106)) {
#   tmpOPEN = valOPEN
#   tmpSURF= valSURF
#   tmpMASK = valMASK
# }
# 
# oldOPEN = metOPEN
# oldSURF = metSURF
# oldMASK = metMASK
# 
# valOPEN = metOPEN
# valSURF = metSURF
# valMASK = metMASK
# 
# valMASK[(COLoffset+1):(COLoffset+112), (ROWoffset+1):(ROWoffset+106)] = tmpMASK
# valOPEN[(COLoffset+1):(COLoffset+112), (ROWoffset+1):(ROWoffset+106)] = tmpOPEN
# valSURF[(COLoffset+1):(COLoffset+112), (ROWoffset+1):(ROWoffset+106)] = tmpSURF
# 
# nCols = 171
# nRows = 198
# 
# gridType = 'AQ24'
# fileOut = paste("OCEAN_", gridType, ".nc", sep="")

# +~~~~~~~~~~~~~~~~~~~~~
# | 5) create and write output file ----
# +~~~~~~~~~~~~~~~~~~~~~
if ( writeFile ) {
  # CREATE DIMENSIONS AND VARIABLES
  dimTSTEP <- ncdim_def('TSTEP', 
                       units = "",
                       vals = as.integer(1:1),
                       unlim = TRUE,
                       create_dimvar = FALSE
  )
  dimVAR <- ncdim_def('VAR', 
                       units = "",
                       vals = as.integer(1:3),
                       create_dimvar = FALSE
  )
  dimDT <- ncdim_def('DATE-TIME', 
                       units = "",
                       vals = as.integer(1:2),
                       create_dimvar = FALSE
  )
  dimLAY <- ncdim_def('LAY', 
                       units = "",
                       vals = as.integer(c(1)), 
                       create_dimvar = FALSE
  )
  dimROW <- ncdim_def('ROW', 
                       units = "",
                       vals = as.integer(1:nRows), 
                       create_dimvar = FALSE
  )
  dimCOL <- ncdim_def('COL', 
                       units = "",
                       vals = as.integer(1:nCols), 
                       create_dimvar = FALSE
  )

  varTFLAG <- ncvar_def( 'TFLAG', '<YYYYDDD,HHMMSS>', list(dimDT, dimVAR, dimTSTEP), longname = 'TFLAG           ', prec = 'integer' )
  varMASK <- ncvar_def( 'MASK', 'none', list(dimCOL, dimROW, dimLAY, dimTSTEP), longname = 'MASK            ', prec = 'float' )
  varSURF <- ncvar_def( 'SURF', 'none', list(dimCOL, dimROW, dimLAY, dimTSTEP), longname = 'SURF            ', prec = 'float' )
  varOPEN <- ncvar_def( 'OPEN', 'none', list(dimCOL, dimROW, dimLAY, dimTSTEP), longname = 'OPEN            ', prec = 'float' )

  
  # CREATE FILE
  ncNew <- nc_create( paste(outDir, fileOut, sep="/"), list(varTFLAG, varMASK, varSURF, varOPEN))

  
  # ADD ATTRIBUTES TO VARIABLES
  ncatt_put(ncNew, varid = 'TFLAG', attname='var_desc', 
            attval = '', 
            prec = 'text')
  ncatt_put(ncNew, varid = 'MASK', attname='var_desc', 
            attval = '2=open ocean, 1=coastline, 0=other', 
            prec = 'text')
  ncatt_put(ncNew, varid = 'SURF', attname='var_desc', 
            attval = 'surf zone area / total area', 
            prec = 'text')
  ncatt_put(ncNew, varid = 'OPEN', attname='var_desc', 
            attval = 'open ocean area / total area', 
            prec = 'text')


  # Add global attributes
  ncatt_put(ncNew, varid=0, attname='FTYPE', attval=as.integer(1), prec='int')
  
  ncatt_put(ncNew, varid=0, attname='CDATE', 
            attval=as.integer(format(Sys.time(), format="%Y")) * 10^3 + 1 + julian(Sys.Date(), origin = as.Date(paste(format(Sys.time(), format="%Y"), "-01-01", sep=""))), 
            prec='int')
  ncatt_put(ncNew, varid=0, attname='CTIME', 
            attval=as.integer(format(Sys.time(), format="%H%M%S")), 
            prec='int')
  ncatt_put(ncNew, varid=0, attname='WDATE', 
            attval=as.integer(format(Sys.time(), format="%Y")) * 10^3 + 1 + julian(Sys.Date(), origin = as.Date(paste(format(Sys.time(), format="%Y"), "-01-01", sep=""))), 
            prec='int')
  ncatt_put(ncNew, varid=0, attname='WTIME', 
            attval=as.integer(format(Sys.time(), format="%H%M%S")), 
            prec='int')
    
  ncatt_put(ncNew, varid=0, attname='SDATE', attval=as.integer(0), prec='int')
  ncatt_put(ncNew, varid=0, attname='STIME', attval=as.integer(0), prec='int')
  ncatt_put(ncNew, varid=0, attname='TSTEP', attval=as.integer(0), prec='int')
  
  ncatt_put(ncNew, varid=0, attname='NTHIK', attval=as.integer(1), prec='int')
  ncatt_put(ncNew, varid=0, attname='NCOLS', attval=as.integer(nCols), prec='int')
  ncatt_put(ncNew, varid=0, attname='NROWS', attval=as.integer(nRows), prec='int')
  ncatt_put(ncNew, varid=0, attname='NLAYS', attval=as.integer(1), prec='int')
  
  ncatt_put(ncNew, varid=0, attname='NVARS', attval=as.integer(3), prec='int')
  
  ncatt_put(ncNew, varid=0, attname='GDTYP', attval=as.integer(2), prec='int')
  
  ncatt_put(ncNew, varid=0, attname='P_ALP', attval = myGrid$sp[1], prec='double')
  ncatt_put(ncNew, varid=0, attname='P_BET', attval = myGrid$sp[2], prec='double')
  ncatt_put(ncNew, varid=0, attname='P_GAM', attval = myGrid$cen$lon, prec='double')
  ncatt_put(ncNew, varid=0, attname='XCENT', attval = myGrid$cen$lon, prec='double')
  ncatt_put(ncNew, varid=0, attname='YCENT', attval = myGrid$cen$lat, prec='double')
  ncatt_put(ncNew, varid=0, attname='XORIG', attval = -myGrid$E0*10^3, prec='double')
  ncatt_put(ncNew, varid=0, attname='YORIG', attval = -myGrid$N0*10^3, prec='double')
  ncatt_put(ncNew, varid=0, attname='XCELL', attval = myGrid$xcell*10^3, prec='double')
  ncatt_put(ncNew, varid=0, attname='YCELL', attval = myGrid$ycell*10^3, prec='double')
  
  ncatt_put(ncNew, varid=0, attname='VGTYP', attval = as.integer(2), prec='int')
  ncatt_put(ncNew, varid=0, attname='VGTOP', attval = 10000.0, prec='float')
  ncatt_put(ncNew, varid=0, attname='VGLVLS', attval = c(1.0, 0.98), prec='float')
  ncatt_put(ncNew, varid=0, attname='GDNAM', attval = gridType, prec='text')
  ncatt_put(ncNew, varid=0, attname='UPNAM', attval = '', prec='text')
  ncatt_put(ncNew, varid=0, attname='VAR-LIST', 
            attval = 'MASK            SURF            OPEN            ', 
            prec='text')
  ncatt_put(ncNew, varid=0, attname='FILEDESC', attval="OCEAN file", prec='text')
  ncatt_put(ncNew, varid=0, attname='HISTORY', 
            attval = "Created at HZG KBT by cmaqtools and some scripts", 
            prec='text')
  
  # WRITE DATA
  ncvar_put(ncNew, varid='TFLAG', rep(as.integer(0),6), start = c(1, 1, 1), count = c(2, 3, 1))
  ncvar_put(ncNew, varid='MASK', valMASK, start = c(1, 1, 1, 1), count = c(nCols, nRows, 1, 1))
  ncvar_put(ncNew, varid='OPEN', valOPEN, start = c(1, 1, 1, 1), count = c(nCols, nRows, 1, 1))
  ncvar_put(ncNew, varid='SURF', valSURF, start = c(1, 1, 1, 1), count = c(nCols, nRows, 1, 1))
  
  nc_close(ncNew)
}

# dummyMASK = valMASK
# dummyMASK[valMASK == 2] = as.integer(0)
# dummySURF = dummyMASK * 0.05/24
# diffSURF = dummySURF - valSURF
# image(diffSURF, col = append(rgb(0:nCol/nCol, 1, 1), rgb(1, 1, nCol:1/nCol)), breaks = myBreaks)
# summary(diffSURF)
# 
# relSURF = diffSURF / dummySURF
# relSURF[is.nan(relSURF)] = 0
# plot(relSURF[order(relSURF)])
  
  }
}