
# version ----
#' @date 2015-11-27
#' @version 10.0.0.0


# functions ----

#' Calculate the last day of the year "year" in the date format YYYYDDD
#' in which the day goes from 1 to 365 or 366, respectively.
#' 
#' Here we test whether "year" is a leap year or not. If it is leap year
#' doy=YYYY366 and if not doy=YYYY365. doy always has 7 digits.
#' 
#' Alternative and shorter form:
#' julian(as.Date(paste(year,"-12-31",sep="")), origin=as.Date(paste(year,"-01-01",sep="")))+1
#' 
#' @param year ; int ; The year for which the last day of the year should be
#'    calculated. Year is an integer of the form YYYY. Thus the year 1993
#'    is parsed as year=1993 and NOT as year=93.
#' @return doy ; int ; The last day of the year "year" in the YYYYDDD format
#'    (1 =< doy-year =< 366). doy is an integer of the form YYYYDDD. Thus
#'    for the year 1993 we have doy=1993365 and NOT 365.
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note There is an alternative and shorter form. 
#' @format R Code
getMaxDoy <- function(year) {
  # Alternative and shorter form:
  # julian(as.Date(paste(year,"-12-31",sep="")), origin=as.Date(paste(year,"-01-01",sep="")))+1
  
  # If year is no integral multiple of 4 we have definitely no leap year
  # and doy is set to YYYY365.
  if(year %% 4 != 0) {
    doy = year*10^3 + 365
  } else {
    # Every century a leap year is skipped (1700, 1800, 1900 etc.) but not
    # it can be divided by 400 (1600, 2000 etc.). In the latter case the 
    # year is a leap year.
    if (year %% 100 == 0 && year %% 400 != 0) {
      doy = year*10^3 + 365
    } else {
      doy = year*10^3 + 366
    }
  }
  return(as.integer(doy))
}


#' Convert a time
#' 
#' The array times contains dates of measurements in
#' days (real variable; + hours divided by 24) with 
#' respect to a reference date refDate. refDate has to
#' be given in Julian format (YYYYDDD).
#' 
#' @param times ; 
#' @param refDate ; 
#' @return newTimes ; 
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note ... 
#' @format R Code
reworkTiming <- function(times, refDate) {
  workingTimes = times
  nDates = length(workingTimes)
  newTimes = rep(0, nDates)
    
  YEAR <- floor(refDate/1000)
  
  actRefDate = refDate
  maxDoy = getMaxDoy(YEAR)
  
  for (i1 in 1:nDates) {
    tmpDate = actRefDate + workingTimes[i1]
        
    while (tmpDate <= YEAR*10^3){
      oldYEAR = YEAR
      YEAR = YEAR - 1
      maxDoy = getMaxDoy(YEAR)
      tmpDate = maxDoy - (oldYEAR*10^3 - tmpDate)
      actRefDate = maxDoy + (actRefDate - oldYEAR*10^3)
    }
    
    while (tmpDate > maxDoy) {
      oldYEAR = YEAR
      YEAR = YEAR + 1
      tmpDate = YEAR*10^3 + (tmpDate-maxDoy)
      actRefDate = YEAR*10^3 - (maxDoy - actRefDate)
      maxDoy = getMaxDoy(YEAR)
    }
    
    newTimes[i1] = tmpDate
  }
  
  return(newTimes)
}


#' Convert the name of a variable into a convenient form.
#' 
#' shortEMEP: sum formular of the substance as used in the EMEP ASCII
#'            export files; standard sum formular with - and +; no spaces
#' shortDB:   sum formular of the substance without - and +; sometimes
#'            shorter than the EMEP short format such as susp_part_mat
#'            instead of susp_part_matter; sometimes more information as
#'            wspd10m instead of wspd; no spaces;
#' long:      long name of the substance as used in the variable definitions
#'            in the EMEP ASCII export file (header, line 13 and following) 
#'            (often IUPAC convention); no spaces;
#'            
#' @param titleIn ; character ; a variable name which should be converted into 
#'    another format
#' @param conv2which ; character ; (optional) defines in which format titleIn 
#'    should be converted; It can be choosen from "long", "short", "shortEMEP",
#'    "shortDB" and "" whereas "" and "short" are equal to "shortDB". If
#'    set to another value a warning is thrown and titleOut=titleIn is 
#'    returned. The default value of conv2which is "shortDB".
#' @return titleOut ; character ; the variable name in the format defined by 
#'    conv2which; by default 
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note If titleIn is unknown then titleOut = titleIn. 
#' @format R Code
titleConvert <- function(titleIn, conv2which="") {
  ##
  # col 1: short name; EMEP files, data columns
  # col 2: short name; no special characters as +/-
  # col 3: long name; EMEP files, variable description in the header
  # col 4: long name; alternative name
  # col 5: long name; official table to EMEP measurement units
  titles = c("SO2", "SO2" ,"sulphur_dioxide", "sulphur_dioxide", "Sulphur dioxide", 
             "SO4--", "SO4", "sulphate_total", "sulphate", "Sulphate total", 
             "XSO4--", "XSO4", "sulphate_corrected", "sulphate_corrected", "Sulphate corrected for sea salt contribution", 
             "PO4---", "SO4", "phosphate", "phosphate", "Phosphate", 
             
             "NH3", "NH3", "ammonia", "ammonia", "Ammonia", 
             "NH4+", "NH4", "ammonium", "ammonium", "Ammonium", 
             "NH3+NH4", "SNH4", "sum_ammonia_and_ammonium", "sum_ammonia_and_ammonium", "Sum ammonia and ammonium", 
             
             "NO", "NO", "nitrogen_oxide", "nitrogen_monoxide", "Nitrogen Oxide", 
             "NO2", "NO2", "nitrogen_dioxide", "nitrogen_dioxide", "Nitrogen Dioxide", 
             "N2O", "N2O", "dinitrogen_oxide", "nitrous_oxide", "Dinitrogen Oxide", 
             "NO3-", "NO3", "nitrate", "nitrate", "Nitrate", 
             "HNO2", "HNO2", "nitrous_acid", "nitrous_acid", "Nitrous Acid",
             "HNO3", "HNO3", "nitric_acid", "nitric_acid", "Nitric Acid",
             "HNO3+NO3", "SNO3", "sum_nitric_acid_and_nitrate", "sum_nitric_acid_and_nitrate", "Sum nitric acid and nitrate", 
             
             "O3", "O3", "ozone", "ozone", "Ozone", 
             
             "CH4", "CH4", "methane", "methane", "Methane", 
             "MSA", "MSA", "methanesulfonic_acid", "methanesulfonic_acid", "Methanesulfonic Acid", 
             
             "CO", "CO", "carbon_oxide", "carbon_monoxide", "Carbon Oxide", 
             "CO2", "CO2", "carbon_dioxide", "carbon_dioxide", "Carbon Dioxide", 
             
             "Mg++", "Mg", "magnesium", "magnesium", "Magnesium", 
             "Ca++", "Ca", "calcium", "calcium", "Calcium", 
             "Na+", "Na", "sodium", "sodium", "Sodium", 
             "K+", "K", "potassium", "potassium", "Potassium", 
             "Cl-", "Cl", "chloride", "chlorine", "Chloride", 
             "Br-", "Br", "bromide", "bromide", "Bromide", 
             "HCl", "HCl", "hydrochloric_acid", "hydrochloric_acid", "Hydrochloric Acid", 
             "H+", "H", "acidity", "acidity", "Acidity", 
             
             "Hg", "Hg", "total_mercury", "mercury", "Mercury", 
             "TGM", "TGM", "total_gaseous_mercury", "total_gaseous_mercury", "Total Gaseous Mercury", 
             "RGM", "GOM", "gaseous_oxidized_mercury", "reactive_gaseous_mercury", "Reactive Gaseous Mercury", 
             "GEM", "GEM", "elemental_gaseous_mercury", "gaseous_elemental_mercury", "Gaseous Elemental Mercury", # NEW_v09_105
             # OLD: "GEM", "GEM", "gaseous_elemental_mercury", "gaseous_elemental_mercury", "Gaseous Elemental Mercury", 
             "PBM", "PBM", "particle_bound_mercury", "particle_bound_mercury", "Particle Bound Mercury", 
             "Al", "Al", "aluminium", "aluminium", "Aluminium", 
             "Al27", "Al27", "aluminium_27", "aluminium_27", "Aluminium 27", 
             "Sb", "Sb", "antimony", "antimony", "Antimony", 
             "As", "As", "arsenic", "arsenic", "Arsenic", 
             "Ba", "Ba", "barium", "barium", "Barium", 
             "Cd", "Cd", "cadmium", "cadmium", "Cadmium", 
             "Cr", "Cr", "chromium", "chromium", "Chromium", 
             "Co", "Co", "cobalt", "cobalt", "Cobalt", 
             "Cu", "Cu", "copper", "copper", "Copper", 
             "Fe", "Fe", "iron", "iron", "Iron", 
             "Fe57", "Fe57", "iron_57", "iron_57", "Iron 57", 
             "Pb", "Pb", "lead", "lead", "Lead", 
             "Li+", "Li", "lithium", "lithium", "Lithium", 
             "Mn", "Mn", "manganese", "manganese", "Manganese", 
             "Ni", "Ni", "nickel", "nickel", "Nickel",
             "Ni58", "Ni58", "nickel", "nickel", "Nickel 58", 
             "Ni60", "Ni60", "nickel", "nickel", "Nickel 60", 
             "Ru", "Ru", "ruthenium", "ruthenium", "Ruthenium", 
             "Sc", "Sc", "scandium", "scandium", "Scandium", 
             "Se", "Se", "selenium", "selenium", "Selenium", 
             "Sr", "Sr", "strontium", "strontium", "Strontium", 
             "Sn", "Sn", "tin", "tin", "Tin", 
             "Ti", "Ti", "titanium", "titan", "Titanium", 
             "Tl", "Tl", "thallium", "thallium", "Thallium", 
             "U", "U", "uranium", "uranium", "Uranium", 
             "V", "V", "vanadium", "vanadium", "Vanadium", 
             "Zn", "Zn", "zinc", "zinc", "Zinc", 
             
             "pm10_mass", "pm10_mass", "pm10_mass", "PM10", "PM10 mass", 
             "pm25_mass", "pm25_mass", "pm25_mass", "PM2.5", "PM2.5 mass", 
             "pm1_mass", "pm1_mass", "pm1_mass", "PM1", "PM1.0 mass",
             "susp_part_matter", "susp_part_mat", "suspended_particulate_matter", "SPM", "Suspended particulate matter", 
             "particle_number_concentration", "pnc", "particle_number_concentration", "PNC", "particle number concentration",
             "EC", "EC", "elemental_carbon", "elemental_carbon", "EC", 
             "OC", "OC", "organic_carbon", "organic_carbon", "OC", 
             "XOC", "XOC", "organic_carbon_corrected", "organic_carbon_corrected", "OC corrected", 
             "TC", "TC", "total_carbon", "total_carbon", "TC",  
             "XTC", "XTC", "total_carbon_corrected", "total_carbon_corrected", "TC corrected", 
             "BC", "BC", "black_carbon", "black_carbon", "BC",
             
             "mm", "precip", "precipitation_amount", "precipitation", "Precipitation amount", 
             "mm_off", "precip_off", "precipitation_amount_off", "precipitation_off", "Precipitation amount official gauge", 
             "k", "Cond", "conductivity", "conductivity", "Conductivity", 
             "pH", "pH", "pH", "pH", "pH",
             
             "rh", "RH", "relative_humitidty", "relative_humidity", "Relative Humidity", 
             "p", "P", "pressure", "atmospheric_pressure", "Pressure", 
             "T", "T", "temperature", "temperature", "Temperature", 
             "T_d", "Td", "dewpoint_temperature", "dewpoint_temperature", "dewpoint_Temperature", 
             "wdir", "wdir10m", "wind_direction", "wind_direction", "Wind Direction", 
             "wspd", "wspd10m", "wind_speed", "wind_speed", "Wind Speed",
             
             "heav_met_dep", "heav_met_dep", "heavy_metal_deposition", "heavy_metal_deposition", "Heavy metals deposition", 
             "POP_dep", "POP_dep", "POP_deposition", "POP_deposition", "POP deposition", 
             
             "", "", "", "", ""
             )
  # "", "", "", "", "", 
  
  # number of variables for which name-conversion is possible
  nTitles = length(titles)/5   
  # Make from the 1-dim titles array a 2-dim titles array
  #  -> nice table with rows corresponding to different formats
  #     and columns correstponding to different substances
  dim(titles) = c(5, nTitles)
  # "rotate" table (actually we transpose it); We want rows and columns
  # to be exchanged.
  titles = t(titles)
  
  titleOut = ""
  
  # Which formats are allowed to chose?
  formats = c("long",
              "short",
              "shortEMEP",
              "shortDB",
              "")
  
  # Test whether a valid format is parsed (conv2which)
  if (sum(conv2which == formats)) {
    colsIn = 1:5    # columns in which it is looked for titleIn
    colOut = 0      # If titleIn is found one column: From which
                    # other column should titleOut be chosen?
    # We have transposed 'titles'. Therefore the 'columns' are
    # 'rows' actually.
    
    # If the format 'short' or no format ("") were parsed, conv2which
    # is set to "shortDB". "shortDB" will cause the least problems. 
    # A '+' or '-' could cause problems in some cases because it could
    # be regarded as a special character.
    if (sum(conv2which == c("", "short"))) conv2which = "shortDB"
    
    # Inquire which format is the wished output format. Depending on
    # the output format 'colsOut' is set.
    if (conv2which == "shortEMEP") {
      colOut = 1
    } else if (conv2which == "shortDB") {
      colOut = 2
    } else if (conv2which == "long") {
      colOut = 3
    } else { # redundant, because if(sum(conv2which == formats)) should have
      # catched this case already. However, one never knows ... :-) .
      warning("Format to convert to titleIn is unknown!")
    }
    
    ## We will go throught all rows of titles (while ... ) till all catalogued
    ## titles are past (i1 < nTitles) or till a row containing titleIn is 
    ## found (!found).
    ## Within each row we go through each column (for ...) defined in colsIn.
    ## If titleIn is found in the current row i1 (titles[i1, i2] == titleIn)
    ## the titleOut is set to the value from this row i1 and column colOut 
    ## in the titles array.
    # set some initial values
    i1 = 0
    found = FALSE
    while (i1 < nTitles && !found) {
      # increase iterator (row number)
      i1 = i1 + 1
      # go through each column
      for (i2 in colsIn) {
        # if titleIn is found in titles
        if (titles[i1, i2] == titleIn) {
          found = TRUE
          titleOut = titles[i1, colOut]
        }
      }
    }
    
    # If titleIn was not found in the titles array, titlesOut will be set
    # to titlesIn
    if (!found) titleOut = titleIn
    
  } else { # If conv2which is no element of formats we do the following:
    titleOut = titleIn
    warning("Format to convert to titleIn is unknown!")
  }
  
  # print(paste(titleIn, '->', titleOut, sep = ' '))
  
  # parse titleOut back to the user
  return(titleOut)
}


#' Test an EMEP data file returned by the emep_file_parser for duplicate
#' variables and parse back the indices of duplicate variables. The short
#' variable names are tested for duplicates.
#' 
#' WE HAVE TWO SIMILAR FUNCTIONS 'findDuplicates' AND 'findDuplicatesB'.
#' WHILE THE FIRST ONE ONLY COMPARES THE short variable name, THE SECOND
#' ONE COMPARES short variables name, matrix and method_ref. 
#' 
#' The short variables names are emep_data$measurements$titleVarsShort
#' 
#' @param emep_data ; an EMEP data file given out by emep_file_parser
#' @return integer_array ; integer array ; an array of integers giving the  
#'    positions of duplicate variables in the input file (titleVarsShort is 
#'    tested)
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note If no duplicates are present, an empty list is returned. 
#' @format R Code
emep_findDuplicates <- function(emep_data) {
  # What do we do here?
  #  - emep_data$measurements$titleVarsShort
  #    + get a character array containing the short variable names
  #  - duplicated({SHORT_VAR_NAMES})
  #    + get a boolean array of the same length as {SHORT_VAR_NAMES}
  #    + The entry in the boolean array is FALSE if the corresponding
  #       value in {SHORT_VAR_NAMES} has no duplicate or if is the 
  #       first duplicate value seen from the left. Otherwise the entry
  #       is TRUE.
  #    + see documentation of duplicated() for details
  #  - (1:emep_data$nVars)[{TAKE_ONLY}]
  #    + list from 1 to emep_data$nVars but take only those values where
  #       {TAKE_ONLY} is TRUE (if this is a boolean array) or those values
  #       which indices are given by {TAKE_ONLY} (if this is a integer array).
  return((1:emep_data$nVars)[duplicated(emep_data$measurements$titleVarsShort)])
}


## NEW_v09_101
#' Test an EMEP data file returned by the emep_file_parser for duplicate
#' variables and parse back the indices of duplicate variables. The short
#' variable names, the matrices and the method_refs are tested for 
#' duplicates.
#' 
#' WE HAVE TWO SIMILAR FUNCTIONS 'findDuplicates' AND 'findDuplicatesB'.
#' WHILE THE FIRST ONE ONLY COMPARES THE short variable name, THE SECOND
#' ONE COMPARES short variables name, matrix and method_ref. 
#' 
#' The short variables names are emep_data$measurements$titleVarsShort
#' The matrices are emep_data$measurements$matrices
#' The method_refs are emep_data$measurements$method_refs
#' 
#' @param emep_data ; an EMEP data file given out by emep_file_parser
#' @return integer_array ; integer array ; an array of integers giving the  
#'    positions of duplicate variables in the input file (titleVarsShort,
#'    matrix and method_ref are tested)
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note If no duplicates are present, an empty list is returned. 
#' @format R Code
emep_findDuplicatesB <- function(emep_data) {
  return((1:emep_data$nVars)[duplicated(
          paste(
              emep_data$measurements$titleVarsShort,
              emep_data$measurements$matrices,
              emep_data$measurements$method_refs,
              sep="_"
              )
          )]
        )
}
## NEW_v09_101


#' Remove duplicate variables from an EMEP data variable/list which was
#' returned by the emep_file_parser.
#' 
#' The array indices of duplicate variables are identified by the function
#' emep_findDuplicates(emep_data).
#' 
#' Following parts of emep_data are changed if duplicates are present:
#'  drop elements:
#'    measurements$titleVarsLong
#'    measurements$titleVarsShort
#'    measurements$units
#'    measurements$variables
#'      note: variables is a 2-dim array; the 2nd dimension has to be changed
#'    flags$titleFlagsLong
#'    flags$titleFlagsShort
#'    flags$numflags
#'      note: variables is a 2-dim array; the 2nd dimension has to be changed
#'  alter integer:
#'    nVars
#' 
#' @param emep_data ; an EMEP data file given out by emep_file_parser
#' @param removeList ; integer array ; (optional) an integer array containing
#'    indices of variables which shoul be removed from emep_data; if not 
#'    parsed, removedList is filled by the call of 
#'    emep_findDuplicates(emep_data)
#' @return \item{name changes}{description boolean ; FALSE if new_data equals
#'          emep_data and TRUE otherwise (= if variables were dropped)}
#'  \item{name removeList}{description integer array ; the same integer array as returned
#'          by emep_findDuplicates(emep_data)}
#'  \item{name new_data}{description a list() of the same format as that 
#'          given back by the emep_file_parser but without duplicate 
#'          variables (titleVarsShort)}
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note If no duplicates are present, new_data equals emep_data and changes
#'    is FALSE
#' @format R Code
emep_removeDuplicates <- function(emep_data, removeList=emep_findDuplicates(emep_data)) {
  
  # Copy emep_data into new variable
  new_data = emep_data
  
  # If removeList has a length of 0 we do nothing. This is the case when 
  # emep_findDuplicates does not find any duplicates in emep_data.
  if (length(removeList) == 0) {
    changes = FALSE
  } else {
    changes = TRUE
    
    # Drop duplicate variables
    new_data$measurements$titleVarsLong = emep_data$measurements$titleVarsLong[-removeList]
    new_data$measurements$titleVarsShort = emep_data$measurements$titleVarsShort[-removeList]
    new_data$measurements$units = emep_data$measurements$units[-removeList]
    new_data$measurements$variables = emep_data$measurements$variables[,-removeList]
    new_data$flags$titleFlagsLong = emep_data$flags$titleFlagsLong[-removeList]
    new_data$flags$titleFlagsShort = emep_data$flags$titleFlagsShort[-removeList]
    new_data$flags$numflags = emep_data$flags$numflags[,-removeList, ] # NEW_v09_102
    # OLD: new_data$flags$numflags = emep_data$flags$numflags[,-removeList]
    
    if(emep_data$nVars - length(removeList) == 1) {
      dim(new_data$measurements$variables) = c(emep_data$nRowData, 1)
      dim(new_data$flags$numflags) = c(emep_data$nRowData, 1)
    }
    
    # Set new number of variables
    new_data$nVars = length(new_data$measurements$titleVarsLong)
  }
  
  # Return the modified data (new_data), the information whether the data was
  # actually modified (changes) and the old indices which were dropped 
  # (removeList).
  return(list(
    'changes'=changes,
    'removeList'=removeList,
    'new_data'=new_data
    ))
}


## NEW_v09_103
#' Remove duplicate variables from an EMEP data variable/list which was
#' returned by the emep_file_parser.
#' 
#' The array indices of duplicate variables are identified by the function
#' emep_findDuplicatesB(emep_data).
#' 
#' WE HAVE TWO SIMILAR FUNCTIONS 'removeDuplicates' AND 'removeDuplicatesB'.
#' WHILE THE FIRST ONE ONLY COMPARES THE short variable name, THE SECOND
#' ONE COMPARES short variables name, matrix and method_ref. 
#' 
#' Following parts of emep_data are changed if duplicates are present:
#'  drop elements:
#'    measurements$titleVarsLong
#'    measurements$titleVarsShort
#'    measurements$units
#'    measurements$matrices
#'    measurements$method_refs
#'    measurements$variables
#'      note: variables is a 2-dim array; the 2nd dimension has to be changed
#'    flags$titleFlagsLong
#'    flags$titleFlagsShort
#'    flags$numflags
#'      note: variables is a 2-dim array; the 2nd dimension has to be changed
#'  alter integer:
#'    nVars
#' 
#' @param emep_data ; an EMEP data file given out by emep_file_parser
#' @param removeList ; integer array ; (optional) an integer array containing
#'    indices of variables which shoul be removed from emep_data; if not 
#'    parsed, removedList is filled by the call of 
#'    emep_findDuplicates(emep_data)
#' @return \item{name changes}{description boolean ; FALSE if new_data equals
#'          emep_data and TRUE otherwise (= if variables were dropped)}
#'  \item{name removeList}{description integer array ; the same integer array as returned
#'          by emep_findDuplicates(emep_data)}
#'  \item{name new_data}{description a list() of the same format as that 
#'          given back by the emep_file_parser but without duplicate 
#'          variables (titleVarsShort)}
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note If no duplicates are present, new_data equals emep_data and changes
#'    is FALSE
#' @format R Code
emep_removeDuplicatesB <- function(emep_data, removeList=emep_findDuplicatesB(emep_data)) {
  
  # Copy emep_data into new variable
  new_data = emep_data
  
  # If removeList has a length of 0 we do nothing. This is the case when 
  # emep_findDuplicates does not find any duplicates in emep_data.
  if (length(removeList) == 0) {
    changes = FALSE
  } else {
    changes = TRUE
    
    # Drop duplicate variables
    new_data$measurements$titleVarsLong = emep_data$measurements$titleVarsLong[-removeList]
    new_data$measurements$titleVarsShort = emep_data$measurements$titleVarsShort[-removeList]
    new_data$measurements$units = emep_data$measurements$units[-removeList]
    new_data$measurements$matrices = emep_data$measurements$matrices[-removeList]
    new_data$measurements$method_refs = emep_data$measurements$method_refs[-removeList]
    new_data$measurements$variables = emep_data$measurements$variables[,-removeList]
    new_data$flags$titleFlagsLong = emep_data$flags$titleFlagsLong[-removeList]
    new_data$flags$titleFlagsShort = emep_data$flags$titleFlagsShort[-removeList]
    new_data$flags$numflags = emep_data$flags$numflags[,-removeList, ] 
    
    if(emep_data$nVars - length(removeList) == 1) {
      dim(new_data$measurements$variables) = c(emep_data$nRowData, 1)
      dim(new_data$flags$numflags) = c(emep_data$nRowData, 3)
    }
    
    # Set new number of variables
    new_data$nVars = length(new_data$measurements$titleVarsLong)
  }
  
  # Return the modified data (new_data), the information whether the data was
  # actually modified (changes) and the old indices which were dropped 
  # (removeList).
  return(list(
    'changes'=changes,
    'removeList'=removeList,
    'new_data'=new_data
  ))
}
## NEW_v09_103


## NEW_v09_104
#' return new short and long name of hg species based on old short and long
#' names and based on matrix and unit. Sometimes mercury measurements are 
#' only denoted as 'Hg'. If it is measured in the air, it is probably TGM 
#' (total gaseous mercury). If it is measured in the aerosol phase, it is 
#' probably PBM (particulate bound mercury). If matrix does not help, the
#' unit of the measurement may help ...
#' 
#' TODO
#' 
#' @param titleVarShort ; short name of variable
#' @param titleVarLong ; long name of variable
#' @param matrix ; matrix of variable (= measured where? e.g. air, aerosol)
#' @param matrix ; unit of variable
#' @author Daniel Neumann, Helmholtz-Zentrum Geesthacht
#'    \email{daniel.neumann@hzg.de}
#' @note Currently: Empty function
#' @format R Code
hgConvert_special <- function(titleVarShort, titleVarLong, matrix, unit) {
  
}
## NEW_v09_104