#' This script plots input data (U10, SST, SAL and RE)
#' for the sea salt emission parameterizations into
#' map plots (one *.eps file).
#' 
#' @author Daniel Neumann, daniel.neumann@hzg.de
#' @version 1.1
#' @date 2015-11-22
#' 



# load / save data ----
## save(list = ls(all=TRUE), file = "inData_map_input_data.RData")
## load("inData_map_input_data.RData")



# control parameters ----
data.read = FALSE
data.work = FALSE
data.plot = TRUE



# load libraries and functions ----
library('ncdf4')
library('cmaqtools')

source('color.bar.my.hori.R')
source('plot.griddata.mod.R')
source('plot.griddata.lambert.mod.R')



# set directories and file names ----
inDir = '/storage/neumannd/data/cctm/CD24/analysis'
otDir = '05_maps'



# set parameters ----
GRID_NAME = 'CD24'
myGrid = griddesc(GRID_NAME)

MONs = c('win2008', 'sum2008')
VARs = c('u10', 'SST', 'SAL', 'RE')

zMin = list('u10'=0, 'SST'=273-273-5/3, 'SAL'=5, 'RE'=(2-2/3))
zMax = list('u10'=12, 'SST'=298-273, 'SAL'=37, 'RE'=12)

zLab = list('u10'=expression('u'[10] ~ '[m/s]'), 'SST'=expression('SST [' * degree * 'C]'), 'SAL'='SAL [g/kg]', 'RE'=expression('Re'['Hw'] ~ '  x 10'^5))
zUnit = list('u10'='m/s', 'SST'='C', 'SAL'='g/kg', 'RE'=expression('10'^5))
rain.bow = rev(rainbow(24)[1:17])
zCol = list('u10' = rain.bow, 'SST'=rain.bow, 'SAL'=rain.bow, 'RE'=rain.bow)
zLimits = list('u10' = (0:16)*3/4, 'SST'=append((0:15)*5/3,-999, after = 0), 
               'SAL'= append((0:15)*2+7, -999, after = 0), 
               'RE'=append((0:15)*2/3+2, -999, after = 0))
zLevels = list('u10' = 17, 'SST'=17, 'SAL'=17, 'RE'=17)
zIndices = list('u10' = (0:4)*4+1, 'SST'=(0:5)*3+2,
                'SAL'=(0:5)*3+2, 'RE'=(0:5)*3+2)

# read data ----
if(data.read) {
  print('... reading data ...')
  rawDat = list()
  for(iMon in MONs) {
    rawDat[[iMon]] = list()
    for(iVar in VARs) { rawDat[[iMon]][[iVar]] = array(0.0, dim = c(myGrid$ncol, myGrid$nrow)) } }
  
  
  for (iMon in MONs) {
    # read data: read met ----
    ncId = nc_open(paste(inDir, paste('METCRO2D', iMon, 'short.timmean', sep = '.'), sep = '/'))
    rawDat[[iMon]][['u10']] = ncvar_get(ncId, 'WSPD10')
    nc_close(ncId)
    
    # read data: read sst and salinity ----
    ncId = nc_open(paste(inDir, paste('bsh', GRID_NAME, iMon, 'short.timmean', sep = '.'), sep = '/'))
    rawDat[[iMon]][['SST']] = ncvar_get(ncId, 'SST') - 273.3
    rawDat[[iMon]][['SAL']] = ncvar_get(ncId, 'SALINITY')
    nc_close(ncId)
    
    # read data: read met ----
    ncId = nc_open(paste(inDir, paste(paste('RE', GRID_NAME, iMon, sep = '_'), 'short.timmean', sep = '.'), sep = '/'))
    rawDat[[iMon]][['RE']] = ncvar_get(ncId, 'RE') * 10^(-5)
    nc_close(ncId)
    
  }
}


# plot data ----
if(data.plot) {
  print('... plotting data ...')
  
  # postscript(file = paste(otDir, 'fig04_input_data_maps.vert.eps', sep = '/'), onefile = TRUE, paper = 'special',
  #            horizontal = TRUE, width = 11.5, height = 6.2)
  pdf(file = paste(otDir, 'fig04_input_data_maps.pdf', sep = '/'), onefile = TRUE, paper = 'special',
             width = 11.5, height = 6.2)
  # postscript(file = paste(otDir, 'fig04_input_data_maps.eps', sep = '/'), onefile = TRUE, paper = 'special',
  #            horizontal = FALSE, width = 14, height = 8.5)
  par(mar = c(0,0,0,0))
  
  # plot data: define screens ----
  vPPd = 0.012  # vertical distance: map to map
  vLPd = 0.012  # vertical distance: map to legend
  vLEGh = 0.17
  vTOPd = 0.01  # vertical distance: top boundary
  vBOTd = 0.01  # vertical distance: bottom boundary
  vPLTh = (1 - vPPd - vLPd - vLEGh - vTOPd - vBOTd) / 2
  
  hLEFd = 0.03 # horizontal distance: left boundary
  hRIGd = 0.005 # horizontal distance: right boundary
  hPLTd = 0.02 # horizontal distance: map to map
  hPLTw = (1 - hLEFd - hRIGd - 3*hPLTd) / 4
  
  h1 = hLEFd
  h2 = hLEFd + 1 * hPLTw
  h3 = hLEFd + 1 * hPLTw + 1 * hPLTd
  h4 = hLEFd + 2 * hPLTw + 1 * hPLTd
  h5 = hLEFd + 2 * hPLTw + 2 * hPLTd
  h6 = hLEFd + 3 * hPLTw + 2 * hPLTd
  h7 = hLEFd + 3 * hPLTw + 3 * hPLTd
  h8 = hLEFd + 4 * hPLTw + 3 * hPLTd
  
  v1 = vBOTd
  v2 = vBOTd + vLEGh
  v3 = vBOTd + vLEGh + vLPd
  v4 = vBOTd + vLEGh + vLPd + 1 * vPLTh
  v5 = vBOTd + vLEGh + vLPd + 1 * vPLTh + vPPd
  v6 = vBOTd + vLEGh + vLPd + 2 * vPLTh + vPPd
  
  split.screen(rbind(c(h1,h2,v5,v6), c(h1,h2,v3,v4), c(h1,h2,v1,v2),
                     c(h3,h4,v5,v6), c(h3,h4,v3,v4), c(h3,h4,v1,v2),
                     c(h5,h6,v5,v6), c(h5,h6,v3,v4), c(h5,h6,v1,v2),
                     c(h7,h8,v5,v6), c(h7,h8,v3,v4), c(h7,h8,v1,v2)))
  
  iScreen = 0
  
  for(iVar in VARs) {
    
    for (iMon in MONs) {
      iScreen = iScreen + 1
      screen(iScreen)
      
      plot.griddata.lambert.mod(griddata.lambert(rawDat[[iMon]][[iVar]], myGrid, var = iVar, unit = zUnit[[iVar]]),
                                hires = FALSE, col = zCol[[iVar]], limits = zLimits[[iVar]],
                                levels = zLevels[[iVar]])
      box(lwd = 3)
    }
    
    
    iScreen = iScreen + 1
    screen(iScreen)
    
    # color.my.hori(zCol, min = zMin[[iVar]], max = zMax[[iVar]], title = zLab[[iVar]])
    color.bar.my.hori(zCol[[iVar]], min = zMin[[iVar]], max = zMax[[iVar]], title = zLab[[iVar]],
                  nticks = length(zIndices[[iVar]]), ticks = zLimits[[iVar]][zIndices[[iVar]]], ltick = zLevels[[iVar]]-1)
  }
  
  screen(1)
  mtext('winter', 2, font = 2, cex = 1.8, line = +0.2)
  
  screen(2)
  mtext('summer', 2, font = 2, cex = 1.8, line = 0.2)
  
  close.screen(all = TRUE)
  dev.off()
}
