# combine_sim_and_obs

#' @author Daniel Neumann, daniel.neumann@hzg.de
#' @date 2016-03-28
#' @version 2.2.2
#' 
#' This script creates plots with correlation coefficients and MNBs
#' of sea salt sulfate calculated by two different means from model
#' data. Additionally, these values are compared against sea salt
#' sulfate from measurements in the EMEP database.
#' 
#' NOTE 1: Load the input data manually if not already loaded. Call
#' the out-commented 'load' command for this purpose.
#' NOTE 2; Set the boolean variables workData, plotA etc. in the
#' section "header: control what to do" according to your needs.


# header: load libraries and other scripts ----
library('ncdf4')
library('cmaqtools')
library('ctmeval')

source('./handy_functions.R')
source('list_stations.R')


# header: load data for plotting ----
## save(list = ls(all=TRUE), file = "inData_combine_sim_and_obs_newPlotOrder.RData")
## load("inData_combine_sim_and_obs_newPlotOrder.RData")


fDirValid = '17_sulfValidation'
fDirConc = '07_conc'
fDirStat = '09_statisticalData'

# header: control what to do ----
# Create plots for Daniel's publication III.
read.stats = FALSE
read.obs = FALSE
work.obs = FALSE
read.sim = FALSE
work.sim = FALSE
plot.dat.A = FALSE  # overview, all stations, all scenarios, concentrations
plot.dat.B = FALSE  # overview, all stations, concentration bar plots (divided by scenario)
plot.dat.C = FALSE  # EAC2016
plot.dat.D = FALSE  # PHD One page, six stations, sulfate (used in thesis)
plot.dat.E = FALSE  # PHD One page, five stations (not used)
doStatistics = FALSE
write.dat = FALSE
so4.validation = TRUE
writeOutData = FALSE  # needs corrections

GRID_NAME = 'CD24'
myGrid = griddesc(GRID_NAME)

# myAero = 'aero5'
myAero = 'aero6'

mech = 'yamo_cb05tump_ae6_aq'
# mech = 'cb05tump_ae6_aq_ssa'
# mech = 'cb05tucl_ae5_aq_ssa'
# mech = 'cb05cl_ae5_aq'

setup = 'sulfScen'

# strYear='2008'
strYear='2013'


# header: functions and constant ----
make_daily_mean <- function(series) {
  dim_old = dim(series)
  
  if (length(dim_old) <= 1) {
    n_old = length(series)
    n_new = n_old/24
    
    means = rep(0.0, n_new)
    
    for(i1 in 1:n_new) {
      means[i1] = mean(series[(1:24)+(i1-1)*24])
    }
  } else if (length(dim_old) == 2) {
    n_old = dim_old[2]
    n_new = n_old/24
    dim_new = c(dim_old[1], n_new)
    
    means = array(0.0, dim = dim_new)
    
    for(i1 in 1:n_new) {
      means[,i1] = rowMeans(series[,(1:24)+(i1-1)*24], dims = 1)
    }
  } else if (length(dim_old) == 3) {
    n_old = dim_old[3]
    n_new = n_old/24
    dim_new = c(dim_old[1], dim_old[2], n_new)
    
    means = array(0.0, dim = dim_new)
    
    for(i1 in 1:n_new) {
      means[,,i1] = rowMeans(series[,,(1:24)+(i1-1)*24], dims = 2)
    }
  } else if (length(dim_old) == 4) {
    n_old = dim_old[4]
    n_new = n_old/24
    dim_new = c(dim_old[1], dim_old[2], dim_old[3], n_new)
    
    means = array(0.0, dim = dim_new)
    
    for(i1 in 1:n_new) {
      means[,,,i1] = rowMeans(series[,,,(1:24)+(i1-1)*24], dims = 3)
    }
  }
  
  return(means)
}


getYlim <- function(yMax, type) {
  if ( yMax == 0 ) {
    yLim = 1
  } else if ( type == 1 ) {
    yLim = ceiling(yMax/10^(floor(log10(yMax))-1))*10^(floor(log10(yMax))-1)
  } else if ( type == 2 ) {
    yLim = signif(yMax, digits = 1) * 2
  } else if ( type == 3 ) {
    yLim = signif(yMax, digits = 1)
    if(yLim < yMax) {
      yLim = signif(2*yMax-yLim, digits = 2)
    }
  } else {
    yLim = 1
  }
  
  return(yLim)
}


myMolMass = list('NO' = 30,'NO2' = 46, 'HNO3' = 63, 'HONO' = 45,
                 'ANO3I' = 62, 'ANO3J' = 62, 'ANO3K' = 62,
                 'NO3' = 62, 'PNA' = 79, 'PAN' = 121,
                 'NH3' = 17, 'ANH4I'= 18, 'ANH4J' = 18, 'ANH4K' = 18,
                 'N' = 15, 'N2O5' = 108,
                 'ASO4I' = 96, 'ASO4J' = 96, 'ASO4K' = 96, 'SO2' = 64, 'S' = 32)

R = 8.314462

if(myAero == 'aero6') {
  ssSO4Mass = 0.0776; ssClMass = 0.5538
  ssNaMass = 0.3086; ssMgMass = 0.0368; ssKMass = 0.0114; ssCaMass = 0.0118
  ssSeaCatMass = 0.3086+0.0368+0.0114+0.0118
} else if (myAero == 'aero5') {
  ssSO4Mass = 0.0755; ssClMass = 0.5389
  ssNaMass = 0.3856
  ssRealNa2Cat = 0.7800
}

# header: set some parameters ----
DBaccess <- list(name="neumannd-pg-emep_v02.1",uid="neumannd",pwd="nd$1,")
inDir = '/storage/neumannd/data/cctm/CD24/analysis'

# MONs = c('win2008')
MONs = c('win2013', 'sum2013')

SPECs = c('SO4', 'Na', 'XSO4', 'SNH4', 'SNO3', 'NH4', 'NO3', 'SO2')
allSPECs = c('SO4', 'Na', 'XSO4', 'SNH4', 'SNO3', 'NH4', 'NO3', 'SO2', 'NH3', 'HNO3', 'SOX')
MATRICEs = list('SNH4'=c('air+aerosol'), 'SNO3'=c('air+aerosol'), 
                'SO4'=c('pm10', 'pm25', 'aerosol'), 'XSO4'=c('pm10', 'pm25', 'aerosol'), 
                'Na'=c('pm10', 'pm25', 'aerosol'), 
                'NH4'=c('pm10', 'pm25', 'aerosol'), 'NO3'=c('pm10', 'pm25', 'aerosol'),
                'SO2' = c('air'), 'NH3' = c('air'), 'HNO3' = c('air'),
                'SOX' = c('pm10'))
allMATRICEs = list('SNH4'=c('pm10'), 'SNO3'=c('pm10'), 
                   'SO4'=c('pm10', 'pm25'), 'XSO4'=c('pm10', 'pm25'), 
                   'Na'=c('pm10', 'pm25'), 
                   'NH4'=c('pm10', 'pm25'), 'NO3'=c('pm10', 'pm25'),
                   'SO2' = c('air'), 'NH3' = c('air'), 'HNO3' = c('air'),
                   'SOX' = c('pm10'))
UNITS = list('SNH4'='ug N/m3', 'SNO3'='ug N/m3', 'SO4'='ug S/m3', 'XSO4'='ug S/m3', 'Na'='ug/m3', 'NH4'='ug N/m3', 'NO3'='ug N/m3', 'SO4'='ug S/m3')

SCENs = c('ALLsurf', 'NOSHIPSsurf', 'NOSALT', 'NOENERGY', 'SALT_OV14')

myStations = c("DE0002R", "DE0007R", "DE0044R", "DK0003R", "DK0008R", "DK0012R", "DK0031R", "FI0009R", "FI0017R", "LT0015R", "NL0091R", "NO0002R", "NO0015R", "NO0039R", "NO0056R", "PL0002R", "PL0004R", "PL0005R", "SE0005R", "SE0011R", "SE0012R", "SE0014R")
nSTATIONS=length(myStations)
theseStations = rep('', length = nSTATIONS)
for(iPos in 1:nSTATIONS) theseStations[iPos] = list_stations[[myStations[iPos]]]

# read station data ----
if (read.stats) {
  
  # get station data from the database
  myCon <- odbcConnect(dsn="neumannd-pg-emep_v02.1", uid="neumannd", pwd="nd$1,")
  stationInfo <- sqlQuery(myCon, paste("SELECT * FROM stations WHERE station_id IN ('", paste(myStations, collapse = "','"), "');", sep = ''))
  
  close(myCon)
  
  # write out coordiates and grid cells
  myPositions = list()
  myCells = list()
  
  for(iStat in myStations) {
    iPos = which(stationInfo$station_id == iStat)
    myPositions[[iStat]] = list('lat' = stationInfo$latitude[iPos],
                                'lon' = stationInfo$longitude[iPos])

    myCells[[iStat]] = findcell(myPositions[[iStat]], myGrid)
  }
  
}


# read observational data ----
if (read.obs) {
  
  rawOBS = list()
  for(iMon in MONs) { rawOBS[[iMon]] = list(); for(iSpec in SPECs) { rawOBS[[iMon]][[iSpec]] = list() } }
  
  for(iMon in MONs) {
    if(iMon == 'sum2013' || iMon == 'sum') {
      from = 2013182; to = 2013243; nT = 62
    } else if(iMon == 'win2013' || iMon == 'win') {
      from = 2013001; to = 2013059; nT = 59
    }
    nTh = nT * 24
  
    print('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
    print('READING OBSERVATIONAL DATA FROM DATABASE')
    print('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
    for (iSpec in SPECs) {
      for (iMtrx in MATRICEs[[iSpec]]) {
        rawOBS[[iMon]][[iSpec]][[iMtrx]] <- getobservationts(from,to,iSpec, stations = myStations, matrix=iMtrx, vlevel = gsub('or', 'and', gsub(', 781,', ',', vlevel(1))))
      }      
    }
  }
}
 

# rework obs data ----
if(work.obs) {
  valOBS = list()
  for(iMon in MONs) { valOBS[[iMon]] = list(); for(iSpec in c(SPECs, 'SOX')) { valOBS[[iMon]][[iSpec]] = list() } }
  
  for(iMon in MONs) {
    for (iSpec in c('SO4', 'Na', 'XSO4', 'NH4', 'NO3')) {
      valOBS[[iMon]][[iSpec]]$pm10 = rawOBS[[iMon]][[iSpec]]$aerosol
      if ( !is.null(rawOBS[[iMon]][[iSpec]]$pm10) ) {
        if (is.null(rawOBS[[iMon]][[iSpec]]$pm10$station) ) {
          for(iStat in names(rawOBS[[iMon]][[iSpec]]$pm10)) {valOBS[[iMon]][[iSpec]]$pm10[[iStat]] = rawOBS[[iMon]][[iSpec]]$pm10[[iStat]]}
        } else {
          valOBS[[iMon]][[iSpec]]$pm10[[as.character(rawOBS[[iMon]][[iSpec]]$pm10$station$station_id)]] = rawOBS[[iMon]][[iSpec]]$pm10
        }
      }
      
      if (!is.null(rawOBS[[iMon]][[iSpec]]$pm25)) {
        if (is.null(rawOBS[[iMon]][[iSpec]]$pm25$station)) {
          valOBS[[iMon]][[iSpec]]$pm25 = rawOBS[[iMon]][[iSpec]]$pm25
        } else {
          valOBS[[iMon]][[iSpec]]$pm25 = list()
          valOBS[[iMon]][[iSpec]]$pm25[[as.character(rawOBS[[iMon]][[iSpec]]$pm25$station$station_id)]] = rawOBS[[iMon]][[iSpec]]$pm25
        }
      }
    }
    for (iSpec in c('SNH4', 'SNO3')) {
      valOBS[[iMon]][[iSpec]]$pm10 = rawOBS[[iMon]][[iSpec]][['air+aerosol']]
    }
    
    for (iSpec in c('SO2', 'NH3', 'HNO3')) {
      valOBS[[iMon]][[iSpec]]$air = rawOBS[[iMon]][[iSpec]]$air
    }
    
    valOBS[[iMon]]$SOX$pm10 = valOBS[[iMon]]$SO4$pm10
    for (iStat in names(valOBS[[iMon]]$SOX$pm10)) {
      if ( is.null(valOBS[[iMon]]$SO2$air[[iStat]]) ) {
        valOBS[[iMon]]$SOX$pm10[[iStat]] = NULL
      } else {
        valOBS[[iMon]]$SOX$pm10[[iStat]]$timeser$values = valOBS[[iMon]]$SOX$pm10[[iStat]]$timeser$values + valOBS[[iMon]]$SO2$air[[iStat]]$timeser$values
        valOBS[[iMon]]$SOX$pm10[[iStat]]$substance = 'SOX'
      }
    }
    
  }
}


# read model data ----
if(read.sim) {
  
  rawSIM = list()
  for(iMon in MONs) { rawSIM[[iMon]] = list()
    for(iScen in SCENs) { rawSIM[[iMon]][[iScen]] = list()
      for(iSpec in allSPECs) { rawSIM[[iMon]][[iScen]][[iSpec]] = list() 
      }
    }
  }
  
  for(iMon in MONs) {
    
    if(iMon == 'sum2013' || iMon == 'sum') {
      from = 2013182; to = 2013243; nT = 62
    } else if(iMon == 'win2013' || iMon == 'win') {
      from = 2013001; to = 2013059; nT = 59
    }
    nTh = nT * 24
    
    
    #' METEO FILE +++++++++++++++++++++++++++++++++++++++++++
    print('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
    print('READING SIMULATION DATA FROM NETCDF FILES')
    print('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
    print('  READING MET FILE')
    ncMet <- nc_open(paste(inDir, paste('METCRO3D', iMon, 'short', sep = '.'), sep = '/'))
    
    nTSTEP = ncMet$dim$TSTEP$len
    nCOL = ncMet$dim$COL$len
    nROW = ncMet$dim$ROW$len
    
    valPRES = matrix(ncvar_get(ncMet, 'PRES')[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS)
    valTA   = matrix(ncvar_get(ncMet, 'TA')[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS)
    
    nc_close(ncMet)
    
    
    #' AEROSOL FILE +++++++++++++++++++++++++++++++++++++++++++
    print('  READING AERO FILE')
    inAero = paste('BL_AERODIAM', paste(GRID_NAME, GRID_NAME, iScen, mech, iMon, sep = '_'), 'short', sep = '.')
    ncAero = nc_open(paste(inDir, inAero, sep = '/'))
    
    valPM25AT = matrix(ncvar_get(ncAero, 'PM25AT')[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS)
    valPM25AC = matrix(ncvar_get(ncAero, 'PM25AC')[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS)
    valPM25CO = matrix(ncvar_get(ncAero, 'PM25CO')[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS)
    
    nc_close(ncAero)
    
    
    #' CONCENTRATION FILE +++++++++++++++++++++++++++++++++++++++++++
    myStart = c(1,1,1,1)
    myCount = c(-1,-1,-1,nTSTEP)
    
    for(iScen in SCENs) {
      print(paste('  READING netCDF files for scenario ', iScen, sep = ''))
      inConc = paste('BL_CONC', paste(GRID_NAME, GRID_NAME, iScen, mech, iMon, sep = '_'), 'short', sep = '.')
      ncId = nc_open(paste(inDir, inConc, sep = '/'))
      
      print('    Reading SNH4')
      tmpNH3 = matrix(ncvar_get(ncId, 'NH3', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) * valPRES / valTA / R * myMolMass$N
      tmpNH4I = matrix(ncvar_get(ncId, 'ANH4I', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) / myMolMass$ANH4I * myMolMass$N
      tmpNH4J = matrix(ncvar_get(ncId, 'ANH4J', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) / myMolMass$ANH4J * myMolMass$N
      tmpNH4K = matrix(ncvar_get(ncId, 'ANH4K', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) / myMolMass$ANH4K * myMolMass$N
      
      rawSIM[[iMon]][[iScen]]$NH3$air = tmpNH3
      rawSIM[[iMon]][[iScen]]$NH4$pm25 = valPM25AT * tmpNH4I + valPM25AC * tmpNH4J + valPM25CO * tmpNH4K
      rawSIM[[iMon]][[iScen]]$NH4$pm10 = tmpNH4I + tmpNH4J + tmpNH4K
      rawSIM[[iMon]][[iScen]]$SNH4$pm10 = tmpNH3 + tmpNH4I + tmpNH4J + tmpNH4K 
      
      print('    Reading SNO3')
      tmpHNO3 = matrix(ncvar_get(ncId, 'HNO3', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) * valPRES / valTA / R * myMolMass$N
      tmpNO3I = matrix(ncvar_get(ncId, 'ANO3I', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) / myMolMass$ANO3I * myMolMass$N
      tmpNO3J = matrix(ncvar_get(ncId, 'ANO3J', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) / myMolMass$ANO3J * myMolMass$N
      tmpNO3K = matrix(ncvar_get(ncId, 'ANO3K', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) / myMolMass$ANO3K * myMolMass$N
      
      rawSIM[[iMon]][[iScen]]$HNO3$air = tmpHNO3
      rawSIM[[iMon]][[iScen]]$NO3$pm25 = valPM25AT * tmpNO3I + valPM25AC * tmpNO3J + valPM25CO * tmpNO3K
      rawSIM[[iMon]][[iScen]]$NO3$pm10 = tmpNO3I + tmpNO3J + tmpNO3K
      rawSIM[[iMon]][[iScen]]$SNO3$pm10 = tmpHNO3 + tmpNO3I + tmpNO3J + tmpNO3K

            
      print('    Reading SO4')
      tmpSO2 = matrix(ncvar_get(ncId, 'SO2', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) * valPRES / valTA / R * myMolMass$S
      tmpSO4I = matrix(ncvar_get(ncId, 'ASO4I', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) / myMolMass$ASO4I * myMolMass$S
      tmpSO4J = matrix(ncvar_get(ncId, 'ASO4J', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) / myMolMass$ASO4J * myMolMass$S
      tmpSO4K = matrix(ncvar_get(ncId, 'ASO4K', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) / myMolMass$ASO4K * myMolMass$S
      
      rawSIM[[iMon]][[iScen]]$SO2$air = tmpSO2
      rawSIM[[iMon]][[iScen]]$SO4$pm25 = valPM25AT * tmpSO4I + valPM25AC * tmpSO4J + valPM25CO * tmpSO4K
      rawSIM[[iMon]][[iScen]]$SO4$pm10 = tmpSO4I + tmpSO4J + tmpSO4K
      rawSIM[[iMon]][[iScen]]$SOX$pm10 = tmpSO4I + tmpSO4J + tmpSO4K + tmpSO2
      
      print('    Reading Na')
      if (iScen %in% c('SALT_OV14')) { tmpNaI = matrix(ncvar_get(ncId, 'ANAI', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS)
      } else {                         tmpNaI = matrix(0.0, ncol = nSTATIONS, nrow = nTSTEP) }
      
      tmpNaJ = matrix(ncvar_get(ncId, 'ANAJ', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS)
      tmpNaK = matrix(ncvar_get(ncId, 'ASEACAT', start = myStart, count = myCount)[array(c(rep(t(array(unlist(myCells), dim = c(2,nSTATIONS))),each=nTSTEP),rep(1:nTSTEP, nSTATIONS)), dim = c(nTSTEP*nSTATIONS,3))], ncol=nSTATIONS) / ssSeaCatMass * ssNaMass
      
      rawSIM[[iMon]][[iScen]]$Na$pm25 = valPM25AT * tmpNaI + valPM25AC * tmpNaJ + valPM25CO * tmpNaK
      rawSIM[[iMon]][[iScen]]$Na$pm10 = tmpNaI + tmpNaJ + tmpNaK
      
      print('    Reading corr SO4')
      rawSIM[[iMon]][[iScen]]$XSO4$pm25 = rawSIM[[iMon]][[iScen]]$SO4$pm25 - rawSIM[[iMon]][[iScen]]$Na$pm25 / ssNaMass * ssSO4Mass / 3
      rawSIM[[iMon]][[iScen]]$XSO4$pm10 = rawSIM[[iMon]][[iScen]]$SO4$pm10 - rawSIM[[iMon]][[iScen]]$Na$pm10 / ssNaMass * ssSO4Mass / 3
      
      nc_close(ncId)
    }
  }
}



# reworking model ----
if (work.sim) {
  valSIM = list()
  
  for (iMon in MONs) {
    valSIM[[iMon]] = list()
    for (iScen in SCENs) {
      valSIM[[iMon]][[iScen]] = list()
      for (iSpec in allSPECs) {
        valSIM[[iMon]][[iScen]][[iSpec]] = list()
        theseMTRCs = names(rawSIM[[iMon]][[iScen]][[iSpec]])
        for (iMtrx in theseMTRCs) {
          thisSIM = rawSIM[[iMon]][[iScen]][[iSpec]][[iMtrx]]
          nDays = length(thisSIM[,1])/24
          
          tmpSIM = list()
          tmpSIM$unit = UNITS[[iSpec]]
          
          for (iLoc in 1:nSTATIONS) {
            iStat = myStations[iLoc]
            tmpSIM[[iStat]] = colMeans(array(thisSIM[,iLoc], dim = c(24,nDays)), na.rm = TRUE)
          }
          
          valSIM[[iMon]][[iScen]][[iSpec]][[iMtrx]] = tmpSIM
        }
      }
    }
  }
}

gc()

# plot A (time series, scenarios, overview) ----
if(plot.dat.A) {  
  
  yAxisLab = list('SNH4' = expression("NH"[4]^"+" ~ "+" ~ "NH"[3] ~ 'conc [' ~ mu * 'g N/' * m^3 * ']'),
                  'SNO3' = expression("NO"[3]^"-" ~ "+" ~ "HNO"[3] ~ 'conc [' ~ mu * 'g N/' * m^3 * ']'),
                  'SO4' = expression("SO"[4]^"2-" ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'SO2' = expression("SO"[2] ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'Na' = expression("Na"^"+" ~ 'conc [' ~ mu * 'g/' * m^3 * ']'),
                  'XSO4' = expression("corr. SO4"[4]^"2-" ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'NH4' = expression("NH"[4]^"+" ~ 'conc [' ~ mu * 'g N/' * m^3 * ']'),
                  'NO3' = expression("NO"[3]^"-" ~ 'conc [' ~ mu * 'g N/' * m^3 * ']'),
                  'NH3' = expression("NH"[3] ~ 'conc [' ~ mu * 'g N/' * m^3 * ']'),
                  'HNO3' = expression("HNO"[3] ~ 'conc [' ~ mu * 'g N/' * m^3 * ']'))
  
  # plot A: define screens ----
  ShlB = 0.07 # left
  ShrB = 0.992 # right
  ShdB = 0.035 # Abstand
  SvtB = 0.93 # top
  SvbB = 0.05 # bottom
  SvdB = 0.01 # Abstand
  ShBox = 0.06
  ShcB = ShlB+(ShrB-ShlB)*3/6
  
  for(iSpec in c(SPECs, 'SOX')) {  
  # for(iSpec in SPECs[c(7)]) {  
  # for(iSpec in SPECs[c(5,4,3,1,2)]) {  
    for (iMtrx in allMATRICEs[[iSpec]]) {
      # postscript(file = paste('CONC.timeseries', setup, iSpec, 'moreStations', 'eps', sep = '.'), onefile=TRUE,
      #            horizontal = FALSE, paper = 'special', width = 10, height = 6.5)
      # postscript(file = paste(fDirConc, paste('figZZ_CONC.timeseries', setup, iSpec, iMtrx, 'allStations', 'eps', sep = '.'), sep = '/'), onefile=TRUE,
      #            horizontal = FALSE, paper = 'special', width = 10, height = 6.5)
      pdf(file = paste(fDirConc, paste('figZZ_CONC.timeseries', setup, iSpec, iMtrx, 'allStations', 'pdf', sep = '.'), sep = '/'), onefile=TRUE,
          paper = 'special', width = 10, height = 6.5)
      # postscript(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'threeStations', 'eps', sep = '.'), onefile=TRUE,
      #           horizontal = FALSE, paper = 'special', width = 10, height = 6.5)
      par(mar = c(0,0,0,0)+0, 
          mgp = c(1.5,0.5,0.0))
      
      iScreen = 0
      
      for(iStat in myStations){
        
        if(iScreen%%13 == 0) {
          if(iScreen != 0) plot.new()
          iScreen = 1
          
          split.screen(rbind(c(ShlB,ShrB, SvtB+SvdB, 1.0-SvdB), # legend row
                             # top left, time series
                             c(ShlB, ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*2/3+SvdB/2, SvtB),
                             c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*2/3+SvdB/2, SvtB),
                             # top right, time series
                             c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*2/3+SvdB/2, SvtB),
                             c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*2/3+SvdB/2, SvtB),
                             # center left, time series
                             c(ShlB,ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*1/3+SvdB/2, SvbB+(SvtB-SvbB)*2/3-SvdB/2),
                             c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*1/3+SvdB/2, SvbB+(SvtB-SvbB)*2/3-SvdB/2),
                             # center right, time series
                             c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*1/3+SvdB/2, SvbB+(SvtB-SvbB)*2/3-SvdB/2),
                             c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*1/3+SvdB/2, SvbB+(SvtB-SvbB)*2/3-SvdB/2),
                             # bottom left, time series
                             c(ShlB,ShcB-ShdB/2-ShBox, SvbB, SvbB+(SvtB-SvbB)*1/3-SvdB/2),
                             c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB, SvbB+(SvtB-SvbB)*1/3-SvdB/2),
                             # bottom right, time series
                             c(ShcB+ShdB/2, ShrB-ShBox, SvbB, SvbB+(SvtB-SvbB)*1/3-SvdB/2),
                             c(ShrB-ShBox, ShrB, SvbB, SvbB+(SvtB-SvbB)*1/3-SvdB/2)
                             ))
          
          # plot A: make legend ----
          screen(iScreen)
          # plot(c(0,1), c(0,1), axes = FALSE, ann = FALSE, type = 'n')
          plot(c(0,1), c(0,1), xaxt = 'n', yaxt = 'n', ann = FALSE, type = 'n')
          if(iSpec == 'Na') { tCol = 'orange'; tPch = 8; tLab = 'NOENERGY'
          } else {            tCol = 'green';  tPch = 4; tLab = 'NOSALT'}
          
          if ( !(iSpec %in% c('NO3', 'NH4', 'HNO3', 'NH3')) ) {
            legend('left', legend = c('obs', 'SALT, GO03', 'NOSHIPS', 'NOSALT', 'NOENERGY', 'SALT, OV14', '  year 2013'),
                   col = c('darkgrey', 'blue', 'red', 'green', 'orange', 'magenta', 'white'), 
                   lwd = c(10,2,2,2,2,2,NA), 
                   lty = c('solid', NA, NA, NA, NA, NA, NA), pch = c(NA, 19, 1, 4, 8, 3, NA),
                   bg = 'white', horiz = TRUE, inset = 0.02,
                   pt.cex = 2, seg.len = 0.5, text.width = 0.12, bty = 'n')
          } else {
            legend('left', legend = c('', 'SALT, GO03', 'NOSHIPS', 'NOSALT', 'NOENERGY', 'SALT, OV14', '  year 2013'),
                   col = c('white', 'blue', 'red', 'green', 'orange', 'magenta', 'white'), 
                   lwd = c(10,2,2,2,2,2,NA), 
                   lty = c('solid', NA, NA, NA, NA, NA, NA), pch = c(NA, 19, 1, 4, 8, 3, NA),
                   bg = 'white', horiz = TRUE, inset = 0.02,
                   pt.cex = 2, seg.len = 0.5, text.width = 0.12, bty = 'n')
          }
        }
        
        myMax = 0
        for (iMon in MONs) {
          myMax = max(myMax, max(valOBS[[iMon]][[iSpec]][[iMtrx]][[iStat]]$timeser$values, na.rm = TRUE))
          for(iScen in SCENs) { myMax = max(myMax, max(valSIM[[iMon]][[iScen]][[iSpec]][[iMtrx]][[iStat]], na.rm = TRUE)) }
        }
        myLim = getYlim(myMax, 3)
        
        #for(iMon in c('win2013', 'win2013')) {
        for(iMon in MONs) {
          refDate=2013001
          strYear='2013'
          if(iMon == 'sum2013' || iMon == 'sum') {
            from = 2013182; to = 2013243; nT = 62
          } else if(iMon == 'win2013' || iMon == 'win') {
            from = 2013001; to = 2013059; nT = 59
          }
          nTh = nT * 24
          
          # plot A: prepare data ----
          timeAxis = as.POSIXct((0:(nT) + (from-refDate))*3600*24, tz = 'UTC', origin = "2008-01-01 00:00:00")
          # timeAxis = as.POSIXct((0:(nT-1) + (from-refDate))*3600*24, tz = 'UTC', origin = "2008-01-01 00:00:00")
          
          pltOBS = rep(0.0, nT)
          pltOBS = valOBS[[iMon]][[iSpec]][[iMtrx]][[iStat]]$timeser$values
          if ( is.null(pltOBS) || length(pltOBS) == 0) { pltOBS = rep(0.0, nT) }
        
          pltOBSnoNA = pltOBS
          pltOBSnoNA[is.na(pltOBS)] = 0
          
          # alternative: colMeans(array(ARRAY, dim = c(24,91)), na.rm = TRUE)
          pltSIM_ALL = valSIM[[iMon]][['ALLsurf']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_NOSH = valSIM[[iMon]][['NOSHIPSsurf']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_NOSA = valSIM[[iMon]][['NOSALT']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_NOEN = valSIM[[iMon]][['NOENERGY']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_OV14 = valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]]
          
          myLim = getYlim(max(c(pltOBS, pltSIM_ALL, pltSIM_NOSH, pltSIM_NOSA, pltSIM_NOEN), na.rm = TRUE), 3)
        
          # plot A: start plotting ----
          iScreen = iScreen + 1 
        
          screen(iScreen)
          plot(timeAxis[-nT-1], pltOBSnoNA,
               type = 'n', ann = FALSE, ylim = c(0,myLim),
               yaxt = 'n', xaxt = 'n')
        
          if ((iScreen-1)%%2 == 1) axis(2)
          # if(iScreen > 12) axis(1)
          
          yTicksWin = c(1199145600, 1201824000, 1204329600, 1207008000)
          yTicksSum = c(1215043200, 1217721600, 1220400000, 1222992000)
          if(iScreen %in% c(2, 6, 10)) axis(1,at = yTicksWin, labels = FALSE, tck = -0.03)
          if(iScreen == 10) axis(1,at = (yTicksWin[-1]+yTicksWin[-4])/2, tick = FALSE, labels = c('Jan', 'Feb', 'Mar'))
          if(iScreen %in% c(4, 8, 12)) axis(1,at = yTicksSum, labels = FALSE, tck = -0.03) # 1212278400, 1214870400, 1217548800, 1220227200
          if(iScreen == 12) axis(1,at = (yTicksSum[-1]+yTicksSum[-4])/2, tick = FALSE, labels = c('Jul', 'Aug', 'Sept'))
          # if((iScreen-1)%%2 == 0) mtext(yAxisLab[[iSpec]],2, line = 1)
          if(iScreen == 6) mtext(yAxisLab[[iSpec]],2, line = 1.6)
          if ((iScreen-1)%%4 == 3) title(paste(list_stations[[iStat]], iStat, sep = ' '), line = -1)
          
          if (iSpec != 'NO3') {
            polygon(rep(timeAxis, each = 2), append(append(rep(pltOBSnoNA, each = 2), 0, after = 0), 0),
                    col = 'darkgrey', border = FALSE)
          }
          
          # lines(rep(timeAxis, each = 2)[c(-1, -2*nT-2)], rep(pltSIM_ALL, each = 2),
          #       col = 'blue', lwd = 1)
          # lines(rep(timeAxis, each = 2)[c(-1, -2*nT-2)], rep(pltSIM_NOSH, each = 2),
          #       col = 'red', lwd = 1)
          points(colMeans(rbind(timeAxis[-1],timeAxis[-nT-1])), pltSIM_ALL,
                col = 'blue', pch = 19, cex = 0.8)
          points(colMeans(rbind(timeAxis[-1],timeAxis[-nT-1])), pltSIM_NOSH,
                col = 'red', pch = 1, cex = 0.8)
          # if (iSpec == 'Na') {
            points(colMeans(rbind(timeAxis[-1],timeAxis[-nT-1])), pltSIM_NOEN,
                  col = 'orange', pch = 8, cex = 0.7)
          # } else {
            points(colMeans(rbind(timeAxis[-1],timeAxis[-nT-1])), pltSIM_NOSA,
                  col = 'green', pch = 4, cex = 0.8)
          # }
            points(colMeans(rbind(timeAxis[-1],timeAxis[-nT-1])), pltSIM_OV14,
                   col = 'magenta', pch = 3, cex = 0.8)
          
          iScreen = iScreen + 1 
          screen(iScreen)
          # NOTE: [,!is.na(pltOBS)] makes that only values are considered which
          #       are available as observations and measurements.
          if (iSpec == 'Na') {
            boxplot(apply(rbind(pltOBS, pltSIM_ALL, pltSIM_NOSH, pltSIM_NOEN)[,!is.na(pltOBS)], MARGIN = 1, FUN = quantile, probs = c(0, 0.25, 0.5, 0.75, 1.0), na.rm = TRUE),
                    yaxt = 'n', xaxt = 'n', xlab = '', ylab = '', ylim = c(0,myLim), border = c('black', 'blue', 'red', 'orange'), range = 0)
            points(1:4, apply(rbind(pltOBS, pltSIM_ALL, pltSIM_NOSH, pltSIM_NOEN), MARGIN = 1, FUN = mean, na.rm = TRUE),
                   pch = 18, col = c('black', 'blue', 'red', 'orange'))
          } else if (iSpec != 'NO3'){
            boxplot(apply(rbind(pltOBS, pltSIM_ALL, pltSIM_NOSH, pltSIM_NOSA)[,!is.na(pltOBS)], MARGIN = 1, FUN = quantile, probs = c(0, 0.25, 0.5, 0.75, 1.0), na.rm = TRUE),
                    yaxt = 'n', xaxt = 'n', xlab = '', ylab = '', ylim = c(0,myLim), border = c('black', 'blue', 'red', 'green'), range = 0)
            points(1:4, apply(rbind(pltOBS, pltSIM_ALL, pltSIM_NOSH, pltSIM_NOSA), MARGIN = 1, FUN = mean, na.rm = TRUE),
                   pch = 18, col = c('black', 'blue', 'red', 'green'))
          } else {
            boxplot(apply(rbind(rep(-100,length(pltOBS)), pltSIM_ALL, pltSIM_NOSH, pltSIM_NOSA), MARGIN = 1, FUN = quantile, probs = c(0, 0.25, 0.5, 0.75, 1.0), na.rm = TRUE),
                    yaxt = 'n', xaxt = 'n', xlab = '', ylab = '', ylim = c(0,myLim), border = c('black', 'blue', 'red', 'green'), range = 0)
            points(1:4, apply(rbind(rep(-100,length(pltOBS)), pltSIM_ALL, pltSIM_NOSH, pltSIM_NOSA), MARGIN = 1, FUN = mean, na.rm = TRUE),
                   pch = 18, col = c('black', 'blue', 'red', 'green'))
          }
          
        }
      }
      close.screen(all.screens = TRUE)
      dev.off()
    }
  }
  
}




# plot B (time series, bar plots, overview) ----
if(plot.dat.B) {  
  
  yAxisLab = list('SNH4' = expression("NH"[4]^"+" ~ "+" ~ "NH"[3] ~ 'conc [' ~ mu * 'g N/' * m^3 * ']'),
                  'SNO3' = expression("NO"[3]^"-" ~ "+" ~ "HNO"[3] ~ 'conc [' ~ mu * 'g N/' * m^3 * ']'),
                  'SO4' = expression("SO"[4]^"2-" ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'SO2' = expression("SO"[2] ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'Na' = expression("Na"^"+" ~ 'conc [' ~ mu * 'g/' * m^3 * ']'),
                  'XSO4' = expression("corr. SO4"[4]^"2-" ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'NH4' = expression("NH"[4]^"+" ~ 'conc [' ~ mu * 'g N/' * m^3 * ']'),
                  'NO3' = expression("NO"[3]^"-" ~ 'conc [' ~ mu * 'g N/' * m^3 * ']'),
                  'NH3' = expression("NH"[3] ~ 'conc [' ~ mu * 'g N/' * m^3 * ']'),
                  'HNO3' = expression("HNO"[3] ~ 'conc [' ~ mu * 'g N/' * m^3 * ']'))
  
  # plot B: define screens ----
  ShlB = 0.07 # left
  ShrB = 0.992 # right
  ShdB = 0.035 # Abstand
  SvtB = 0.93 # top
  SvbB = 0.05 # bottom
  SvdB = 0.01 # Abstand
  ShBox = 0.06
  ShcB = ShlB+(ShrB-ShlB)*3/6
  
  for(iSpec in c(SPECs, 'SOX')) {  
    # for(iSpec in SPECs[c(7)]) {  
    # for(iSpec in SPECs[c(5,4,3,1,2)]) {  
    for (iMtrx in allMATRICEs[[iSpec]]) {
      # postscript(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'moreStations', 'eps', sep = '.'), onefile=TRUE,
      #            horizontal = FALSE, paper = 'special', width = 10, height = 6.5)
      # postscript(file = paste(fDirConc, paste('figZZ_CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'allStations', 'eps', sep = '.'), sep = '/'), onefile=TRUE,
      #           horizontal = FALSE, paper = 'special', width = 10, height = 6.5)
      pdf(file = paste(fDirConc, paste('figZZ_CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'allStations', 'pdf', sep = '.'), sep = '/'), onefile=TRUE,
          paper = 'special', width = 10, height = 6.5)
      # postscript(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'threeStations', 'eps', sep = '.'), onefile=TRUE,
      #           horizontal = FALSE, paper = 'special', width = 10, height = 6.5)
      # pdf(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'threeStations', 'pdf', sep = '.'), 
      #            paper = 'special', width = 10, height = 6.5)
      par(mar = c(0,0,0,0)+0, 
          mgp = c(1.5,0.5,0.0))
      
      iScreen = 0
      
      for(iStat in myStations){
      # for(iStat in myStations[c(3,5,6)]){
        
        if ( iSpec %in% c('NO3', 'NH4', 'HNO3', 'NH3') ) { showOBS = FALSE
        } else { showOBS = TRUE }
        
        if(iScreen%%13 == 0) {
          if(iScreen != 0) plot.new()
          iScreen = 1
          
          split.screen(rbind(c(ShlB,ShrB, SvtB+SvdB, 1.0-SvdB), # legend row
                             # top left, time series
                             c(ShlB, ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*2/3+SvdB/2, SvtB),
                             c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*2/3+SvdB/2, SvtB),
                             # top right, time series
                             c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*2/3+SvdB/2, SvtB),
                             c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*2/3+SvdB/2, SvtB),
                             # center left, time series
                             c(ShlB,ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*1/3+SvdB/2, SvbB+(SvtB-SvbB)*2/3-SvdB/2),
                             c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*1/3+SvdB/2, SvbB+(SvtB-SvbB)*2/3-SvdB/2),
                             # center right, time series
                             c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*1/3+SvdB/2, SvbB+(SvtB-SvbB)*2/3-SvdB/2),
                             c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*1/3+SvdB/2, SvbB+(SvtB-SvbB)*2/3-SvdB/2),
                             # bottom left, time series
                             c(ShlB,ShcB-ShdB/2-ShBox, SvbB, SvbB+(SvtB-SvbB)*1/3-SvdB/2),
                             c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB, SvbB+(SvtB-SvbB)*1/3-SvdB/2),
                             # bottom right, time series
                             c(ShcB+ShdB/2, ShrB-ShBox, SvbB, SvbB+(SvtB-SvbB)*1/3-SvdB/2),
                             c(ShrB-ShBox, ShrB, SvbB, SvbB+(SvtB-SvbB)*1/3-SvdB/2)
          ))
          
          # plot B: make legend ----
          screen(iScreen)
          # plot(c(0,1), c(0,1), axes = FALSE, ann = FALSE, type = 'n')
          plot(c(0,1), c(0,1), xaxt = 'n', yaxt = 'n', ann = FALSE, type = 'n')
          
          if ( showOBS ) {
            # legend('left', legend = c('obs', 'SALT, GO03', 'SALT, OV14', 'SHIPPING', 'ENERGY', 'UNSPEC.', '  year 2013'),
            legend('left', legend = c('obs', 'SALT, GO03', 'SALT, OV14', 'SHIPPING', 'ENERGY', 'UNSPEC.', '  year 2013'),
                   col = c('black', 'blue', 'magenta', 'green', 'red', 'orange', 'white'),
                   lwd = c(3,NA,NA,NA,NA,NA,NA), 
                   pch = c(NA,15,15,15,15,15,NA),
                   lty = c('solid', 'solid', 'solid', 'solid', 'solid', 'solid', NA),
                   bg = 'white', horiz = TRUE, inset = 0.02,
                   pt.cex = 2, seg.len = 0.5, text.width = 0.12, bty = 'n')
          } else {
            # legend('left', legend = c('','SALT, GO03', 'SALT, OV14', 'SHIPPING', 'ENERGY', 'UNSPEC.', '  year 2013'),
            legend('left', legend = c('', 'SALT, GO03', 'SALT, OV14', 'SHIPPING', 'ENERGY', 'UNSPEC.', '  year 2013'),
                   col = c('black', 'blue', 'magenta', 'green', 'red', 'orange', 'white'),
                   lwd = c(NA,NA,NA,NA,NA,NA,NA), 
                   pch = c(NA,15,15,15,15,15,NA),
                   lty = c('solid', 'solid', 'solid', 'solid', 'solid', 'solid', NA),
                   bg = 'white', horiz = TRUE, inset = 0.02,
                   pt.cex = 2, seg.len = 0.5, text.width = 0.12, bty = 'n')
          }
        }
        
        myMax = 0
        for (iMon in MONs) {
          myMax = max(myMax, max(valOBS[[iMon]][[iSpec]][[iMtrx]][[iStat]]$timeser$value, na.rm = TRUE))
          for(iScen in SCENs) { myMax = max(myMax, max(valSIM[[iMon]][[iScen]][[iSpec]][[iMtrx]][[iStat]], na.rm = TRUE)) }
        }
        myLim = getYlim(myMax, 3)
        
        #for(iMon in c('win2013', 'win2013')) {
        for(iMon in MONs) {
          refDate=2013001
          strYear='2013'
          if(iMon == 'sum2013' || iMon == 'sum') {
            from = 2013182; to = 2013243; nT = 62
          } else if(iMon == 'win2013' || iMon == 'win') {
            from = 2013001; to = 2013059; nT = 59
          }
          nTh = nT * 24
          
          
          # plot B: prepare data ----
          timeAxis = as.POSIXct((0:(nT) + (from-refDate))*3600*24, tz = 'UTC', origin = "2008-01-01 00:00:00")
          # timeAxis = as.POSIXct((0:(nT-1) + (from-refDate))*3600*24, tz = 'UTC', origin = "2008-01-01 00:00:00")
          timeMean = colMeans(rbind(timeAxis[-1],timeAxis[-nT-1]))
          
          pltOBS = rep(NA, nT)
          pltOBS = valOBS[[iMon]][[iSpec]][[iMtrx]][[iStat]]$timeser$value
          if ( is.null(pltOBS) || length(pltOBS) == 0) { pltOBS = rep(NA, nT) }
          
          pltOBSnoNA = pltOBS
          pltOBSnoNA[is.na(pltOBS)] = 0
          
          # alternative: colMeans(array(ARRAY, dim = c(24,91)), na.rm = TRUE)
          pltSIM_ALL = valSIM[[iMon]][['ALLsurf']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_NOSH = valSIM[[iMon]][['NOSHIPSsurf']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_NOSA = valSIM[[iMon]][['NOSALT']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_NOEN = valSIM[[iMon]][['NOENERGY']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_OV14 = valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]]
          
          pltSIM_SHIP = pltSIM_ALL - pltSIM_NOSH
          pltSIM_ENER = pltSIM_ALL - pltSIM_NOEN
          if (iSpec == 'SO4') {
            pltSIM_SALT_GO03 = valSIM[[iMon]][['ALLsurf']][['Na']][[iMtrx]][[iStat]] / ssNaMass * ssSO4Mass / 3
            pltSIM_SALT_OV14 = valSIM[[iMon]][['SALT_OV14']][['Na']][[iMtrx]][[iStat]] / ssNaMass * ssSO4Mass / 3
          } else {
            pltSIM_SALT_GO03 = pltSIM_ALL - pltSIM_NOSA
            pltSIM_SALT_OV14 = pltSIM_OV14 - pltSIM_NOSA
          }
          pltSIM_OTHR = pltSIM_ALL - pltSIM_SHIP - pltSIM_ENER - pltSIM_SALT_GO03
          
          myLim = getYlim(max(c(pltOBS, pltSIM_ALL, pltSIM_NOSH, pltSIM_NOSA, pltSIM_NOEN, 0.0), na.rm = TRUE), 3)
          
          # plot B: start plotting ----
          iScreen = iScreen + 1 
          
          screen(iScreen)
          plot(timeAxis[-nT-1], pltOBSnoNA,
               type = 'n', ann = FALSE,
               ylim = c(0,myLim),
               yaxt = 'n', xaxt = 'n')
          
          if ((iScreen-1)%%2 == 1) axis(2)
          # if(iScreen > 12) axis(1)
          
          yTicksWin = c(1199145600, 1201824000, 1204329600, 1207008000)
          yTicksSum = c(1215043200, 1217721600, 1220400000, 1222992000)
          if(iScreen %in% c(2, 6, 10)) axis(1,at = yTicksWin, labels = FALSE, tck = -0.03)
          if(iScreen == 10) axis(1,at = (yTicksWin[-1]+yTicksWin[-4])/2, tick = FALSE, labels = c('Jan', 'Feb', 'Mar'))
          if(iScreen %in% c(4, 8, 12)) axis(1,at = yTicksSum, labels = FALSE, tck = -0.03) # 1212278400, 1214870400, 1217548800, 1220227200
          if(iScreen == 12) axis(1,at = (yTicksSum[-1]+yTicksSum[-4])/2, tick = FALSE, labels = c('Jul', 'Aug', 'Sept'))
          # if((iScreen-1)%%2 == 0) mtext(yAxisLab[[iSpec]],2, line = 1)
          if(iScreen == 6) mtext(yAxisLab[[iSpec]],2, line = 1.6)
          if ((iScreen-1)%%4 == 3) title(paste(list_stations[[iStat]], iStat, sep = ' '), line = -1)
          
          polygon(rbind(c(0.0,timeMean), c(0.0,timeMean), timeAxis,timeAxis)[-(1:2)], 
                  c(0, rbind(rep(0.0, length(pltSIM_OV14)),rep(0.0, length(pltSIM_OV14)), pltSIM_OV14, pltSIM_OV14), 0),
                  col = 'magenta', border = FALSE)
          polygon(rbind(c(0.0,timeMean), c(0.0,timeMean), timeAxis,timeAxis)[-(1:2)],
                  c(0, rbind(pltSIM_ALL, pltSIM_ALL, rep(0.0, length(pltSIM_ALL)),rep(0.0, length(pltSIM_ALL))), 0),
                  col = 'blue', border = FALSE)
          polygon(rep(timeAxis, each = 2), c(0.0, rep(pltSIM_ENER + pltSIM_OTHR + pltSIM_SHIP, each = 2), 0.0),
                 col = 'green', border = FALSE)
          polygon(rep(timeAxis, each = 2), c(0.0, rep(pltSIM_ENER + pltSIM_OTHR, each = 2), 0.0),
                 col = 'red', border = FALSE)
          polygon(rep(timeAxis, each = 2), c(0.0, rep(pltSIM_OTHR, each = 2), 0.0),
                 col = 'orange', border = FALSE)
          
          if ( showOBS ) {
            lines(rep(timeMean, each = 3), c(rbind(rep(0.0,length(pltOBS)), pltOBS, rep(NA,length(pltOBS)))),
                  col = 'black')
            lines(rep(timeAxis, each = 3)[-c(1,2,length(timeAxis)*3)], c(rbind(pltOBS, pltOBS, rep(NA,length(pltOBS)))),
                  col = 'black')
            lines(rep(timeAxis, each = 3)[-c(1,2,length(timeAxis)*3)], c(rbind(pltOBS*0.0, pltOBS*0.0, rep(NA,length(pltOBS)))),
                  col = 'black')
          }
          
          
          # plot B: make mean plot ----
          iScreen = iScreen + 1 
          screen(iScreen)
          # NOTE: [,!is.na(pltOBS)] makes that only values are considered which
          #       are available as observations and measurements.
          plot(c(0.0,0.0), c(0.0,0.0),
               type = 'n', ann = FALSE, 
               ylim = c(0,myLim), xlim = c(0,3),
               yaxt = 'n', xaxt = 'n')
          
          meanSALT_GO03_all = mean(pltSIM_SALT_GO03)
          meanSALT_GO03_obs = mean(pltSIM_SALT_GO03[!is.na(pltOBS)])
          meanSALT_OV14_all = mean(pltSIM_SALT_OV14)
          meanSALT_OV14_obs = mean(pltSIM_SALT_OV14[!is.na(pltOBS)])
          meanSHIP_all = mean(pltSIM_SHIP)
          meanSHIP_obs = mean(pltSIM_SHIP[!is.na(pltOBS)])
          meanENER_all = mean(pltSIM_ENER)
          meanENER_obs = mean(pltSIM_ENER[!is.na(pltOBS)])
          meanOTHR_all = mean(pltSIM_OTHR)
          meanOTHR_obs = mean(pltSIM_OTHR[!is.na(pltOBS)])
          meanOBS = mean(pltOBS[!is.na(pltOBS)])
          
          if ( showOBS ) {
            polygon(c(0.8,0.8,1.3,1.3),
                    c(0.0,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_OV14_obs,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_OV14_obs,0.0),
                    col = 'magenta', border = FALSE)
            polygon(c(0.3,0.3,0.8,0.8),
                    c(0.0,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_GO03_obs,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_GO03_obs,0.0),
                    col = 'blue', border = FALSE)
            polygon(c(0.3,0.3,1.3,1.3),
                    c(0.0,meanOTHR_obs+meanENER_obs+meanSHIP_obs,meanOTHR_obs+meanENER_obs+meanSHIP_obs,0.0),
                    col = 'green', border = FALSE)
            polygon(c(0.3,0.3,1.3,1.3),
                    c(0.0,meanOTHR_obs+meanENER_obs,meanOTHR_obs+meanENER_obs,0.0),
                    col = 'red', border = FALSE)
            polygon(c(0.3,0.3,1.3,1.3),
                    c(0.0,meanOTHR_obs,meanOTHR_obs,0.0),
                    col = 'orange', border = FALSE)
            lines(c(0.8,0.8),c(0.0, meanOBS), col = 'black')
            lines(c(0.3,1.3), c(meanOBS, meanOBS), col = 'black')
          }
          polygon(c(2.2,2.2,2.7,2.7),
                  c(0.0,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_OV14_all,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_OV14_all,0.0),
                  col = 'magenta', border = FALSE)
          polygon(c(1.7,1.7,2.2,2.2),
                  c(0.0,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_GO03_all,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_GO03_all,0.0),
                  col = 'blue', border = FALSE)
          polygon(c(1.7,1.7,2.7,2.7),
                  c(0.0,meanOTHR_all+meanENER_all+meanSHIP_all,meanOTHR_all+meanENER_all+meanSHIP_all,0.0),
                  col = 'green', border = FALSE)
          polygon(c(1.7,1.7,2.7,2.7),
                  c(0.0,meanOTHR_all+meanENER_all,meanOTHR_all+meanENER_all,0.0),
                  col = 'red', border = FALSE)
          polygon(c(1.7,1.7,2.7,2.7),
                  c(0.0,meanOTHR_all,meanOTHR_all,0.0),
                  col = 'orange', border = FALSE)
          
        }
      }
      close.screen(all.screens = TRUE)
      dev.off()
    }
  }
  
}


# plot C (abstract EAC2016) ----
if(plot.dat.C) {  
  
  yAxisLab = expression("SO"[4]^"2-" ~ 'conc [' ~ mu * 'g S/' * m^3 * ']')
  
  # plot C: define screens ----
  ShlB = 0.134 # left
  ShrB = 0.985 # right
  ShdB = 0.067 # Abstand
  SvtB = 0.828 # top      0.455
  SvbB = 0.065 # bottom   0.325
  SvdB = 0.013 # Abstand  0.065
  # plot: 1.874166
  ShBox = 0.115
  
  iSpec = 'SO4'
  iMtrx = 'pm10'
  iMon = 'sum2013'
  
  refDate=2013001
  strYear='2013'
  if(iMon == 'sum2013' || iMon == 'sum') {
    from = 2013182; to = 2013243; nT = 62
  } else if(iMon == 'win2013' || iMon == 'win') {
    from = 2013001; to = 2013059; nT = 59
  }
  nTh = nT * 24
  
  
  # postscript(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'moreStations', 'eps', sep = '.'), onefile=TRUE,
  #            horizontal = FALSE, paper = 'special', width = 10, height = 6.5)
  postscript(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'twoStations', 'eps', sep = '.'), onefile=TRUE,
             horizontal = FALSE, paper = 'special', width = 5.215, height = 5)
  # 5.215 = 10*(ShlB+1-ShrB+(-ShlB+ShrB-ShdB)/2) (alte Werte)
  # 5.215 = 10*(ShlB+1-ShrB+(-ShlB+ShrB-ShdB)/2)
  # postscript(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'threeStations', 'eps', sep = '.'), onefile=TRUE,
  #           horizontal = FALSE, paper = 'special', width = 10, height = 6.5)
  # pdf(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'threeStations', 'pdf', sep = '.'), 
  #            paper = 'special', width = 10, height = 6.5)
  par(mar = c(0,0,0,0)+0, 
      mgp = c(1.5,0.5,0.0))
  
  iScreen = 0
  showOBS = TRUE
  
  if(iScreen != 0) plot.new()
  iScreen = 1
  split.screen(rbind(c(ShlB,ShrB, SvtB+SvdB, 1.0-SvdB), # legend row
                     # top right, time series
                     c(ShlB, ShrB-ShBox, SvbB+(SvtB-SvbB)/2+SvdB/2, SvtB),
                     c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)/2+SvdB/2, SvtB),
                     # bottom right, time series
                     c(ShlB, ShrB-ShBox, SvbB, SvbB+(SvtB-SvbB)*1/2-SvdB/2),
                     c(ShrB-ShBox, ShrB, SvbB, SvbB+(SvtB-SvbB)*1/2-SvdB/2),
                     c(ShlB*1/5, ShlB, SvbB, SvtB)
  ))
  
  # plot C: make legend ----
  screen(iScreen)
  plot(c(0,1), c(0,1), xaxt = 'n', yaxt = 'n', ann = FALSE, type = 'n')
  
  legend('topleft', legend = c('obs', 'SALT, GO03', 'SALT, OV14'),
         col = c('black', 'blue', 'magenta'),
         lwd = c(3,NA,NA), 
         lty = c('solid', NA,NA), pch = c(NA,15,15),
         bg = 'white', inset = 0.035, horiz = TRUE,  # ncol = 3, # horiz = TRUE, 
         pt.cex = 2, seg.len = 0.5, text.width = 0.3, bty = 'n')
  legend('bottomleft', legend = c('SHIPPING', 'ENERGY', 'UNSPEC.'),
         col = c('green', 'red', 'orange'),
         lwd = NA, 
         lty = NA, pch = 15,
         bg = 'white', inset = 0.035, horiz = TRUE, # ncol = 3, # 
         pt.cex = 2, seg.len = 0.5, text.width = 0.3, bty = 'n')
  
  for(iStat in myStations[c(1,5)]){
    # plot C: prepare data ----
    timeAxis = as.POSIXct((0:(nT) + (from-refDate))*3600*24, tz = 'UTC', origin = "2008-01-01 00:00:00")
    # timeAxis = as.POSIXct((0:(nT-1) + (from-refDate))*3600*24, tz = 'UTC', origin = "2008-01-01 00:00:00")
    timeMean = colMeans(rbind(timeAxis[-1],timeAxis[-nT-1]))
    
    pltOBS = rep(NA, nT)
    pltOBS = valOBS[[iMon]][[iSpec]][[iMtrx]][[iStat]]$timeser$value
    if ( is.null(pltOBS) || length(pltOBS) == 0) { pltOBS = rep(NA, nT) }
    
    pltOBSnoNA = pltOBS
    pltOBSnoNA[is.na(pltOBS)] = 0
    
    # alternative: colMeans(array(ARRAY, dim = c(24,91)), na.rm = TRUE)
    pltSIM_ALL = valSIM[[iMon]][['ALLsurf']][[iSpec]][[iMtrx]][[iStat]]
    pltSIM_NOSH = valSIM[[iMon]][['NOSHIPSsurf']][[iSpec]][[iMtrx]][[iStat]]
    pltSIM_NOSA = valSIM[[iMon]][['NOSALT']][[iSpec]][[iMtrx]][[iStat]]
    pltSIM_NOEN = valSIM[[iMon]][['NOENERGY']][[iSpec]][[iMtrx]][[iStat]]
    pltSIM_OV14 = valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]]
    
    pltSIM_SHIP = pltSIM_ALL - pltSIM_NOSH
    pltSIM_ENER = pltSIM_ALL - pltSIM_NOEN
    pltSIM_SALT_GO03 = pltSIM_ALL - pltSIM_NOSA
    pltSIM_OTHR = pltSIM_ALL - pltSIM_SHIP - pltSIM_ENER - pltSIM_SALT_GO03
    pltSIM_SALT_OV14 = pltSIM_OV14 - pltSIM_NOSA
    
    # myLim = getYlim(max(c(pltOBS, pltSIM_ALL, pltSIM_NOSH, pltSIM_NOSA, pltSIM_NOEN, 0.0), na.rm = TRUE), 3)
    myLim = 1.4
    
    # plot C: start plotting ----
    iScreen = iScreen + 1 
    
    screen(iScreen)
    plot(timeAxis[-nT-1], pltOBSnoNA,
         type = 'n', ann = FALSE,
         ylim = c(0,myLim),
         yaxt = 'n', xaxt = 'n')
    
    if ((iScreen-1)%%2 == 1) axis(2)
    # if(iScreen > 12) axis(1)
    
    yTicksWin = c(1199145600, 1201824000, 1204329600, 1207008000)
    yTicksSum = c(1215043200, 1217721600, 1220400000, 1222992000)
    # if(iScreen %in% c(2, 6, 10)) axis(1,at = yTicksWin, labels = FALSE, tck = -0.03)
    # if(iScreen == 10) axis(1,at = (yTicksWin[-1]+yTicksWin[-4])/2, tick = FALSE, labels = c('Jan', 'Feb', 'Mar'))
    if(iScreen %in% c(2, 4)) axis(1,at = yTicksSum, labels = FALSE, tck = -0.03) # 1212278400, 1214870400, 1217548800, 1220227200
    if(iScreen == 4) axis(1,at = (yTicksSum[-1]+yTicksSum[-4])/2, tick = FALSE, labels = c('Jul 2013', 'Aug 2013', 'Sept 2013'))
    # if((iScreen-1)%%2 == 0) mtext(yAxisLab[[iSpec]],2, line = 1)
    if(iScreen == 6) mtext(yAxisLab[[iSpec]],2, line = 1.6)
    if ((iScreen-1)%%2 == 1) title(paste(valOBS[['win2013']][['SO4']]$pm10[[iStat]]$station$station_name, ' (', iStat, ')', sep = ''), line = -1)
    
    polygon(rbind(c(0.0,timeMean), c(0.0,timeMean), timeAxis,timeAxis)[-(1:2)], 
            c(0, rbind(rep(0.0, length(pltSIM_OV14)),rep(0.0, length(pltSIM_OV14)), pltSIM_OV14, pltSIM_OV14), 0),
            col = 'magenta', border = FALSE)
    polygon(rbind(c(0.0,timeMean), c(0.0,timeMean), timeAxis,timeAxis)[-(1:2)],
            c(0, rbind(pltSIM_ALL, pltSIM_ALL, rep(0.0, length(pltSIM_ALL)),rep(0.0, length(pltSIM_ALL))), 0),
            col = 'blue', border = FALSE)
    polygon(rep(timeAxis, each = 2), c(0.0, rep(pltSIM_ENER + pltSIM_OTHR + pltSIM_SHIP, each = 2), 0.0),
            col = 'green', border = FALSE)
    polygon(rep(timeAxis, each = 2), c(0.0, rep(pltSIM_ENER + pltSIM_OTHR, each = 2), 0.0),
            col = 'red', border = FALSE)
    polygon(rep(timeAxis, each = 2), c(0.0, rep(pltSIM_OTHR, each = 2), 0.0),
            col = 'orange', border = FALSE)
    
    if ( showOBS ) {
      lines(rep(timeMean, each = 3), c(rbind(rep(0.0,length(pltOBS)), pltOBS, rep(NA,length(pltOBS)))),
           col = 'black')
      lines(rep(timeAxis, each = 3)[-c(1,2,length(timeAxis)*3)], c(rbind(pltOBS, pltOBS, rep(NA,length(pltOBS)))),
           col = 'black')
      lines(rep(timeAxis, each = 3)[-c(1,2,length(timeAxis)*3)], c(rbind(pltOBS*0.0, pltOBS*0.0, rep(NA,length(pltOBS)))),
           col = 'black')
    }
    # points(timeMean, pltOBS, col = 'black', pch = 3)
    
    
    
    # plot C: make mean plot ----
    iScreen = iScreen + 1 
    screen(iScreen)
    # NOTE: [,!is.na(pltOBS)] makes that only values are considered which
    #       are available as observations and measurements.
    plot(c(0.0,0.0), c(0.0,0.0),
         type = 'n', ann = FALSE, 
         # ylim = c(0,myLim), xlim = c(0,3),
         ylim = c(0,myLim), xlim = c(0,1.8),
         yaxt = 'n', xaxt = 'n')
    
    meanSALT_GO03_all = mean(pltSIM_SALT_GO03)
    meanSALT_GO03_obs = mean(pltSIM_SALT_GO03[!is.na(pltOBS)])
    meanSALT_OV14_all = mean(pltSIM_SALT_OV14)
    meanSALT_OV14_obs = mean(pltSIM_SALT_OV14[!is.na(pltOBS)])
    meanSHIP_all = mean(pltSIM_SHIP)
    meanSHIP_obs = mean(pltSIM_SHIP[!is.na(pltOBS)])
    meanENER_all = mean(pltSIM_ENER)
    meanENER_obs = mean(pltSIM_ENER[!is.na(pltOBS)])
    meanOTHR_all = mean(pltSIM_OTHR)
    meanOTHR_obs = mean(pltSIM_OTHR[!is.na(pltOBS)])
    meanOBS = mean(pltOBS[!is.na(pltOBS)])
    
    if ( showOBS ) {
      polygon(c(0.8,0.8,1.3,1.3),
              c(0.0,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_OV14_obs,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_OV14_obs,0.0),
              col = 'magenta', border = FALSE)
      polygon(c(0.3,0.3,0.8,0.8),
              c(0.0,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_GO03_obs,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_GO03_obs,0.0),
              col = 'blue', border = FALSE)
      polygon(c(0.3,0.3,1.3,1.3),
              c(0.0,meanOTHR_obs+meanENER_obs+meanSHIP_obs,meanOTHR_obs+meanENER_obs+meanSHIP_obs,0.0),
              col = 'green', border = FALSE)
      polygon(c(0.3,0.3,1.3,1.3),
              c(0.0,meanOTHR_obs+meanENER_obs,meanOTHR_obs+meanENER_obs,0.0),
              col = 'red', border = FALSE)
      polygon(c(0.3,0.3,1.3,1.3),
              c(0.0,meanOTHR_obs,meanOTHR_obs,0.0),
              col = 'orange', border = FALSE)
      lines(c(0.8,0.8),c(0.0, meanOBS), col = 'black')
      lines(c(0.3,1.3), c(meanOBS, meanOBS), col = 'black')
    }
#     polygon(c(2.2,2.2,2.7,2.7),
#             c(0.0,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_OV14_all,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_OV14_all,0.0),
#             col = 'magenta', border = FALSE)
#     polygon(c(1.7,1.7,2.2,2.2),
#             c(0.0,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_GO03_all,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_GO03_all,0.0),
#             col = 'blue', border = FALSE)
#     polygon(c(1.7,1.7,2.7,2.7),
#             c(0.0,meanOTHR_all+meanENER_all+meanSHIP_all,meanOTHR_all+meanENER_all+meanSHIP_all,0.0),
#             col = 'green', border = FALSE)
#     polygon(c(1.7,1.7,2.7,2.7),
#             c(0.0,meanOTHR_all+meanENER_all,meanOTHR_all+meanENER_all,0.0),
#             col = 'red', border = FALSE)
#     polygon(c(1.7,1.7,2.7,2.7),
#             c(0.0,meanOTHR_all,meanOTHR_all,0.0),
#             col = 'orange', border = FALSE)
   
    if (iScreen == 5) mtext('mean', 1, line = 0.5)
    # mtext('mean: only model\n vals considered\n where obs. exist',2, line = -2.5, at = 1.0, cex = 0.8)
    # mtext('all model vals',4, line = -1,  at = 1.0)
     
  }
  
  screen(6)
  plot(c(-1,2), c(-1,2), axes = FALSE, xlim = c(0,1), ylim = c(0,1))
  mtext(yAxisLab, side = 2, line = -1)
  
  close.screen(all.screens = TRUE)
  dev.off()
}


# plot D (six stations, bar, PhD Thesis) ----
if(plot.dat.D) {  
  
  yAxisLab = list('SO4' = expression("SO"[4]^"2-" ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'SO4pm10' = expression("SO"[4]^"2-" ~ "PM"[10] ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'SO4pm25' = expression("SO"[4]^"2-" ~ "PM"[2.5] ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'SOX' = expression("SO"[x] ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'SO2' = expression("SO"[2] ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'Na' = expression("Na"^"+" ~ 'conc [' ~ mu * 'g/' * m^3 * ']'))
  # iSpec = 'SO4'
  for (iSpec in c('SO4', 'SOX', 'SO2')) {
  
    # plot D: define screens ----
    ShlB = 0.07 # left
    ShrB = 0.992 # right
    ShdB = 0.035 # Abstand
    SvtB = 0.955 # 0.963 # 0.93 # top
    SvbB = 0.0264 # 0.05 # bottom
    SvdB = 0.01 # 0.0053 # 0.01 # Abstand
    ShBox = 0.06
    ShcB = ShlB+(ShrB-ShlB)*3/6
    
    if(iSpec == 'SO4') { thisMtrx = c('pm25','pm10'); thisNumb = 'fig04'
    } else if (iSpec == 'SOX') { thisMtrx = 'pm10'; thisNumb = 'figS03'
    } else if (iSpec == 'SO2') { thisMtrx = 'air'; thisNumb = 'figS02'
    }
    
    # for (iMtrx in allMATRICEs[[iSpec]]) {
    # for (iMtrx in c('pm10')) {
    for (iMtrx in thisMtrx) {
      # postscript(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'moreStations', 'eps', sep = '.'), onefile=TRUE,
      #            horizontal = FALSE, paper = 'special', width = 10, height = 6.5)
      pdf(file = paste(fDirConc, paste(paste(thisNumb, 'CONC_timeseries', setup, iSpec, iMtrx, 'bar', 'sixStations_onePage', sep = '_'), 'pdf', sep = '.'), sep = '/'),
                 paper = 'special', width = 10, height = 12.3) # 6.5
      # postscript(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'threeStations', 'eps', sep = '.'), onefile=TRUE,
      #           horizontal = FALSE, paper = 'special', width = 10, height = 6.5)
      # pdf(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'threeStations', 'pdf', sep = '.'), 
      #            paper = 'special', width = 10, height = 6.5)
      par(mar = c(0,0,0,0)+0, 
          mgp = c(1.5,0.5,0.0))
      
      thisyAxisLab = yAxisLab[[iSpec]]
      if(iSpec == 'SO4') thisyAxisLab = yAxisLab[[paste(iSpec, iMtrx, sep = '')]]
      
      if ( iSpec %in% c('NO3', 'NH4', 'HNO3', 'NH3') ) { showOBS = FALSE
      } else { showOBS = TRUE }
      
      iScreen = 1
      
      split.screen(rbind(c(ShlB,ShrB, SvtB+SvdB, 1.0-SvdB), # legend row
                         # row1 left, time series
                         c(ShlB, ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*5/6+SvdB/2, SvtB),
                         c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*5/6+SvdB/2, SvtB),
                         # row1 right, time series
                         c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*5/6+SvdB/2, SvtB),
                         c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*5/6+SvdB/2, SvtB),
                         # row2 left, time series
                         c(ShlB,ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*4/6+SvdB/2, SvbB+(SvtB-SvbB)*5/6-SvdB/2),
                         c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*4/6+SvdB/2, SvbB+(SvtB-SvbB)*5/6-SvdB/2),
                         # row2 right, time series
                         c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*4/6+SvdB/2, SvbB+(SvtB-SvbB)*5/6-SvdB/2),
                         c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*4/6+SvdB/2, SvbB+(SvtB-SvbB)*5/6-SvdB/2),
                         # row3 left, time series
                         c(ShlB,ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*3/6+SvdB/2, SvbB+(SvtB-SvbB)*4/6-SvdB/2),
                         c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*3/6+SvdB/2, SvbB+(SvtB-SvbB)*4/6-SvdB/2),
                         # row3 right, time series
                         c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*3/6+SvdB/2, SvbB+(SvtB-SvbB)*4/6-SvdB/2),
                         c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*3/6+SvdB/2, SvbB+(SvtB-SvbB)*4/6-SvdB/2),
                         # row4 left, time series
                         c(ShlB,ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*2/6+SvdB/2, SvbB+(SvtB-SvbB)*3/6-SvdB/2),
                         c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*2/6+SvdB/2, SvbB+(SvtB-SvbB)*3/6-SvdB/2),
                         # row4 right, time series
                         c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*2/6+SvdB/2, SvbB+(SvtB-SvbB)*3/6-SvdB/2),
                         c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*2/6+SvdB/2, SvbB+(SvtB-SvbB)*3/6-SvdB/2),
                         # row5 left, time series
                         c(ShlB,ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*1/6+SvdB/2, SvbB+(SvtB-SvbB)*2/6-SvdB/2),
                         c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*1/6+SvdB/2, SvbB+(SvtB-SvbB)*2/6-SvdB/2),
                         # row2 right, time series
                         c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*1/6+SvdB/2, SvbB+(SvtB-SvbB)*2/6-SvdB/2),
                         c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*1/6+SvdB/2, SvbB+(SvtB-SvbB)*2/6-SvdB/2),
                         # bottom left, time series
                         c(ShlB,ShcB-ShdB/2-ShBox, SvbB, SvbB+(SvtB-SvbB)*1/6-SvdB/2),
                         c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB, SvbB+(SvtB-SvbB)*1/6-SvdB/2),
                         # bottom right, time series
                         c(ShcB+ShdB/2, ShrB-ShBox, SvbB, SvbB+(SvtB-SvbB)*1/6-SvdB/2),
                         c(ShrB-ShBox, ShrB, SvbB, SvbB+(SvtB-SvbB)*1/6-SvdB/2)
      ))
      
      # plot D: make legend ----
      screen(iScreen)
      # plot(c(0,1), c(0,1), axes = FALSE, ann = FALSE, type = 'n')
      plot(c(0,1), c(0,1), xaxt = 'n', yaxt = 'n', ann = FALSE, type = 'n')
      
      if ( showOBS ) {
        # legend('left', legend = c('obs', 'SALT, GO03', 'SALT, OV14', 'SHIPPING', 'ENERGY', 'UNSPEC.', '  year 2013'),
        legend('left', legend = c(expression('O'[Sigma]), expression('P'['SeaSalt,GO3']), expression('P'['SeaSalt,OV14']), 
                                  expression('P'['Shipping']), expression('P'['Energy']), expression('P'['unspecified']), '  year 2013'),
               col = c('black', 'blue', 'magenta', 'green', 'red', 'orange', 'white'),
               lwd = c(3,NA,NA,NA,NA,NA,NA), 
               pch = c(NA, 15, 15, 15, 15, 15, NA), 
               lty = c('solid', 'solid', 'solid', 'solid', 'solid', 'solid', NA),
               bg = 'white', horiz = TRUE, inset = 0.02,
               pt.cex = 2, seg.len = 0.5, text.width = 0.12, bty = 'n')
      } else {
        legend('left', legend = c('', 'ALL', 'NOSHIPS', tLab, '  year 2013'),
               col = c('white', 'blue', 'red', tCol, 'white'), lwd = c(10,2,2,2, NA), 
               lty = c('solid', NA, NA, NA, NA), pch = c(NA, 19, 1, tPch, NA),
               bg = 'white', horiz = TRUE, inset = 0.02,
               pt.cex = 2, seg.len = 0.5, text.width = 0.15, bty = 'n')
      }
      
      for(iStat in myStations[c(3,5,9,11,14,17)]){
      # for(iStat in myStations[c(3,5,9,14,17)]){
        # for(iStat in myStations[c(3,5,6)]){
        
        myMax = 0
        for (iMon in MONs) {
          myMax = max(myMax, max(valOBS[[iMon]][[iSpec]][[iMtrx]][[iStat]]$timeser$value, na.rm = TRUE))
          for(iScen in SCENs) { myMax = max(myMax, max(valSIM[[iMon]][[iScen]][[iSpec]][[iMtrx]][[iStat]], na.rm = TRUE)) }
        }
        myLim = getYlim(myMax, 3)
        
        #for(iMon in c('win2013', 'win2013')) {
        for(iMon in MONs) {
          refDate=2013001
          strYear='2013'
          if(iMon == 'sum2013' || iMon == 'sum') {
            from = 2013182; to = 2013243; nT = 62
          } else if(iMon == 'win2013' || iMon == 'win') {
            from = 2013001; to = 2013059; nT = 59
          }
          nTh = nT * 24
          
          
          # plot D: prepare data ----
          timeAxis = as.POSIXct((0:(nT) + (from-refDate))*3600*24, tz = 'UTC', origin = "2008-01-01 00:00:00")
          # timeAxis = as.POSIXct((0:(nT-1) + (from-refDate))*3600*24, tz = 'UTC', origin = "2008-01-01 00:00:00")
          timeMean = colMeans(rbind(timeAxis[-1],timeAxis[-nT-1]))
          
          pltOBS = rep(NA, nT)
          pltOBS = valOBS[[iMon]][[iSpec]][[iMtrx]][[iStat]]$timeser$value
          if ( is.null(pltOBS) || length(pltOBS) == 0) { pltOBS = rep(NA, nT) }
          
          pltOBSnoNA = pltOBS
          pltOBSnoNA[is.na(pltOBS)] = 0
          
          # alternative: colMeans(array(ARRAY, dim = c(24,91)), na.rm = TRUE)
          pltSIM_ALL = valSIM[[iMon]][['ALLsurf']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_ALL_Na = valSIM[[iMon]][['ALLsurf']][['Na']][[iMtrx]][[iStat]]
          pltSIM_NOSH = valSIM[[iMon]][['NOSHIPSsurf']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_NOSA = valSIM[[iMon]][['NOSALT']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_NOEN = valSIM[[iMon]][['NOENERGY']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_OV14 = valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_OV14_Na = valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]]
          
          pltSIM_SHIP = pltSIM_ALL - pltSIM_NOSH
          pltSIM_ENER = pltSIM_ALL - pltSIM_NOEN
          if (iSpec == 'SO4') {
            pltSIM_SALT_GO03 = pltSIM_ALL_Na / ssNaMass * ssSO4Mass / 3 # /3 because of mass in S and not SO4
            pltSIM_SALT_OV14 = pltSIM_OV14_Na / ssNaMass * ssSO4Mass / 3 # /3 becaues of mass in S
          } else if (iSpec == 'SOX') {
            pltSIM_SALT_GO03 = ( valSIM[[iMon]][['ALLsurf']][['Na']][[iMtrx]][[iStat]] / ssNaMass * ssSO4Mass / 3
                                 + ( valSIM[[iMon]][['ALLsurf']][['SO2']][['air']][[iStat]]
                                     - valSIM[[iMon]][['NOSALT']][['SO2']][['air']][[iStat]] ) )
            pltSIM_SALT_OV14 = ( valSIM[[iMon]][['SALT_OV14']][['Na']][[iMtrx]][[iStat]] / ssNaMass * ssSO4Mass / 3
                                 + ( valSIM[[iMon]][['SALT_OV14']][['SO2']][['air']][[iStat]]
                                     - valSIM[[iMon]][['NOSALT']][['SO2']][['air']][[iStat]] ) )
          } else {
            pltSIM_SALT_GO03 = pltSIM_ALL - pltSIM_NOSA
            pltSIM_SALT_OV14 = pltSIM_OV14 - pltSIM_NOSA
          }
          pltSIM_OTHR = pltSIM_ALL - pltSIM_SHIP - pltSIM_ENER - pltSIM_SALT_GO03
          
          # if (iStat == 'DK0008R' && iSpec == 'SO2') browser()
          
          myLim = getYlim(max(c(pltOBS, pltSIM_ALL, pltSIM_NOSH, pltSIM_NOSA, pltSIM_NOEN, 0.0), na.rm = TRUE), 3)
          
          # plot D: start plotting ----
          iScreen = iScreen + 1 
          
          screen(iScreen)
          plot(timeAxis[-nT-1], pltOBSnoNA,
               type = 'n', ann = FALSE,
               ylim = c(0,myLim),
               yaxt = 'n', xaxt = 'n')
          
          if ((iScreen-1)%%2 == 1) axis(2)
          # if(iScreen > 12) axis(1)
          
          yTicksWin = c(1199145600, 1201824000, 1204329600, 1207008000)
          yTicksSum = c(1215043200, 1217721600, 1220400000, 1222992000)
          if(iScreen %in% c(2, 6, 10, 14, 18, 22)) axis(1,at = yTicksWin, labels = FALSE, tck = -0.03)
          if(iScreen == 22) axis(1,at = (yTicksWin[-1]+yTicksWin[-4])/2, tick = FALSE, labels = c('Jan', 'Feb', 'Mar'))
          if(iScreen %in% c(4, 8, 12, 16, 20, 24)) axis(1,at = yTicksSum, labels = FALSE, tck = -0.03) # 1212278400, 1214870400, 1217548800, 1220227200
          if(iScreen == 24) axis(1,at = (yTicksSum[-1]+yTicksSum[-4])/2, tick = FALSE, labels = c('Jul', 'Aug', 'Sept'))
          # if((iScreen-1)%%2 == 0) mtext(thisyAxisLab,2, line = 1)
          if(iScreen %in% c(10)) mtext(thisyAxisLab,2, line = 1.6)
          if ((iScreen-1)%%4 == 3) title(list_stations[[iStat]], line = -1)
          
          polygon(rbind(c(0.0,timeMean), c(0.0,timeMean), timeAxis,timeAxis)[-(1:2)], 
                  c(0, rbind(rep(0.0, length(pltSIM_OV14)),rep(0.0, length(pltSIM_OV14)), pltSIM_OV14, pltSIM_OV14), 0),
                  col = 'magenta', border = FALSE)
          polygon(rbind(c(0.0,timeMean), c(0.0,timeMean), timeAxis,timeAxis)[-(1:2)],
                  c(0, rbind(pltSIM_ALL, pltSIM_ALL, rep(0.0, length(pltSIM_ALL)),rep(0.0, length(pltSIM_ALL))), 0),
                  col = 'blue', border = FALSE)
          polygon(rep(timeAxis, each = 2), c(0.0, rep(pltSIM_ENER + pltSIM_OTHR + pltSIM_SHIP, each = 2), 0.0),
                  col = 'green', border = FALSE)
          polygon(rep(timeAxis, each = 2), c(0.0, rep(pltSIM_ENER + pltSIM_OTHR, each = 2), 0.0),
                  col = 'red', border = FALSE)
          polygon(rep(timeAxis, each = 2), c(0.0, rep(pltSIM_OTHR, each = 2), 0.0),
                  col = 'orange', border = FALSE)
          
          if ( showOBS && sum(!is.na(pltOBS)) > 0 ) {
            lines(rep(timeMean, each = 3), c(rbind(rep(0.0,length(pltOBS)), pltOBS, rep(NA,length(pltOBS)))),
                  col = 'black')
            lines(rep(timeAxis, each = 3)[-c(1,2,length(timeAxis)*3)], c(rbind(pltOBS, pltOBS, rep(NA,length(pltOBS)))),
                  col = 'black')
            lines(rep(timeAxis, each = 3)[-c(1,2,length(timeAxis)*3)], c(rbind(pltOBS*0.0, pltOBS*0.0, rep(NA,length(pltOBS)))),
                  col = 'black')
          }
          
          
          # plot D: make mean plot ----
          iScreen = iScreen + 1 
          screen(iScreen)
          # NOTE: [,!is.na(pltOBS)] makes that only values are considered which
          #       are available as observations and measurements.
          plot(c(0.0,0.0), c(0.0,0.0),
               type = 'n', ann = FALSE, 
               ylim = c(0,myLim), xlim = c(0,3),
               yaxt = 'n', xaxt = 'n')
          
          meanSALT_GO03_all = mean(pltSIM_SALT_GO03)
          meanSALT_GO03_obs = mean(pltSIM_SALT_GO03[!is.na(pltOBS)])
          meanSALT_OV14_all = mean(pltSIM_SALT_OV14)
          meanSALT_OV14_obs = mean(pltSIM_SALT_OV14[!is.na(pltOBS)])
          meanSHIP_all = mean(pltSIM_SHIP)
          meanSHIP_obs = mean(pltSIM_SHIP[!is.na(pltOBS)])
          meanENER_all = mean(pltSIM_ENER)
          meanENER_obs = mean(pltSIM_ENER[!is.na(pltOBS)])
          meanOTHR_all = mean(pltSIM_OTHR)
          meanOTHR_obs = mean(pltSIM_OTHR[!is.na(pltOBS)])
          meanOBS = mean(pltOBS[!is.na(pltOBS)])
          
          if ( showOBS && sum(!is.na(pltOBS)) > 0 ) {
            polygon(c(0.8,0.8,1.3,1.3),
                    c(0.0,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_OV14_obs,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_OV14_obs,0.0),
                    col = 'magenta', border = FALSE)
            polygon(c(0.3,0.3,0.8,0.8),
                    c(0.0,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_GO03_obs,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_GO03_obs,0.0),
                    col = 'blue', border = FALSE)
            polygon(c(0.3,0.3,1.3,1.3),
                    c(0.0,meanOTHR_obs+meanENER_obs+meanSHIP_obs,meanOTHR_obs+meanENER_obs+meanSHIP_obs,0.0),
                    col = 'green', border = FALSE)
            polygon(c(0.3,0.3,1.3,1.3),
                    c(0.0,meanOTHR_obs+meanENER_obs,meanOTHR_obs+meanENER_obs,0.0),
                    col = 'red', border = FALSE)
            polygon(c(0.3,0.3,1.3,1.3),
                    c(0.0,meanOTHR_obs,meanOTHR_obs,0.0),
                    col = 'orange', border = FALSE)
            lines(c(0.8,0.8),c(0.0, meanOBS), col = 'black')
            lines(c(0.3,1.3), c(meanOBS, meanOBS), col = 'black')
          }
          polygon(c(2.2,2.2,2.7,2.7),
                  c(0.0,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_OV14_all,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_OV14_all,0.0),
                  col = 'magenta', border = FALSE)
          polygon(c(1.7,1.7,2.2,2.2),
                  c(0.0,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_GO03_all,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_GO03_all,0.0),
                  col = 'blue', border = FALSE)
          polygon(c(1.7,1.7,2.7,2.7),
                  c(0.0,meanOTHR_all+meanENER_all+meanSHIP_all,meanOTHR_all+meanENER_all+meanSHIP_all,0.0),
                  col = 'green', border = FALSE)
          polygon(c(1.7,1.7,2.7,2.7),
                  c(0.0,meanOTHR_all+meanENER_all,meanOTHR_all+meanENER_all,0.0),
                  col = 'red', border = FALSE)
          polygon(c(1.7,1.7,2.7,2.7),
                  c(0.0,meanOTHR_all,meanOTHR_all,0.0),
                  col = 'orange', border = FALSE)
          
        }
      }
      close.screen(all.screens = TRUE)
      dev.off()
    }
  }
}



# plot E (five stations, bar) ----
if(plot.dat.E) {  
  
  yAxisLab = list('SO4' = expression("SO"[4]^"2-" ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'SO2' = expression("SO"[2] ~ 'conc [' ~ mu * 'g S/' * m^3 * ']'),
                  'Na' = expression("Na"^"+" ~ 'conc [' ~ mu * 'g/' * m^3 * ']'))
  iSpec = 'SO4'
  
  # plot E: define screens ----
  ShlB = 0.07 # left
  ShrB = 0.992 # right
  ShdB = 0.035 # Abstand
  SvtB = 0.9577 # 0.93 # top
  SvbB = 0.0302 # 0.05 # bottom
  SvdB = 0.0121 # 0.0053 # 0.01 # Abstand
  ShBox = 0.06
  ShcB = ShlB+(ShrB-ShlB)*3/6
  
  # for (iMtrx in allMATRICEs[[iSpec]]) {
  for (iMtrx in c('pm10')) {
    # postscript(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'moreStations', 'eps', sep = '.'), onefile=TRUE,
    #            horizontal = FALSE, paper = 'special', width = 10, height = 6.5)
    pdf(file = paste(fDirConc, paste(paste('fig04_CONC_timeseries', setup, iSpec, iMtrx, 'bar', 'fiveStations_onePage', sep = '_'), 'pdf', sep = '.'), sep = '/'),
        paper = 'special', width = 10, height = 10.75) # 6.5
    # postscript(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'threeStations', 'eps', sep = '.'), onefile=TRUE,
    #           horizontal = FALSE, paper = 'special', width = 10, height = 6.5)
    # pdf(file = paste('CONC.timeseries', setup, iSpec, iMtrx, 'bar', 'threeStations', 'pdf', sep = '.'), 
    #            paper = 'special', width = 10, height = 6.5)
    par(mar = c(0,0,0,0)+0, 
        mgp = c(1.5,0.5,0.0))
    
    
    if ( iSpec %in% c('NO3', 'NH4', 'HNO3', 'NH3') ) { showOBS = FALSE
    } else { showOBS = TRUE }
    
    iScreen = 1
    
    nPlt = 5
    
    split.screen(rbind(c(ShlB,ShrB, SvtB+SvdB, 1.0-SvdB), # legend row
                       # row1 left, time series
                       c(ShlB, ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*(nPlt-1)/nPlt+SvdB/2, SvtB),
                       c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*(nPlt-1)/nPlt+SvdB/2, SvtB),
                       # row1 right, time series
                       c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*(nPlt-1)/nPlt+SvdB/2, SvtB),
                       c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*(nPlt-1)/nPlt+SvdB/2, SvtB),
                       # row2 left, time series
                       c(ShlB,ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*(nPlt-2)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-1)/nPlt-SvdB/2),
                       c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*(nPlt-2)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-1)/nPlt-SvdB/2),
                       # row2 right, time series
                       c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*(nPlt-2)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-1)/nPlt-SvdB/2),
                       c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*(nPlt-2)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-1)/nPlt-SvdB/2),
                       # row3 left, time series
                       c(ShlB,ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*(nPlt-3)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-2)/nPlt-SvdB/2),
                       c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*(nPlt-3)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-2)/nPlt-SvdB/2),
                       # row3 right, time series
                       c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*(nPlt-3)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-2)/nPlt-SvdB/2),
                       c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*(nPlt-3)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-2)/nPlt-SvdB/2),
                       # row4 left, time series
                       c(ShlB,ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*(nPlt-4)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-3)/nPlt-SvdB/2),
                       c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*(nPlt-4)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-3)/nPlt-SvdB/2),
                       # row4 right, time series
                       c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*(nPlt-4)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-3)/nPlt-SvdB/2),
                       c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*(nPlt-4)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-3)/nPlt-SvdB/2),
                       # # row5 left, time series
                       # c(ShlB,ShcB-ShdB/2-ShBox, SvbB+(SvtB-SvbB)*(nPlt-5)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-4)/nPlt-SvdB/2),
                       # c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB+(SvtB-SvbB)*(nPlt-5)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-4)/nPlt-SvdB/2),
                       # # row2 right, time series
                       # c(ShcB+ShdB/2, ShrB-ShBox, SvbB+(SvtB-SvbB)*(nPlt-5)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-4)/nPlt-SvdB/2),
                       # c(ShrB-ShBox, ShrB, SvbB+(SvtB-SvbB)*(nPlt-5)/nPlt+SvdB/2, SvbB+(SvtB-SvbB)*(nPlt-4)/nPlt-SvdB/2),
                       # # bottom left, time series
                       # c(ShlB,ShcB-ShdB/2-ShBox, SvbB, SvbB+(SvtB-SvbB)*(nPlt-5)/nPlt-SvdB/2),
                       # c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB, SvbB+(SvtB-SvbB)*(nPlt-5)/nPlt-SvdB/2),
                       # bottom right, time series
                       # c(ShcB+ShdB/2, ShrB-ShBox, SvbB, SvbB+(SvtB-SvbB)*(nPlt-5)/nPlt-SvdB/2),
                       # c(ShrB-ShBox, ShrB, SvbB, SvbB+(SvtB-SvbB)*(nPlt-5)/nPlt-SvdB/2),
                       # bottom left, time series
                       c(ShlB,ShcB-ShdB/2-ShBox, SvbB, SvbB+(SvtB-SvbB)*(nPlt-4)/nPlt-SvdB/2),
                       c(ShcB-ShdB/2-ShBox, ShcB-ShdB/2, SvbB, SvbB+(SvtB-SvbB)*(nPlt-4)/nPlt-SvdB/2),
                       # bottom right, time series
                       c(ShcB+ShdB/2, ShrB-ShBox, SvbB, SvbB+(SvtB-SvbB)*(nPlt-4)/nPlt-SvdB/2),
                       c(ShrB-ShBox, ShrB, SvbB, SvbB+(SvtB-SvbB)*(nPlt-4)/nPlt-SvdB/2)
    ))
    
    # plot E: make legend ----
    screen(iScreen)
    # plot(c(0,1), c(0,1), axes = FALSE, ann = FALSE, type = 'n')
    plot(c(0,1), c(0,1), xaxt = 'n', yaxt = 'n', ann = FALSE, type = 'n')
    
    if ( showOBS ) {
      legend('left', legend = c('obs', 'SALT, GO03', 'SALT, OV14', 'SHIPPING', 'ENERGY', 'UNSPEC.', '  year 2013'),
             col = c('darkgrey', 'blue', 'magenta', 'green', 'red', 'orange', 'white'),
             lwd = c(10,10,10,10,10,10,NA), 
             lty = c('solid', 'solid', 'solid', 'solid', 'solid', 'solid', NA), pch = NA,
             bg = 'white', horiz = TRUE, inset = 0.02,
             pt.cex = 2, seg.len = 0.3, text.width = 0.12, bty = 'n')
    } else {
      legend('left', legend = c('', 'ALL', 'NOSHIPS', tLab, '  year 2013'),
             col = c('white', 'blue', 'red', tCol, 'white'), lwd = c(10,2,2,2, NA), 
             lty = c('solid', NA, NA, NA, NA), pch = c(NA, 19, 1, tPch, NA),
             bg = 'white', horiz = TRUE, inset = 0.02,
             pt.cex = 2, seg.len = 0.5, text.width = 0.15, bty = 'n')
    }
    
    # for(iStat in myStations[c(3,5,9,11,14,17)]){
    for(iStat in myStations[c(3,5,9,14,17)]){
      # for(iStat in myStations[c(3,5,6)]){
      
      myMax = 0
      for (iMon in MONs) {
        myMax = max(myMax, max(valOBS[[iMon]][[iSpec]][[iMtrx]][[iStat]]$timeser$value, na.rm = TRUE))
        for(iScen in SCENs) { myMax = max(myMax, max(valSIM[[iMon]][[iScen]][[iSpec]][[iMtrx]][[iStat]], na.rm = TRUE)) }
      }
      myLim = getYlim(myMax, 3)
      
      #for(iMon in c('win2013', 'win2013')) {
      for(iMon in MONs) {
        refDate=2013001
        strYear='2013'
        if(iMon == 'sum2013' || iMon == 'sum') {
          from = 2013182; to = 2013243; nT = 62
        } else if(iMon == 'win2013' || iMon == 'win') {
          from = 2013001; to = 2013059; nT = 59
        }
        nTh = nT * 24
        
        
        # plot E: prepare data ----
        timeAxis = as.POSIXct((0:(nT) + (from-refDate))*3600*24, tz = 'UTC', origin = "2008-01-01 00:00:00")
        # timeAxis = as.POSIXct((0:(nT-1) + (from-refDate))*3600*24, tz = 'UTC', origin = "2008-01-01 00:00:00")
        timeMean = colMeans(rbind(timeAxis[-1],timeAxis[-nT-1]))
        
        pltOBS = rep(NA, nT)
        pltOBS = valOBS[[iMon]][[iSpec]][[iMtrx]][[iStat]]$timeser$value
        if ( is.null(pltOBS) || length(pltOBS) == 0) { pltOBS = rep(NA, nT) }
        
        pltOBSnoNA = pltOBS
        pltOBSnoNA[is.na(pltOBS)] = 0
        
        # alternative: colMeans(array(ARRAY, dim = c(24,91)), na.rm = TRUE)
        pltSIM_ALL = valSIM[[iMon]][['ALLsurf']][[iSpec]][[iMtrx]][[iStat]]
        pltSIM_ALL_Na = valSIM[[iMon]][['ALLsurf']][['Na']][[iMtrx]][[iStat]]
        pltSIM_NOSH = valSIM[[iMon]][['NOSHIPSsurf']][[iSpec]][[iMtrx]][[iStat]]
        pltSIM_NOSA = valSIM[[iMon]][['NOSALT']][[iSpec]][[iMtrx]][[iStat]]
        pltSIM_NOEN = valSIM[[iMon]][['NOENERGY']][[iSpec]][[iMtrx]][[iStat]]
        pltSIM_OV14 = valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]]
        pltSIM_OV14_Na = valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]]
        
        pltSIM_SHIP = pltSIM_ALL - pltSIM_NOSH
        pltSIM_ENER = pltSIM_ALL - pltSIM_NOEN
        # pltSIM_SALT_GO03 = pltSIM_ALL - pltSIM_NOSA
        pltSIM_SALT_GO03 = pltSIM_ALL_Na / ssNaMass * ssSO4Mass / 3 # /3 because of mass in S and not SO4
        pltSIM_OTHR = pltSIM_ALL - pltSIM_SHIP - pltSIM_ENER - pltSIM_SALT_GO03
        # pltSIM_SALT_OV14 = pltSIM_OV14 - pltSIM_NOSA
        pltSIM_SALT_OV14 = pltSIM_OV14_Na / ssNaMass * ssSO4Mass / 3 # /3 becaues of mass in S
        
        myLim = getYlim(max(c(pltOBS, pltSIM_ALL, pltSIM_NOSH, pltSIM_NOSA, pltSIM_NOEN, 0.0), na.rm = TRUE), 3)
        
        # plot E: start plotting ----
        iScreen = iScreen + 1 
        
        screen(iScreen)
        plot(timeAxis[-nT-1], pltOBSnoNA,
             type = 'n', ann = FALSE,
             ylim = c(0,myLim),
             yaxt = 'n', xaxt = 'n')
        
        if ((iScreen-1)%%2 == 1) axis(2)
        # if(iScreen > 12) axis(1)
        
        yTicksWin = c(1199145600, 1201824000, 1204329600, 1207008000)
        yTicksSum = c(1215043200, 1217721600, 1220400000, 1222992000)
        if(iScreen %in% c(2, 6, 10, 14, 18, 22)) axis(1,at = yTicksWin, labels = FALSE, tck = -0.03)
        if(iScreen == 18) axis(1,at = (yTicksWin[-1]+yTicksWin[-4])/2, tick = FALSE, labels = c('Jan', 'Feb', 'Mar'))
        if(iScreen %in% c(4, 8, 12, 16, 20, 24)) axis(1,at = yTicksSum, labels = FALSE, tck = -0.03) # 1212278400, 1214870400, 1217548800, 1220227200
        if(iScreen == 20) axis(1,at = (yTicksSum[-1]+yTicksSum[-4])/2, tick = FALSE, labels = c('Jul', 'Aug', 'Sept'))
        # if((iScreen-1)%%2 == 0) mtext(yAxisLab[[iSpec]],2, line = 1)
        if(iScreen %in% c(10)) mtext(yAxisLab[[iSpec]],2, line = 1.6)
        if ((iScreen-1)%%4 == 3) title(paste(list_stations[[iStat]], iStat, sep = ' '), line = -1)
        
        polygon(rbind(c(0.0,timeMean), c(0.0,timeMean), timeAxis,timeAxis)[-(1:2)], 
                c(0, rbind(rep(0.0, length(pltSIM_OV14)),rep(0.0, length(pltSIM_OV14)), pltSIM_OV14, pltSIM_OV14), 0),
                col = 'magenta', border = FALSE)
        polygon(rbind(c(0.0,timeMean), c(0.0,timeMean), timeAxis,timeAxis)[-(1:2)],
                c(0, rbind(pltSIM_ALL, pltSIM_ALL, rep(0.0, length(pltSIM_ALL)),rep(0.0, length(pltSIM_ALL))), 0),
                col = 'blue', border = FALSE)
        polygon(rep(timeAxis, each = 2), c(0.0, rep(pltSIM_ENER + pltSIM_OTHR + pltSIM_SHIP, each = 2), 0.0),
                col = 'green', border = FALSE)
        polygon(rep(timeAxis, each = 2), c(0.0, rep(pltSIM_ENER + pltSIM_OTHR, each = 2), 0.0),
                col = 'red', border = FALSE)
        polygon(rep(timeAxis, each = 2), c(0.0, rep(pltSIM_OTHR, each = 2), 0.0),
                col = 'orange', border = FALSE)
        
        if ( showOBS ) {
          lines(rep(timeMean, each = 3), c(rbind(rep(0.0,length(pltOBS)), pltOBS, rep(NA,length(pltOBS)))),
                col = 'black')
          lines(rep(timeAxis, each = 3)[-c(1,2,length(timeAxis)*3)], c(rbind(pltOBS, pltOBS, rep(NA,length(pltOBS)))),
                col = 'black')
          lines(rep(timeAxis, each = 3)[-c(1,2,length(timeAxis)*3)], c(rbind(pltOBS*0.0, pltOBS*0.0, rep(NA,length(pltOBS)))),
                col = 'black')
        }
        
        
        # plot E: make mean plot ----
        iScreen = iScreen + 1 
        screen(iScreen)
        # NOTE: [,!is.na(pltOBS)] makes that only values are considered which
        #       are available as observations and measurements.
        plot(c(0.0,0.0), c(0.0,0.0),
             type = 'n', ann = FALSE, 
             ylim = c(0,myLim), xlim = c(0,3),
             yaxt = 'n', xaxt = 'n')
        
        meanSALT_GO03_all = mean(pltSIM_SALT_GO03)
        meanSALT_GO03_obs = mean(pltSIM_SALT_GO03[!is.na(pltOBS)])
        meanSALT_OV14_all = mean(pltSIM_SALT_OV14)
        meanSALT_OV14_obs = mean(pltSIM_SALT_OV14[!is.na(pltOBS)])
        meanSHIP_all = mean(pltSIM_SHIP)
        meanSHIP_obs = mean(pltSIM_SHIP[!is.na(pltOBS)])
        meanENER_all = mean(pltSIM_ENER)
        meanENER_obs = mean(pltSIM_ENER[!is.na(pltOBS)])
        meanOTHR_all = mean(pltSIM_OTHR)
        meanOTHR_obs = mean(pltSIM_OTHR[!is.na(pltOBS)])
        meanOBS = mean(pltOBS[!is.na(pltOBS)])
        
        if ( showOBS ) {
          polygon(c(0.8,0.8,1.3,1.3),
                  c(0.0,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_OV14_obs,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_OV14_obs,0.0),
                  col = 'magenta', border = FALSE)
          polygon(c(0.3,0.3,0.8,0.8),
                  c(0.0,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_GO03_obs,meanOTHR_obs+meanENER_obs+meanSHIP_obs+meanSALT_GO03_obs,0.0),
                  col = 'blue', border = FALSE)
          polygon(c(0.3,0.3,1.3,1.3),
                  c(0.0,meanOTHR_obs+meanENER_obs+meanSHIP_obs,meanOTHR_obs+meanENER_obs+meanSHIP_obs,0.0),
                  col = 'green', border = FALSE)
          polygon(c(0.3,0.3,1.3,1.3),
                  c(0.0,meanOTHR_obs+meanENER_obs,meanOTHR_obs+meanENER_obs,0.0),
                  col = 'red', border = FALSE)
          polygon(c(0.3,0.3,1.3,1.3),
                  c(0.0,meanOTHR_obs,meanOTHR_obs,0.0),
                  col = 'orange', border = FALSE)
          lines(c(0.8,0.8),c(0.0, meanOBS), col = 'black')
          lines(c(0.3,1.3), c(meanOBS, meanOBS), col = 'black')
        }
        polygon(c(2.2,2.2,2.7,2.7),
                c(0.0,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_OV14_all,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_OV14_all,0.0),
                col = 'magenta', border = FALSE)
        polygon(c(1.7,1.7,2.2,2.2),
                c(0.0,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_GO03_all,meanOTHR_all+meanENER_all+meanSHIP_all+meanSALT_GO03_all,0.0),
                col = 'blue', border = FALSE)
        polygon(c(1.7,1.7,2.7,2.7),
                c(0.0,meanOTHR_all+meanENER_all+meanSHIP_all,meanOTHR_all+meanENER_all+meanSHIP_all,0.0),
                col = 'green', border = FALSE)
        polygon(c(1.7,1.7,2.7,2.7),
                c(0.0,meanOTHR_all+meanENER_all,meanOTHR_all+meanENER_all,0.0),
                col = 'red', border = FALSE)
        polygon(c(1.7,1.7,2.7,2.7),
                c(0.0,meanOTHR_all,meanOTHR_all,0.0),
                col = 'orange', border = FALSE)
        
      }
    }
    close.screen(all.screens = TRUE)
    dev.off()
  }
}



# statistics ----
if(doStatistics) {
  units = list()
  fStat = paste(fDirStat, paste('stat', 'new', setup, mech, strYear, 'moreStations', 'csv', sep = '.'), sep = '/')
  
  write('SPECIES,MATRIX,SEASON,STATION,CASE,n,rae,mnb,cor,mean.sim,mean.obs,median.sim,median.obs', 
        file = fStat, append = FALSE)
  
  for(iMon in MONs) {
    refDate=2013001
    strYear='2013'
    if(iMon == 'sum2013' || iMon == 'sum') {
      from = 2013182; to = 2013243; nT = 62
    } else if(iMon == 'win2013' || iMon == 'win') {
      from = 2013001; to = 2013059; nT = 59
    }
    nTh = nT * 24
    
    for(iSpec in c(SPECs, 'SOX')) {
      for (iMtrx in allMATRICEs[[iSpec]]) {
        for (iStat in myStations) {
          obs = valOBS[[iMon]][[iSpec]][[iMtrx]][[iStat]]$timeser$values
          
          for(iScen in SCENs) {
            # print(paste(iMon, iScen, iSpec, iStat))
            
            sim = valSIM[[iMon]][[iScen]][[iSpec]][[iMtrx]][[iStat]]
            validVals = !is.na(obs + sim)
            
            if ( !(is.null(sim) || is.null(obs) || sum(!is.na(sim)) == 0) ) {
              # statistics: calculate ----
              n = sum(validVals)
              rae = sum(abs(obs-sim), na.rm = TRUE)/sum(validVals)
              mean.sim = mean(sim[validVals])
              mean.obs = mean(obs[validVals])
              median.sim = median(sim[validVals])
              median.obs = median(obs[validVals])
              mnb = MNB(obs, sim)
              myCor = cor(sim[validVals], obs[validVals], method = 'spearman')
              
              # statistics: write ----
              # write(paste(iSpec, iMon, iStat, iScen, formatC(rae, format = 'f', width = 4), sep = ','), 
              #       file = fStat, append = TRUE)
              write(paste(iSpec, iMtrx, iMon, iStat, iScen, n, rae, mnb, myCor, mean.sim, mean.obs, median.sim, median.obs, sep = ','), 
                    file = fStat, append = TRUE)
            }
          }
          
        }
      }
    }
  }
  #' make the files nice with
  #' sed -r ':a;N;$!ba;s/([a-zA-Z]+_[a-zA-Z0-9]+_[a-zA-Z]+)\n"station"/\n"\1" "station"/g' statistic.effectSeaSalt.cb05cl_ae5_aq.2008.csv | sed ':a;N;$!ba;s/^\n//g' > statistic.effectSeaSalt.cb05cl_ae5_aq.2008.csv
  #' sed -r ':a;N;$!ba;s/([a-zA-Z0-9]+_[a-zA-Z0-9]+_[a-zA-Z]+)\n"station"/\n"\1" "station"/g' statistic.effectSeaSalt.v5.cb05tucl_ae5_aq_ssa.2008.csv | sed ':a;N;$!ba;s/^\n//g' > statistic.effectSeaSalt.v5.cb05tucl_ae5_aq.2008.csv
  #' first sed: Remove newline between 'index' and 'statistics' (see above) and add a new
  #'            line above 'index'. The column headings of stations are no move on column
  #'            to the right. The first column of the heading row contains index.
  #' second sed: Remove the newline before the first data set.
  #' 
  #' make it even more nicer :-)
  #' cat statistic.effectSeaSalt.v5.cb05tucl_ae5_aq.2008.csv | tr '\n' ';' | sed -r 's/("(win|sum)2008_(SNH4|SNO3|SO4|XSO4|Na)_(noSurf|base|zero|full)") ([^;]*);"[123]" ([^;]*);"[123]" ([^;]*);"[123]" ([^;]*);/\1 \5;\1 \6;\1 \7;\1 \8/g' | sed -r 's/;"([^;"]*)" "station" [^;]*;/;/g' | sed 's/^"[^"]*"/"setup"/g' | tr ';' '\n' > statistic.effectSeaSalt.v5.cb05tucl_ae5_aq.2008.A.csv
  #' 
}



# write overview data ----
if (write.dat) {
  
  fData = paste(fDirConc, paste('data', setup, mech, strYear, 'means', 'csv', sep = '.'), sep = '/')
  
  write('Season,Station_ID,Species,Matrix,n_obs,meanOBS, meanOTHR_obs, meanENER_obs, meanSHIP_obs, meanSALT_GO03_obs, meanSALT_OV14_obs, n_all, meanOTHR_all, meanENER_all, meanSHIP_all, meanSALT_GO03_all, meanSALT_OV14_all',
        file = fData, append = FALSE)
  
  for (iMon in MONs) {
    refDate=2013001
    strYear='2013'
    if(iMon == 'sum2013' || iMon == 'sum') {
      from = 2013182; to = 2013243; nT = 62
    } else if(iMon == 'win2013' || iMon == 'win') {
      from = 2013001; to = 2013059; nT = 59
    }
    nTh = nT * 24
    for (iSpec in c(SPECs, 'SOX')) {
      for (iMtrx in allMATRICEs[[iSpec]]) {
        for (iStat in myStations) {# write overview data: get data ----
          pltOBS = rep(NA, nT)
          pltOBS = valOBS[[iMon]][[iSpec]][[iMtrx]][[iStat]]$timeser$value
          if ( is.null(pltOBS) || length(pltOBS) == 0) { pltOBS = rep(NA, nT) }
          
          pltSIM_ALL = valSIM[[iMon]][['ALLsurf']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_NOSH = valSIM[[iMon]][['NOSHIPSsurf']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_NOSA = valSIM[[iMon]][['NOSALT']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_NOEN = valSIM[[iMon]][['NOENERGY']][[iSpec]][[iMtrx]][[iStat]]
          pltSIM_OV14 = valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]]
          
          pltSIM_SHIP = pltSIM_ALL - pltSIM_NOSH
          pltSIM_ENER = pltSIM_ALL - pltSIM_NOEN
          if (iSpec == 'SO4') {
            pltSIM_SALT_GO03 = valSIM[[iMon]][['ALLsurf']][['Na']][[iMtrx]][[iStat]] / ssNaMass * ssSO4Mass / 3
            pltSIM_SALT_OV14 = valSIM[[iMon]][['SALT_OV14']][['Na']][[iMtrx]][[iStat]] / ssNaMass * ssSO4Mass / 3
          } else {
            pltSIM_SALT_GO03 = pltSIM_ALL - pltSIM_NOSA
            pltSIM_SALT_OV14 = pltSIM_OV14 - pltSIM_NOSA
          }
          pltSIM_OTHR = pltSIM_ALL - pltSIM_SHIP - pltSIM_ENER - pltSIM_SALT_GO03
          
          meanSALT_GO03_all = mean(pltSIM_SALT_GO03)
          meanSALT_GO03_obs = mean(pltSIM_SALT_GO03[!is.na(pltOBS)])
          meanSALT_OV14_all = mean(pltSIM_SALT_OV14)
          meanSALT_OV14_obs = mean(pltSIM_SALT_OV14[!is.na(pltOBS)])
          meanSHIP_all = mean(pltSIM_SHIP)
          meanSHIP_obs = mean(pltSIM_SHIP[!is.na(pltOBS)])
          meanENER_all = mean(pltSIM_ENER)
          meanENER_obs = mean(pltSIM_ENER[!is.na(pltOBS)])
          meanOTHR_all = mean(pltSIM_OTHR)
          meanOTHR_obs = mean(pltSIM_OTHR[!is.na(pltOBS)])
          meanOBS = mean(pltOBS[!is.na(pltOBS)])
          
          n_obs = sum(!is.na(pltOBS))
          n_all = length(pltSIM_ALL)
          
          # write overview data: write ----
          write(paste(iMon, iStat, iSpec, iMtrx, n_obs, meanOBS, meanOTHR_obs, meanENER_obs, meanSHIP_obs, meanSALT_GO03_obs, meanSALT_OV14_obs, n_all, meanOTHR_all, meanENER_all, meanSHIP_all, meanSALT_GO03_all, meanSALT_OV14_all, sep = ','), 
                file = fData, append = TRUE)
        }
      }
    }
  }
}


# validation of sea salt sulfate ----
if (so4.validation) {
  iSpec = 'SO4'
  
  fData = paste(fDirValid, paste('validation', setup, mech, strYear, 'means', 'csv', sep = '.'), sep = '/')
  
  write('Season,Station_ID,Species,Matrix,n_obsA,n_obsB,n_all,mnb,mnb,mnb,mnb,R,R,R,R,mnb,mnb,mnb,mnb,R,R,R,R',
        file = fData, append = FALSE)
  write(' , , , , , ,go03_obsdif,go03_obsder,go03_derdif,go03_dedial,go03_obsdif,go03_obsder,go03_derdif,go03_dedial,ov14_obsdif,ov14_obsder,ov14_derdif,ov14_dedial,ov14_obsdif,ov14_obsder,ov14_derdif,ov14_dedial',
        file = fData, append = TRUE)
  
  cor_dedial = list()
  mnb_dedial = list()
  cor_obsder = list()
  mnb_obsder = list()
  cor_obsdif = list()
  mnb_obsdif = list()
  
  for(iMon in MONs) {
    refDate=2013001
    strYear='2013'
    if(iMon == 'sum2013' || iMon == 'sum') {
      from = 2013182; to = 2013243; nT = 62
    } else if(iMon == 'win2013' || iMon == 'win') {
      from = 2013001; to = 2013059; nT = 59
    }
    nTh = nT * 24
    cor_dedial[[iMon]] = list()
    mnb_dedial[[iMon]] = list()
    cor_obsder[[iMon]] = list()
    mnb_obsder[[iMon]] = list()
    cor_obsdif[[iMon]] = list()
    mnb_obsdif[[iMon]] = list()
    
      for (iMtrx in allMATRICEs[['SO4']]) {
        
        cor_dedial[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
        mnb_dedial[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
        cor_obsder[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
        mnb_obsder[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
        cor_obsdif[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
        mnb_obsdif[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
        
        if(iMon == 'win2013' && iMtrx == 'pm10') {iFig = 'figS06'
        } else if(iMon == 'sum2013' && iMtrx == 'pm10') {iFig = 'figS07'
        } else if(iMon == 'win2013' && iMtrx == 'pm25') {iFig = 'figS08'
        } else if(iMon == 'sum2013' && iMtrx == 'pm25') {iFig = 'figS09'
        } else {iFig = 'figS00'}
        pdf(file = paste(fDirValid, paste(paste(iFig, 'combine_sim_and_obs_sulfValidation', iMon, iMtrx, sep = '_'), 'pdf', sep = '.'), sep = '/'),
            paper = 'a4', width = 7.7, height = 11.1)
        par(mfrow = c(6,4), mar = c(1.5,1.5,0,0), mgp = c(3,0.5,0))
        
        # validation: get data ----
        for (iStat in myStations) {
          pltOBS = rep(NA, nT)
          pltOBS = valOBS[[iMon]][['SO4']][[iMtrx]][[iStat]]$timeser$value - valOBS[[iMon]][['XSO4']][[iMtrx]][[iStat]]$timeser$value
          if ( is.null(pltOBS) || length(pltOBS) == 0) { pltOBS = rep(NA, nT) }
          
          # der = derived from Na
          # dif = by difference between to cases
          # pltSIM_NA_GO03 = valSIM[[iMon]][['ALLsurf']][['Na']][[iMtrx]][[iStat]]
          # pltSIM_NA_OV14 = valSIM[[iMon]][['SALT_OV14']][['Na']][[iMtrx]][[iStat]] 
          pltSIM_ALL_GO03 = valSIM[[iMon]][['ALLsurf']][[iSpec]][[iMtrx]][[iStat]] 
          pltSIM_ALL_OV14 = valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]] 
          pltSIM_SALT_GO03_dif = pmax(valSIM[[iMon]][['ALLsurf']][[iSpec]][[iMtrx]][[iStat]] - valSIM[[iMon]][['NOSALT']][[iSpec]][[iMtrx]][[iStat]], 0)
          pltSIM_SALT_OV14_dif = pmax(valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]] - valSIM[[iMon]][['NOSALT']][[iSpec]][[iMtrx]][[iStat]], 0)
          pltSIM_SALT_GO03_der = valSIM[[iMon]][['ALLsurf']][['Na']][[iMtrx]][[iStat]] / ssNaMass * ssSO4Mass / 3
          pltSIM_SALT_OV14_der = valSIM[[iMon]][['SALT_OV14']][['Na']][[iMtrx]][[iStat]] / ssNaMass * ssSO4Mass / 3
          
          obsBADnZERO = !pmin(pltOBS==0, 1, na.rm = T)
          mnb_go03_obsdif = MNB(pltOBS[obsBADnZERO], pltSIM_SALT_GO03_dif[obsBADnZERO])
          mnb_go03_obsder = MNB(pltOBS[obsBADnZERO], pltSIM_SALT_GO03_der[obsBADnZERO])
          mnb_go03_derdif = MNB(pltSIM_SALT_GO03_der[obsBADnZERO], pltSIM_SALT_GO03_dif[obsBADnZERO])
          mnb_go03_dedial = MNB(pltSIM_SALT_GO03_der, pltSIM_SALT_GO03_dif)
          
          cor_go03_obsdif = cor(pltOBS[!is.na(pltOBS)], pltSIM_SALT_GO03_dif[!is.na(pltOBS)], method = 'pearson')
          cor_go03_obsder = cor(pltOBS[!is.na(pltOBS)], pltSIM_SALT_GO03_der[!is.na(pltOBS)], method = 'pearson')
          cor_go03_derdif = cor(pltSIM_SALT_GO03_der[!is.na(pltOBS)], pltSIM_SALT_GO03_dif[!is.na(pltOBS)], method = 'pearson')
          cor_go03_dedial = cor(pltSIM_SALT_GO03_der, pltSIM_SALT_GO03_dif, method = 'pearson')
          
          mnb_ov14_obsdif = MNB(pltOBS[obsBADnZERO], pltSIM_SALT_OV14_dif[obsBADnZERO])
          mnb_ov14_obsder = MNB(pltOBS[obsBADnZERO], pltSIM_SALT_OV14_der[obsBADnZERO])
          mnb_ov14_derdif = MNB(pltSIM_SALT_OV14_der[obsBADnZERO], pltSIM_SALT_OV14_dif[obsBADnZERO])
          mnb_ov14_dedial = MNB(pltSIM_SALT_OV14_der, pltSIM_SALT_OV14_dif)
          
          cor_ov14_obsdif = cor(pltOBS[!is.na(pltOBS)], pltSIM_SALT_OV14_dif[!is.na(pltOBS)], method = 'pearson')
          cor_ov14_obsder = cor(pltOBS[!is.na(pltOBS)], pltSIM_SALT_OV14_der[!is.na(pltOBS)], method = 'pearson')
          cor_ov14_derdif = cor(pltSIM_SALT_OV14_der[!is.na(pltOBS)], pltSIM_SALT_OV14_dif[!is.na(pltOBS)], method = 'pearson')
          cor_ov14_dedial = cor(pltSIM_SALT_OV14_der, pltSIM_SALT_OV14_dif, method = 'pearson')
          
          iLoc = which(iStat == myStations)
          cor_dedial[[iMon]][[iMtrx]]$go03[iLoc] = cor_go03_dedial
          cor_dedial[[iMon]][[iMtrx]]$ov14[iLoc] = cor_ov14_dedial
          cor_obsder[[iMon]][[iMtrx]]$go03[iLoc] = cor_go03_obsder
          cor_obsder[[iMon]][[iMtrx]]$ov14[iLoc] = cor_ov14_obsder
          cor_obsdif[[iMon]][[iMtrx]]$go03[iLoc] = cor_go03_obsdif
          cor_obsdif[[iMon]][[iMtrx]]$ov14[iLoc] = cor_ov14_obsdif
          mnb_dedial[[iMon]][[iMtrx]]$go03[iLoc] = mnb_go03_dedial
          mnb_dedial[[iMon]][[iMtrx]]$ov14[iLoc] = mnb_ov14_dedial
          mnb_obsder[[iMon]][[iMtrx]]$go03[iLoc] = mnb_go03_obsder
          mnb_obsder[[iMon]][[iMtrx]]$ov14[iLoc] = mnb_ov14_obsder
          mnb_obsdif[[iMon]][[iMtrx]]$go03[iLoc] = mnb_go03_obsdif
          mnb_obsdif[[iMon]][[iMtrx]]$ov14[iLoc] = mnb_ov14_obsdif
          
          n_obsA = sum(!is.na(pltOBS))
          n_obsB = sum(obsBADnZERO)
          n_all = length(pltSIM_SALT_GO03_dif)
          
          
          # validation: write ascii and make overview plots ----
          write(paste(iMon, iStat, iSpec, iMtrx, n_obsA, n_obsB, n_all, mnb_go03_obsdif,mnb_go03_obsder,mnb_go03_derdif,mnb_go03_dedial,cor_go03_obsdif,cor_go03_obsder,cor_go03_derdif,cor_go03_dedial,mnb_ov14_obsdif,mnb_ov14_obsder,mnb_ov14_derdif,mnb_ov14_dedial,cor_ov14_obsdif,cor_ov14_obsder,cor_ov14_derdif,cor_ov14_dedial, sep = ','), 
                file = fData, append = TRUE)
          
          maxVal = max(c(pltSIM_SALT_GO03_der, pltSIM_SALT_GO03_dif,pltSIM_SALT_OV14_der, pltSIM_SALT_OV14_dif))
          plot(2*c(0,maxVal), 2*c(0, maxVal), type = 'l', 
               xlim = c(0,maxVal), ylim = c(0,maxVal),
               col = 'black', yaxt = 'n')
          axis(2, labels = FALSE)
          # lines(c(0,0), c(maxVal, maxVal), col = 'grey')
          points(pltSIM_SALT_GO03_der, pltSIM_SALT_GO03_dif, col = 'blue')
          points(pltSIM_SALT_OV14_der, pltSIM_SALT_OV14_dif, col = 'magenta', pch = 0)
          # points(pltOBS[!is.na(pltOBS)], pltSIM_SALT_GO03_der[!is.na(pltOBS)], col = 'blue')
          # points(pltOBS[!is.na(pltOBS)], pltSIM_SALT_OV14_der[!is.na(pltOBS)], col = 'red', pch = 2)
          mtext(list_stations[[iStat]], 3, line = -1.5)
          mtext(paste('R=',formatC(cor_go03_dedial, format = 'f', digits = 2), sep = ''), 3, col = 'blue', line = -3, at = maxVal/5)
          mtext(paste('R=',formatC(cor_ov14_dedial, format = 'f', digits = 2), sep = ''), 3, col = 'magenta', line = -4.5, at = maxVal/5)
          mtext(paste('S/T=',formatC(mean(pltSIM_SALT_GO03_der/pltSIM_ALL_GO03), format = 'f', digits = 2), sep = ''), 1, col = 'blue', line = -3, at = maxVal*4/5)
          mtext(paste('S/T=',formatC(mean(pltSIM_SALT_OV14_der/pltSIM_ALL_OV14), format = 'f', digits = 2), sep = ''), 1, col = 'magenta', line = -1.5, at = maxVal*4/5)
        }
        
        plot(c(0,1), c(0,1), ann = F, axes = F, type = 'n')
        legend('topleft', legend = c(if(iMon == 'sum2013') 'summer 2013' else 'winter 2013',
                                     paste(formatC(nT, format = 'd'), 'data points', sep = ' '),
                                     if(iMtrx == 'pm10') expression('PM'[10] ~ 'data') else expression('PM'[2.5] ~ 'data'),
                                     '',
                                     "R    = Pearsons's R",
                                     expression('S/T = P'['SeaSalt,Na']*'/P'[Sigma]),
                                     expression('x-axis: P'['SeaSalt,Na']),
                                     expression('y-axis: P'['SeaSalt,diff'])),
               cex = 1.5, bty = 'n', inset = c(-0.1,0.0))
        plot(c(0,1), c(0,1), ann = F, axes = F, type = 'n')
        legend('bottomright', legend = c('GO03', 'OV14'), col = c('blue', 'magenta'),
               pch = c(1,0), cex = 1.8)
        
        dev.off()
      }
  }
}


# write out data for supplement ----
if(writeOutData) {
  myPos = names(myCells)
  nPos = length(myPos)
  headings = append(c('doy', 'hour'), myPos)
  
  
  for(iMon in MONs) {
    if(iMon == 'sum2013') {
      nT = 61*24
      from = 2013183
      to = 2013243
    } else if(iMon == 'win2013') {
      nT = 60*24
      from = 2013001
      to = 2013060
    }
    nDays = nT/24
    
    for(iScen in SCENs) {
      for(iSpec in SPECs[c(1,2,4,7)]) {
        fOut = paste(fDirConc, paste('conc.timeseries.values', iMon, iScen, iSpec, 'moreStations','csv', sep = '.'), sep = '/')
        
        tmpOut = array(0.0, dim = c(nT,nPos+2))
        tmpOut[,1] = rep(seq(from, length.out = nDays), each = 24)
        tmpOut[,2] = rep(0:23, nDays)
        
        for (iPos in 1:nPos) {
          strPos = myPos[iPos]
          tmpOut[,iPos+2] = valSIM[[iMon]][[iScen]][[iSpec]][1:nT,iPos]
        }
        write.table(tmpOut, fOut, col.names = headings, row.names = FALSE, sep = ',')
      }
    }
  }
  
}