# create_validation_plot

#' @author Daniel Neumann, daniel.neumann@hzg.de
#' @date 2016-03-27
#' @version 1.1.0
#' 
#' This script creates plots with correlation coefficients and MNBs
#' of sea salt sulfate calculated by two different means from model
#' data. Additionally, these values are compared against sea salt
#' sulfate from measurements in the EMEP database.
#' 
#' NOTE 1: Load the input data manually if not already loaded. Call
#' the out-commented 'load' command for this purpose.
#' NOTE 2; Set the boolean variables workData, plotA and plotB
#' according to your needs.
#' NOTE 3: This script depends on input data that was originally
#' read in by combine_sim_and_obs_v2.2.1_newPlotOrder.R
#' 
#' Unfortunately, this script is not nicely commented. Therefore, 
#' I will give a short description of the structure, here. This
#' script consists of four major sections: header, workData, 
#' plot A and plot B.
#'   In the header, variables are set, libraries and scripts are
#' loaded, functions are defined, and data is loaded.
#'   In the workData section, the loaded data is preprocessed.
#' This task is necessary to create plotB. For plotA it is not
#' necessary.
#'   In the plotA section, a figure containing three scatter
#' plots is created. The scatter plots show sea salt sulfate
#' compared from Na+ and from subtracting sea salt and non-sea
#' salt simulations.
#'   In the plotB section, correlation coefficients and MNBs
#' between sea salt sulfate time series obtained via different
#' methods are plotted. The two methods described for plotA
#' (multiply Na+ and subtract two simulations) are compared
#' and measurements are considered.

# header ----
# header: load libraries ----
library('ctmeval')
source('list_stations.R')


# header: load data for plotting ----
## load("inData_combine_sim_and_obs_newPlotOrder.RData")
## This data was extracted via combine_sim_and_obs_v2.2.1_newPlotOrder.R
## from the 66_Tools/pub_neumannd2016a folder.


# header: set some parameters ----
myX = 1:nSTATIONS # X coordinates
myShft = 0.1  # half distance between vertical lines of GO03 and
              # OV14 data lines

fNameA = 'fig06_create_validation_plot_A.pdf'
fNameB = 'fig07_create_validation_plot_B.pdf'
fNameC = 'fig07_create_validation_plot_C.pdf'
fDir = '17_sulfValidation'

workData = TRUE
plotA = TRUE
plotB = TRUE
plotC = TRUE


# header: define functions for plotting ----
linepointPlot = function(xVals, yVals, col, pch, cex) {
  nVals = length(yVals)
  lines(rep(xVals, each = 3), drop(matrix(rbind(rep(0.0, nVals), yVals, rep(NA, nVals)), nrow = 1)), col = col, lwd = 2)
  points(xVals, yVals, pch = pch, cex = cex, col = col)
}

linepointsAbPlot = function(xVals, yValsBot, yValsTop, col, pchBot, pchTop, cex) {
  nVals = length(yValsBot)
  lines(rep(xVals, each = 3), drop(matrix(rbind(yValsBot, yValsTop, rep(NA, nVals)), nrow = 1)), col = col, lwd = 2)
  points(xVals, yValsTop, pch = pchTop, cex = cex, col = col)
  points(xVals, yValsBot, pch = pchBot, cex = cex, col = col)
}


# workData ----
if (workData) {
  iSpec = 'SO4'
  
  # wordData: create output arrays ----
  cor_dedial = list()
  mnb_dedial = list()
  frac_dedial = list()
  cor_obsder = list()
  mnb_obsder = list()
  cor_obsdif = list()
  mnb_obsdif = list()
  
  
  for(iMon in MONs) {
    refDate=2013001
    strYear='2013'
    if(iMon == 'sum2013' || iMon == 'sum') { from = 2013182; to = 2013243; nT = 62
    } else if(iMon == 'win2013' || iMon == 'win') { from = 2013001; to = 2013059; nT = 59 }
    nTh = nT * 24
    
    cor_dedial[[iMon]] = list()
    mnb_dedial[[iMon]] = list()
    frac_dedial[[iMon]] = list()
    cor_obsder[[iMon]] = list()
    mnb_obsder[[iMon]] = list()
    cor_obsdif[[iMon]] = list()
    mnb_obsdif[[iMon]] = list()
    
    for (iMtrx in allMATRICEs[['SO4']]) {
      
      cor_dedial[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
      mnb_dedial[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
      frac_dedial[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
      cor_obsder[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
      # wordData: copy data into output arrays ----nb_obsder[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
      cor_obsdif[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
      mnb_obsdif[[iMon]][[iMtrx]] = list('go03' = rep(0.0, nSTATIONS), 'ov14' = rep(0.0, nSTATIONS))
      
      for (iStat in myStations) {
        # wordData: get data ----
        pltOBS = rep(NA, nT)
        pltOBS = valOBS[[iMon]][['SO4']][[iMtrx]][[iStat]]$timeser$value - valOBS[[iMon]][['XSO4']][[iMtrx]][[iStat]]$timeser$value
        if ( is.null(pltOBS) || length(pltOBS) == 0) { pltOBS = rep(NA, nT) }
        
        # der = derived from Na
        # dif = by difference between to cases
        pltSIM_ALL_GO03 = valSIM[[iMon]][['ALLsurf']][[iSpec]][[iMtrx]][[iStat]] 
        pltSIM_ALL_OV14 = valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]] 
        pltSIM_SALT_GO03_dif = pmax(valSIM[[iMon]][['ALLsurf']][[iSpec]][[iMtrx]][[iStat]] - valSIM[[iMon]][['NOSALT']][[iSpec]][[iMtrx]][[iStat]], 0)
        pltSIM_SALT_OV14_dif = pmax(valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]] - valSIM[[iMon]][['NOSALT']][[iSpec]][[iMtrx]][[iStat]], 0)
        pltSIM_SALT_GO03_der = valSIM[[iMon]][['ALLsurf']][['Na']][[iMtrx]][[iStat]] / ssNaMass * ssSO4Mass / 3
        pltSIM_SALT_OV14_der = valSIM[[iMon]][['SALT_OV14']][['Na']][[iMtrx]][[iStat]] / ssNaMass * ssSO4Mass / 3
        
        obsBADnZERO = !pmin(pltOBS==0, 1, na.rm = T)
        
        # wordData: process data ----
        frac_go03_dedial = mean(pltSIM_SALT_GO03_der/pltSIM_ALL_GO03)
        frac_ov14_dedial = mean(pltSIM_SALT_OV14_der/pltSIM_ALL_OV14)
        
        mnb_go03_obsdif = MNB(pltOBS[obsBADnZERO], pltSIM_SALT_GO03_dif[obsBADnZERO])
        mnb_go03_obsder = MNB(pltOBS[obsBADnZERO], pltSIM_SALT_GO03_der[obsBADnZERO])
        mnb_go03_derdif = MNB(pltSIM_SALT_GO03_der[obsBADnZERO], pltSIM_SALT_GO03_dif[obsBADnZERO])
        mnb_go03_dedial = MNB(pltSIM_SALT_GO03_der, pltSIM_SALT_GO03_dif)
        
        cor_go03_obsdif = cor(pltOBS[!is.na(pltOBS)], pltSIM_SALT_GO03_dif[!is.na(pltOBS)], method = 'pearson')
        cor_go03_obsder = cor(pltOBS[!is.na(pltOBS)], pltSIM_SALT_GO03_der[!is.na(pltOBS)], method = 'pearson')
        cor_go03_derdif = cor(pltSIM_SALT_GO03_der[!is.na(pltOBS)], pltSIM_SALT_GO03_dif[!is.na(pltOBS)], method = 'pearson')
        cor_go03_dedial = cor(pltSIM_SALT_GO03_der, pltSIM_SALT_GO03_dif, method = 'pearson')
        
        mnb_ov14_obsdif = MNB(pltOBS[obsBADnZERO], pltSIM_SALT_OV14_dif[obsBADnZERO])
        mnb_ov14_obsder = MNB(pltOBS[obsBADnZERO], pltSIM_SALT_OV14_der[obsBADnZERO])
        mnb_ov14_derdif = MNB(pltSIM_SALT_OV14_der[obsBADnZERO], pltSIM_SALT_OV14_dif[obsBADnZERO])
        mnb_ov14_dedial = MNB(pltSIM_SALT_OV14_der, pltSIM_SALT_OV14_dif)
        
        cor_ov14_obsdif = cor(pltOBS[!is.na(pltOBS)], pltSIM_SALT_OV14_dif[!is.na(pltOBS)], method = 'pearson')
        cor_ov14_obsder = cor(pltOBS[!is.na(pltOBS)], pltSIM_SALT_OV14_der[!is.na(pltOBS)], method = 'pearson')
        cor_ov14_derdif = cor(pltSIM_SALT_OV14_der[!is.na(pltOBS)], pltSIM_SALT_OV14_dif[!is.na(pltOBS)], method = 'pearson')
        cor_ov14_dedial = cor(pltSIM_SALT_OV14_der, pltSIM_SALT_OV14_dif, method = 'pearson')
        
        # wordData: copy data into output arrays ----
        iLoc = which(iStat == myStations)
        frac_dedial[[iMon]][[iMtrx]]$go03[iLoc] = frac_go03_dedial
        frac_dedial[[iMon]][[iMtrx]]$ov14[iLoc] = frac_ov14_dedial
        cor_dedial[[iMon]][[iMtrx]]$go03[iLoc] = cor_go03_dedial
        cor_dedial[[iMon]][[iMtrx]]$ov14[iLoc] = cor_ov14_dedial
        cor_obsder[[iMon]][[iMtrx]]$go03[iLoc] = cor_go03_obsder
        cor_obsder[[iMon]][[iMtrx]]$ov14[iLoc] = cor_ov14_obsder
        cor_obsdif[[iMon]][[iMtrx]]$go03[iLoc] = cor_go03_obsdif
        cor_obsdif[[iMon]][[iMtrx]]$ov14[iLoc] = cor_ov14_obsdif
        mnb_dedial[[iMon]][[iMtrx]]$go03[iLoc] = mnb_go03_dedial
        mnb_dedial[[iMon]][[iMtrx]]$ov14[iLoc] = mnb_ov14_dedial
        mnb_obsder[[iMon]][[iMtrx]]$go03[iLoc] = mnb_go03_obsder
        mnb_obsder[[iMon]][[iMtrx]]$ov14[iLoc] = mnb_ov14_obsder
        mnb_obsdif[[iMon]][[iMtrx]]$go03[iLoc] = mnb_go03_obsdif
        mnb_obsdif[[iMon]][[iMtrx]]$ov14[iLoc] = mnb_ov14_obsdif
      }
    }
  }
}



# plot A ----
if (plotA) {
  # plot A: set parameters ----
  iSpec = 'SO4'
  iMtrx = 'pm10'
  iMon = 'sum2013'
  
  go03Col = 'blue'
  ov14Col = 'magenta'
  
  refDate=2013001; strYear='2013'
  if(iMon == 'sum2013' || iMon == 'sum') { from = 2013182; to = 2013243; nT = 62 
  } else if(iMon == 'win2013' || iMon == 'win') { from = 2013001; to = 2013059; nT = 59 }
  nTh = nT * 24
    
  pdf(file = paste(fDir, fNameA, sep = '/'),
      paper = 'special', width = 7.5, height = 3.8)
  par(mar = c(0.5,0.5,0,0), mgp = c(3,0.5,0))
  
  
  # plot A: define plot order ----
  ## horizontal data
  lH = 0.07; rH = 0.015; dH = 0.01
  pltH = (1 - lH - rH - 2*dH)/3
  
  ## vertical data
  tV = 0.01; bV = 0.03;
  dLPV = 0.02
  dPLV = 0.2
  tLegV = 0.001 # legV = 0.03
  bLegV = 0.15 # legV = 0.03
  pltV = (1 - tV - bV - dLPV - dPLV - tLegV - bLegV)
  
  ## derived boundaries
  v0 = bV
  v1 = bV + bLegV
  v2 = bV + bLegV + dPLV
  v3 = bV + bLegV + dPLV + pltV
  v4 = bV + bLegV + dPLV + pltV + dLPV
  v5 = 1 - tV
  
  #TODO
  hz = rH
  h0 = lH
  h1 = lH + pltH
  h2 = lH + pltH + dH
  h3 = lH + 2*pltH + dH
  h4 = lH + 2*pltH + 2*dH
  h5 = lH + 3*pltH + 2*dH
  
  ## split the plotting pane
  split.screen(rbind(c(hz, h5, v4, v5), # legend
                     c(h0, h1, v2, v3), # top left
                     c(h2, h3, v2, v3), # top center
                     c(h4, h5, v2, v3), # top right
                     c(hz, h5, v0, v1) # legend
  ))
  
  iScreen = 2
  
  # plot A: iterate stations and create data plots ----
  # for (iStat in myStations) {
  for (iStat in c('DE0007R', 'DK0031R', 'FI0009R')) {
    screen(iScreen)
    iScreen = iScreen + 1
    
    pltOBS = rep(NA, nT)
    pltOBS = valOBS[[iMon]][['SO4']][[iMtrx]][[iStat]]$timeser$value - valOBS[[iMon]][['XSO4']][[iMtrx]][[iStat]]$timeser$value
    if ( is.null(pltOBS) || length(pltOBS) == 0) { pltOBS = rep(NA, nT) }
    
    # der = derived from Na
    # dif = by difference b
    pltSIM_ALL_GO03 = valSIM[[iMon]][['ALLsurf']][[iSpec]][[iMtrx]][[iStat]] 
    pltSIM_ALL_OV14 = valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]] 
    pltSIM_SALT_GO03_dif = pmax(valSIM[[iMon]][['ALLsurf']][[iSpec]][[iMtrx]][[iStat]] - valSIM[[iMon]][['NOSALT']][[iSpec]][[iMtrx]][[iStat]], 0)
    pltSIM_SALT_OV14_dif = pmax(valSIM[[iMon]][['SALT_OV14']][[iSpec]][[iMtrx]][[iStat]] - valSIM[[iMon]][['NOSALT']][[iSpec]][[iMtrx]][[iStat]], 0)
    pltSIM_SALT_GO03_der = valSIM[[iMon]][['ALLsurf']][['Na']][[iMtrx]][[iStat]] / ssNaMass * ssSO4Mass / 3
    pltSIM_SALT_OV14_der = valSIM[[iMon]][['SALT_OV14']][['Na']][[iMtrx]][[iStat]] / ssNaMass * ssSO4Mass / 3
    
    obsBADnZERO = !pmin(pltOBS==0, 1, na.rm = T)
    
    n_obsA = sum(!is.na(pltOBS))
    n_obsB = sum(obsBADnZERO)
    n_all = length(pltSIM_SALT_GO03_dif)
    
    maxVal = max(c(pltSIM_SALT_GO03_der, pltSIM_SALT_GO03_dif,pltSIM_SALT_OV14_der, pltSIM_SALT_OV14_dif))
    plot(2*c(0,maxVal), 2*c(0, maxVal), type = 'l', 
         xlim = c(0,maxVal), ylim = c(0,maxVal),
         col = 'black', yaxt = 'n', xlab = '', ylab = '')
    axis(2, labels = FALSE)
    # lines(c(0,0), c(maxVal, maxVal), col = 'grey')
    points(pltSIM_SALT_GO03_der, pltSIM_SALT_GO03_dif, col = go03Col)
    points(pltSIM_SALT_OV14_der, pltSIM_SALT_OV14_dif, col = ov14Col, pch = 2)
    # points(pltOBS[!is.na(pltOBS)], pltSIM_SALT_GO03_der[!is.na(pltOBS)], col = go03Col)
    # points(pltOBS[!is.na(pltOBS)], pltSIM_SALT_OV14_der[!is.na(pltOBS)], col = ov14Col, pch = 2)
    # mtext(list_stations[[iStat]], 3, line = 0.5, cex = 1.5)
    mtext(list_stations[[iStat]], 3, line = -1.5, cex = 1.2, at = 0, adj = 0)
    mtext(paste('R=',formatC(cor(pltSIM_SALT_GO03_der,pltSIM_SALT_GO03_dif), format = 'f', digits = 2), sep = ''), 3, col = go03Col, line = -2.8, at = 0, adj = 0)
    mtext(paste('R=',formatC(cor(pltSIM_SALT_OV14_der,pltSIM_SALT_OV14_dif), format = 'f', digits = 2), sep = ''), 3, col = ov14Col, line = -3.8, at = 0, adj = 0)
    mtext(paste('S/T=',formatC(mean(pltSIM_SALT_GO03_der/pltSIM_ALL_GO03), format = 'f', digits = 2), sep = ''), 1, col = go03Col, line = -2.3, at = maxVal*6.5/10, adj = 0)
    mtext(paste('S/T=',formatC(mean(pltSIM_SALT_OV14_der/pltSIM_ALL_OV14), format = 'f', digits = 2), sep = ''), 1, col = ov14Col, line = -1.3, at = maxVal*6.5/10, adj = 0)
  }
  
  # plot A: add annotations and legend ----
  ## PLOT AXIS
  screen(2)
  mtext(expression('sulf'['SeaSalt,diff'] ~ 'PM'[10] ~ '['*mu*'g/m'^3*']'), 2, line = 1.1, cex = 1.2)
  # mtext(expression('sulf'['SeaSalt,diff'] ~ 'PM'[10] ~ '['*mu*'g/m'^3*']'), 2, line = 2.5, cex = 1.5)
  # mtext(expression('= sulf'['total']*' - sulf'['noSeaSalt']), 2, line = 0.9, cex = 1.5)
  screen(3)
  mtext(expression('sulf'['SeaSalt,Na'] ~ 'PM'[10] ~ '['*mu*'g/m'^3*']'), 1, line = 2.3, cex = 1.2)
  # mtext(expression('sulf'['SeaSalt,Na'] ~ 'PM'[10] ~ '['*mu*'g/m'^3*'] = Na'^'+'['total'] %*% ' 0.0838194'), 1, line = 2.5, cex = 1.5)
  # mtext(expression('sea salt sulfate ['*mu*'g/m'^3*']'), 1, line = 2.5, cex = 1.5)
  # mtext(expression('= Na'^'+'['total']*' x 0.0838194'), 1, line = 0.9, cex = 1.5)
  
  ## TOP LEGEND
  # screen(1)
  # par(mar = c(0,0,0,0))
  # plot(c(0,1), c(0,1), ann = FALSE, xaxt = 'n', yaxt = 'n', type = 'n')
  # legend('topleft', legend = c(expression('PM'[10]~'sea salt sulfate ['*mu*'g/m'^3*']')),
  #        horiz = TRUE, bty = 'n')
  # legend('topright', legend = c(if(iMon == 'sum2013') 'summer 2013' else 'winter 2013',
  #                              paste(formatC(nT, format = 'd'), 'data points', sep = ' ')),
  #        horiz = TRUE, bty = 'n')
  
  ## BOTTOM LEGEND
  screen(5)
  par(mar = c(0,0,0,0))
  plot(c(0,1), c(0,1), ann = FALSE, xaxt = 'n', yaxt = 'n', type = 'n')
  legend('center', legend = c('GO03', 'OV14'), col = c(go03Col, ov14Col),
         pch = c(1,2), cex = 1.2, bty = 'n')
  legend('left', legend = c("R    = Pearsons's R",
                               expression('S/T = sulf'['seaSalt']*'/sulf'['total'])),
         cex = 1.2, bty = 'n', seg.len = 0.5, inset = c(-0.00, 0))
  legend('right', legend = c(if(iMon == 'sum2013') 'summer 2013' else 'winter 2013',
                                paste(formatC(nT, format = 'd'), 'data points', sep = ' ')),
         bty = 'n', cex = 1.2)
  # legend('topleft', legend = c("R    = Pearsons's R",
  #                              expression('S/T = sulf'['seaSalt']*'/sulf'['total']),
  #                              'GO03', 'OV14',
  #                              if(iMon == 'sum2013') 'summer 2013' else 'winter 2013',
  #                              paste(formatC(nT, format = 'd'), 'data points', sep = ' ')),
  #        col = c(NA, NA, go03Col, ov14Col, NA, NA), pch = c(NA, NA, 1, 2, NA, NA), 
  #        cex = 1.5, bty = 'n', seg.len = 0.5, inset = c(-0.05, 0),
  #        ncol = 3)
  
  # plot A: finalize ----
  close.screen(all.screens = TRUE)
  dev.off()
  
}



# plot B ----
if (plotB) {
  # plot B: create output file ----
  pdf(file = paste(fDir, fNameB, sep = '/'), paper = 'special', width = 11, height = 13)
  par(mar = c(0,0,0,0))
  
  winCol = 'cyan'
  sumCol = 'orange'
  
  # plot B: define plot order ----
  ## horizontal data
  lH = 0.085; rH = 0.01; dH = 0.01
  pltaH = (1 - lH - rH - dH)/2
  pltbH = 1 - lH - rH
  
  ## vertical data
  tV = 0.01; bV = 0.1;
  dLPV = 0.03
  dPPaV = 0.01
  dPPbV = 0.1
  legV = 0.0001 # legV = 0.03
  pltV = (1 - tV - bV - dLPV - dPPaV - dPPbV - legV) / 3
  
  ## derived boundaries
  v0 = bV
  v1 = bV + pltV
  v2 = bV + pltV + dPPbV
  v3 = bV + 2*pltV + dPPbV
  v4 = bV + 2*pltV + dPPbV + dPPaV
  v5 = bV + 3*pltV + dPPbV + dPPaV
  v6 = bV + 3*pltV + dPPbV + dPPaV + dLPV
  v7 = 1 - tV
  
  h0 = lH
  h1 = lH + pltaH
  h2 = lH + pltaH + dH
  h3 = lH + 2*pltaH + dH
  
  ## split the plotting pane
  split.screen(rbind(c(h0, h3, v6, v7), # legend
                     c(h0, h1, v4, v5), # top left
                     c(h2, h3, v4, v5), # top right
                     c(h0, h1, v2, v3), # center left
                     c(h2, h3, v2, v3), # center right
                     c(h0, h1, v0, v1), # bot left
                     c((h2), h3, (1-v7)*2, v1) # bot right
                     ))
  
  
  # plot B: start plotting ----
  ## top legend
  # screen(1)
  # plot(c(0,1), c(0,1), type = 'n', ann = FALSE, xaxt = 'n', yaxt = 'n')
  # legend('left', legend = c('winter 2013', 'summer 2013', ' ', 'GO03', 'OV14'),
  #        col = c(winCol, sumCol, NA, 'black', 'black'),
  #        lty = c(1, 1, NA, NA, NA), lwd = 3, seg.len = 1,
  #        pch = c(NA, NA, NA, 16, 15), pt.cex = 2, 
  #        horiz = TRUE, cex = 1.3, bty = 'n')
  
  ## top left plot
  screen(2)
  iMtrx = 'pm10'
  plot(c(1,22), c(-0.2, 1.0), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  linepointPlot(myX - myShft, cor_dedial[['win2013']][[iMtrx]][['go03']], col = winCol, pch = 16, cex = 1.5)
  linepointPlot(myX - myShft, cor_dedial[['sum2013']][[iMtrx]][['go03']], col = sumCol, pch = 16, cex = 1.5)
  linepointPlot(myX + myShft, cor_dedial[['win2013']][[iMtrx]][['ov14']], col = winCol, pch = 15, cex = 1.5)
  linepointPlot(myX + myShft, cor_dedial[['sum2013']][[iMtrx]][['ov14']], col = sumCol, pch = 15, cex = 1.5)
  axis(2, las = 1)
  mtext("Pearson's R", 2, line = 3, cex = 1.5, font = 2)
  mtext(expression('PM'[10]), 3, line = 0.2, cex = 1.5, font = 2)
  legend('bottomleft', legend = c(expression('cor(ssSulf'['p,Na']*' , ssSulf'['p,diff']*')')),
         pch = c(NA), col = 'black', cex = 1.2, bty = 'n')
  ## / top left plot
  
  ## top right plot
  screen(3)
  iMtrx = 'pm25'
  plot(c(1,22), c(-0.2, 1.0), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  linepointPlot(myX - myShft, cor_dedial[['win2013']][[iMtrx]][['go03']], col = winCol, pch = 16, cex = 1.5)
  linepointPlot(myX - myShft, cor_dedial[['sum2013']][[iMtrx]][['go03']], col = sumCol, pch = 16, cex = 1.5)
  linepointPlot(myX + myShft, cor_dedial[['win2013']][[iMtrx]][['ov14']], col = winCol, pch = 15, cex = 1.5)
  linepointPlot(myX + myShft, cor_dedial[['sum2013']][[iMtrx]][['ov14']], col = sumCol, pch = 15, cex = 1.5)
  mtext(expression('PM'[2.5]), 3, line = 0.2, cex = 1.5, font = 2)
  legend('bottomleft', legend = c(expression('cor(ssSulf'['p,Na']*' , ssSulf'['p,diff']*')')),
         pch = c(NA), col = 'black', cex = 1.2, bty = 'n')
  ## / top right plot
  
  ## center left plot
  screen(4)
  iMtrx = 'pm10'
  plot(c(1,22), c(-0.8, 0.8), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  linepointPlot(myX - myShft, mnb_dedial[['win2013']][[iMtrx]][['go03']], col = winCol, pch = 16, cex = 1.5)
  linepointPlot(myX - myShft, mnb_dedial[['sum2013']][[iMtrx]][['go03']], col = sumCol, pch = 16, cex = 1.5)
  linepointPlot(myX + myShft, mnb_dedial[['win2013']][[iMtrx]][['ov14']], col = winCol, pch = 15, cex = 1.5)
  linepointPlot(myX + myShft, mnb_dedial[['sum2013']][[iMtrx]][['ov14']], col = sumCol, pch = 15, cex = 1.5)
  # par(mgp = c(3,3,0) )
  axis(1, at = myX, labels = myStations, las = 3, hadj = 1)
  axis(2, las = 1)
  mtext("Mean Normalized Bias", 2, line = 3, cex = 1.5, font = 2)
  legend('bottomleft', legend = c(expression('mnb(ssSulf'['p,Na']*' , ssSulf'['p,diff']*')')),
         pch = c(NA), col = 'black', cex = 1.2, bty = 'n')
  ## / center left plot
  
  ## center right plot
  screen(5)
  iMtrx = 'pm25'
  plot(c(1,22), c(-0.8, 0.8), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  linepointPlot(myX - myShft, mnb_dedial[['win2013']][[iMtrx]][['go03']], col = winCol, pch = 16, cex = 1.5)
  linepointPlot(myX - myShft, mnb_dedial[['sum2013']][[iMtrx]][['go03']], col = sumCol, pch = 16, cex = 1.5)
  linepointPlot(myX + myShft, mnb_dedial[['win2013']][[iMtrx]][['ov14']], col = winCol, pch = 15, cex = 1.5)
  linepointPlot(myX + myShft, mnb_dedial[['sum2013']][[iMtrx]][['ov14']], col = sumCol, pch = 15, cex = 1.5)
  axis(1, at = myX, labels = myStations, las = 3)
  legend('bottomleft', legend = c(expression('mnb(ssSulf'['p,Na']*' , ssSulf'['p,diff']*')')),
         pch = c(NA), col = 'black', cex = 1.2, bty = 'n')
  ## / center right plot
  
  
  
  ## center left plot
  screen(6)
  iMtrx = 'pm10'
  plot(c(1,22), c(-0.8, 0.8), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  linepointPlot(myX - myShft, frac_dedial[['win2013']][[iMtrx]][['go03']], col = winCol, pch = 16, cex = 1.5)
  linepointPlot(myX - myShft, frac_dedial[['sum2013']][[iMtrx]][['go03']], col = sumCol, pch = 16, cex = 1.5)
  linepointPlot(myX + myShft, frac_dedial[['win2013']][[iMtrx]][['ov14']], col = winCol, pch = 15, cex = 1.5)
  linepointPlot(myX + myShft, frac_dedial[['sum2013']][[iMtrx]][['ov14']], col = sumCol, pch = 15, cex = 1.5)
  # par(mgp = c(3,3,0) )
  axis(1, at = myX, labels = myStations, las = 3, hadj = 1)
  axis(2, las = 1)
  mtext("Mean Normalized Bias", 2, line = 3, cex = 1.5, font = 2)
  legend('bottomleft', legend = c(expression('mnb(ssSulf'['p,Na']*' , ssSulf'['p,diff']*')')),
         pch = c(NA), col = 'black', cex = 1.2, bty = 'n')
  ## / center left plot
  
  ## center right plot
  screen(7)
  iMtrx = 'pm25'
  plot(c(1,22), c(-0.8, 0.8), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  linepointPlot(myX - myShft, frac_dedial[['win2013']][[iMtrx]][['go03']], col = winCol, pch = 16, cex = 1.5)
  linepointPlot(myX - myShft, frac_dedial[['sum2013']][[iMtrx]][['go03']], col = sumCol, pch = 16, cex = 1.5)
  linepointPlot(myX + myShft, frac_dedial[['win2013']][[iMtrx]][['ov14']], col = winCol, pch = 15, cex = 1.5)
  linepointPlot(myX + myShft, frac_dedial[['sum2013']][[iMtrx]][['ov14']], col = sumCol, pch = 15, cex = 1.5)
  axis(1, at = myX, labels = myStations, las = 3)
  legend('bottomleft', legend = c(expression('mnb(ssSulf'['p,Na']*' , ssSulf'['p,diff']*')')),
         pch = c(NA), col = 'black', cex = 1.2, bty = 'n')
  ## / center right plot
  
  
  
#   ## bottom left plot
#   screen(6)
#   myVals = list('pm10' = which(!is.na(cor_obsdif[['win2013']][['pm10']][['go03']])),
#                 'pm25' = which(!is.na(cor_obsdif[['win2013']][['pm25']][['go03']])))
#   myLen = list('pm10' = length(myVals$pm10), 'pm25' = length(myVals$pm25))
#   maxLen = length(myX)
#   brkLen = (maxLen - myLen$pm10 + 1 + myLen$pm25)/2
#   
#   plot(c(1,maxLen), c(-0.8, 1), type = 'n', xaxt = 'n', yaxt = 'n')
#   lines(c(-10,50), c(0,0), col = 'black')
#   lines(c(brkLen, brkLen), c(-10, 10), col = 'black')
#   axis(2, las = 1)
#   mtext("Pearson's R", 2, line = 3, cex = 1.5, font = 2)
#   
#   legend('bottomright', legend = c(expression('cor(ssSulf'['obs']*' , ssSulf'['p,Na']*')'),
#                                    expression('cor(ssSulf'['obs']*' , ssSulf'['p,diff']*')')),
#          pch = c(18, 5), col = 'black', cex = 1.2, pt.cex = c(2,1.5))
#   
#   iMtrx = 'pm25'
#   linepointsAbPlot(1:myLen[[iMtrx]] - myShft, cor_obsdif[['win2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], cor_obsder[['win2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], 
#                 col = winCol, pchBot = 1, pchTop = 16, cex = 1.5)
#   linepointsAbPlot(1:myLen[[iMtrx]] - myShft, cor_obsdif[['sum2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], cor_obsder[['sum2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], 
#                 col = sumCol, pchBot = 1, pchTop = 16, cex = 1.5)
#   linepointsAbPlot(1:myLen[[iMtrx]] + myShft, cor_obsdif[['win2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], cor_obsder[['win2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], 
#                 col = winCol, pchBot = 0, pchTop = 15, cex = 1.5)
#   linepointsAbPlot(1:myLen[[iMtrx]] + myShft, cor_obsdif[['sum2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], cor_obsder[['sum2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], 
#                 col = sumCol, pchBot = 0, pchTop = 15, cex = 1.5)
#   mtext(expression('PM'[2.5]), 3, line = -2, at = brkLen / 2, cex = 1.5, font = 2)
#   axis(1, at = 1:myLen[[iMtrx]], labels = myStations[myVals[[iMtrx]]], las = 3)
#   
#   iMtrx = 'pm10'
#   linepointsAbPlot((maxLen - myLen[[iMtrx]] + 1):maxLen - myShft, cor_obsdif[['win2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], cor_obsder[['win2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], 
#                    col = winCol, pchBot = 1, pchTop = 16, cex = 1.5)
#   linepointsAbPlot((maxLen - myLen[[iMtrx]] + 1):maxLen - myShft, cor_obsdif[['sum2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], cor_obsder[['sum2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], 
#                    col = sumCol, pchBot = 1, pchTop = 16, cex = 1.5)
#   linepointsAbPlot((maxLen - myLen[[iMtrx]] + 1):maxLen + myShft, cor_obsdif[['win2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], cor_obsder[['win2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], 
#                    col = winCol, pchBot = 0, pchTop = 15, cex = 1.5)
#   linepointsAbPlot((maxLen - myLen[[iMtrx]] + 1):maxLen + myShft, cor_obsdif[['sum2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], cor_obsder[['sum2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], 
#                    col = sumCol, pchBot = 0, pchTop = 15, cex = 1.5)
#   mtext(expression('PM'[10]), 3, line = -2, at = brkLen + (maxLen - brkLen) / 2, cex = 1.5, font = 2)
#   axis(1, at = (maxLen - myLen[[iMtrx]] + 1):maxLen, labels = myStations[myVals[[iMtrx]]], las = 3)
#   ## / bottom left plot
#   
#   
#   ## bottom right legend
#   screen(7)
#   plot(c(0,1), c(0,1), type = 'n', ann = FALSE, xaxt = 'n', yaxt = 'n')
#   legend('topleft', legend = c('winter 2013', 'summer 2013', 'GO03', 'OV14'),
#          col = c(winCol, sumCol, 'black', 'black'),
#          lty = c(1, 1, NA, NA), lwd = 3, seg.len = 1,
#          pch = c(NA, NA, 16, 15), pt.cex = 2, 
#          cex = 1.3, bty = 'n', ncol = 3, 
#          text.width = 0.4, inset = 0.07, 
#          title = 'Legend:', title.adj = 0)
#   legend('bottomleft', legend = c(expression('ssSulf'['obs'] ~ '= measured sea salt sulfate'),
#                                   expression('                     from EMEP'),
#                                   '', 
#                                expression('ssSulf'['p,Na'] ~ '= predicted sea salt sulfate, '),
#                                expression('                     calculated from Na'^'+'),
#                                '', 
#                                expression('ssSulf'['p,diff'] ~ '= sea salt sulfate obs'),
#                                expression('                     calculated from NoSalt case')),
#          col = NA, bty = 'n', cex = 1.3, 
#          text.width = 0.4, inset = c(0, 0.1))
#   ## / bottom right legend
  
  
  # plot B: finalize plot ----
  close.screen(all.screens = TRUE)
  dev.off()

}




# plot C ----
if (plotC) {
  # plot B: create output file ----
  pdf(file = paste(fDir, fNameC, sep = '/'), paper = 'special', width = 11, height = 16.38)
  par(mar = c(0,0,0,0))
  
  winCol = 'cyan'
  sumCol = 'orange'
  
  # plot C: define plot order ----
  ## horizontal data
  lH = 0.085; rH = 0.01; dH = 0.01
  pltaH = (1 - lH - rH - dH)/2
  pltbH = 1 - lH - rH
  
  ## vertical data
  tV = 0.01; bV = 0.1;
  dLPV = 0.03
  dPPaV = 0.006
  dPPbV = 0.075
  legV = 0.0001 # legV = 0.03
  pltV = (1 - tV - bV - dLPV - 2*dPPaV - dPPbV - legV) / 4
  
  ## derived boundaries
  v0 = bV
  v1 = bV + pltV
  v2 = bV + pltV + dPPbV
  v3 = bV + 2*pltV + dPPbV
  v4 = bV + 2*pltV + dPPbV + dPPaV
  v5 = bV + 3*pltV + dPPbV + dPPaV
  v6 = bV + 3*pltV + dPPbV + 2*dPPaV
  v7 = bV + 4*pltV + dPPbV + 2*dPPaV
  v8 = bV + 4*pltV + dPPbV + 2*dPPaV + dLPV
  v9 = 1 - tV
  
  h0 = lH
  h1 = lH + pltaH
  h2 = lH + pltaH + dH
  h3 = lH + 2*pltaH + dH
  
  ## split the plotting pane
  split.screen(rbind(c(h0, h3, v8, v9), # legend
                     c(h0, h1, v6, v7), # top left
                     c(h2, h3, v6, v7), # top right
                     c(h0, h1, v4, v5), # center left
                     c(h2, h3, v4, v5), # center right
                     c(h0, h1, v2, v3), # center left
                     c(h2, h3, v2, v3), # center right
                     c(h0, h1, v0, v1), # bot left
                     c((h2), h3, (1-v7), v1) # bot right
  ))
  
  
  # plot C: start plotting ----
  ## top legend
  # screen(1)
  # plot(c(0,1), c(0,1), type = 'n', ann = FALSE, xaxt = 'n', yaxt = 'n')
  # legend('left', legend = c('winter 2013', 'summer 2013', ' ', 'GO03', 'OV14'),
  #        col = c(winCol, sumCol, NA, 'black', 'black'),
  #        lty = c(1, 1, NA, NA, NA), lwd = 3, seg.len = 1,
  #        pch = c(NA, NA, NA, 16, 15), pt.cex = 2, 
  #        horiz = TRUE, cex = 1.3, bty = 'n')
  
  ## top left plot
  screen(2)
  iMtrx = 'pm10'
  plot(c(1,22), c(-0.2, 1.0), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  linepointPlot(myX - myShft, cor_dedial[['win2013']][[iMtrx]][['go03']], col = winCol, pch = 16, cex = 1.5)
  linepointPlot(myX - myShft, cor_dedial[['sum2013']][[iMtrx]][['go03']], col = sumCol, pch = 16, cex = 1.5)
  linepointPlot(myX + myShft, cor_dedial[['win2013']][[iMtrx]][['ov14']], col = winCol, pch = 15, cex = 1.5)
  linepointPlot(myX + myShft, cor_dedial[['sum2013']][[iMtrx]][['ov14']], col = sumCol, pch = 15, cex = 1.5)
  axis(2, las = 1)
  mtext("Pearson's R", 2, line = 3, cex = 1.5, font = 2)
  mtext(expression('PM'[10]), 3, line = 0.2, cex = 1.5, font = 2)
  legend('bottomleft', legend = c(expression('cor(ssSulf'['p,Na']*' , ssSulf'['p,diff']*')')),
         pch = c(NA), col = 'black', cex = 1.2, bty = 'n')
  ## / top left plot
  
  ## top right plot
  screen(3)
  iMtrx = 'pm25'
  plot(c(1,22), c(-0.2, 1.0), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  linepointPlot(myX - myShft, cor_dedial[['win2013']][[iMtrx]][['go03']], col = winCol, pch = 16, cex = 1.5)
  linepointPlot(myX - myShft, cor_dedial[['sum2013']][[iMtrx]][['go03']], col = sumCol, pch = 16, cex = 1.5)
  linepointPlot(myX + myShft, cor_dedial[['win2013']][[iMtrx]][['ov14']], col = winCol, pch = 15, cex = 1.5)
  linepointPlot(myX + myShft, cor_dedial[['sum2013']][[iMtrx]][['ov14']], col = sumCol, pch = 15, cex = 1.5)
  mtext(expression('PM'[2.5]), 3, line = 0.2, cex = 1.5, font = 2)
  legend('bottomleft', legend = c(expression('cor(ssSulf'['p,Na']*' , ssSulf'['p,diff']*')')),
         pch = c(NA), col = 'black', cex = 1.2, bty = 'n')
  ## / top right plot
  
  ## center left plot
  screen(4)
  iMtrx = 'pm10'
  plot(c(1,22), c(-0.8, 0.8), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  linepointPlot(myX - myShft, mnb_dedial[['win2013']][[iMtrx]][['go03']], col = winCol, pch = 16, cex = 1.5)
  linepointPlot(myX - myShft, mnb_dedial[['sum2013']][[iMtrx]][['go03']], col = sumCol, pch = 16, cex = 1.5)
  linepointPlot(myX + myShft, mnb_dedial[['win2013']][[iMtrx]][['ov14']], col = winCol, pch = 15, cex = 1.5)
  linepointPlot(myX + myShft, mnb_dedial[['sum2013']][[iMtrx]][['ov14']], col = sumCol, pch = 15, cex = 1.5)
  # par(mgp = c(3,3,0) )
  axis(2, las = 1)
  mtext("Mean Normalized Bias", 2, line = 3, cex = 1.5, font = 2)
  legend('bottomleft', legend = c(expression('mnb(ssSulf'['p,Na']*' , ssSulf'['p,diff']*')')),
         pch = c(NA), col = 'black', cex = 1.2, bty = 'n')
  ## / center left plot
  
  ## center right plot
  screen(5)
  iMtrx = 'pm25'
  plot(c(1,22), c(-0.8, 0.8), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  linepointPlot(myX - myShft, mnb_dedial[['win2013']][[iMtrx]][['go03']], col = winCol, pch = 16, cex = 1.5)
  linepointPlot(myX - myShft, mnb_dedial[['sum2013']][[iMtrx]][['go03']], col = sumCol, pch = 16, cex = 1.5)
  linepointPlot(myX + myShft, mnb_dedial[['win2013']][[iMtrx]][['ov14']], col = winCol, pch = 15, cex = 1.5)
  linepointPlot(myX + myShft, mnb_dedial[['sum2013']][[iMtrx]][['ov14']], col = sumCol, pch = 15, cex = 1.5)
  legend('bottomleft', legend = c(expression('mnb(ssSulf'['p,Na']*' , ssSulf'['p,diff']*')')),
         pch = c(NA), col = 'black', cex = 1.2, bty = 'n')
  ## / center right plot
  
  
  
  ## center left plot
  screen(6)
  iMtrx = 'pm10'
  plot(c(1,22), c(-0.1, 0.7), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  linepointPlot(myX - myShft, frac_dedial[['win2013']][[iMtrx]][['go03']], col = winCol, pch = 16, cex = 1.5)
  linepointPlot(myX - myShft, frac_dedial[['sum2013']][[iMtrx]][['go03']], col = sumCol, pch = 16, cex = 1.5)
  linepointPlot(myX + myShft, frac_dedial[['win2013']][[iMtrx]][['ov14']], col = winCol, pch = 15, cex = 1.5)
  linepointPlot(myX + myShft, frac_dedial[['sum2013']][[iMtrx]][['ov14']], col = sumCol, pch = 15, cex = 1.5)
  # par(mgp = c(3,3,0) )
  axis(1, at = myX, labels = myStations, las = 3, hadj = 1)
  axis(2, las = 1)
  mtext("Sea Salt Sulfate Fraction", 2, line = 3, cex = 1.5, font = 2)
  legend('bottomleft', legend = c(expression('sulf'['seaSalt,Na']*'/sulf'['tot'])),
         pch = c(NA), col = 'black', cex = 1.2, bty = 'n')
  ## / center left plot
  
  ## center right plot
  screen(7)
  iMtrx = 'pm25'
  plot(c(1,22), c(-0.1, 0.7), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  linepointPlot(myX - myShft, frac_dedial[['win2013']][[iMtrx]][['go03']], col = winCol, pch = 16, cex = 1.5)
  linepointPlot(myX - myShft, frac_dedial[['sum2013']][[iMtrx]][['go03']], col = sumCol, pch = 16, cex = 1.5)
  linepointPlot(myX + myShft, frac_dedial[['win2013']][[iMtrx]][['ov14']], col = winCol, pch = 15, cex = 1.5)
  linepointPlot(myX + myShft, frac_dedial[['sum2013']][[iMtrx]][['ov14']], col = sumCol, pch = 15, cex = 1.5)
  axis(1, at = myX, labels = myStations, las = 3)
  legend('bottomleft', legend = c(expression('sulf'['seaSalt,Na']*'/sulf'['tot'])),
         pch = c(NA), col = 'black', cex = 1.2, bty = 'n')
  ## / center right plot
  
  
  
  ## bottom left plot
  screen(8)
  myVals = list('pm10' = which(!is.na(cor_obsdif[['win2013']][['pm10']][['go03']])),
                'pm25' = which(!is.na(cor_obsdif[['win2013']][['pm25']][['go03']])))
  myLen = list('pm10' = length(myVals$pm10), 'pm25' = length(myVals$pm25))
  maxLen = length(myX)
  brkLen = (maxLen - myLen$pm10 + 1 + myLen$pm25)/2
  
  plot(c(1,maxLen), c(-0.8, 1), type = 'n', xaxt = 'n', yaxt = 'n')
  lines(c(-10,50), c(0,0), col = 'black')
  lines(c(brkLen, brkLen), c(-10, 10), col = 'black')
  axis(2, las = 1)
  mtext("Pearson's R", 2, line = 3, cex = 1.5, font = 2)
  
  legend('bottomright', legend = c(expression('cor(ssSulf'['obs']*' , ssSulf'['p,Na']*')'),
                                   expression('cor(ssSulf'['obs']*' , ssSulf'['p,diff']*')')),
         pch = c(18, 5), col = 'black', cex = 1.2, pt.cex = c(2,1.5))
  
  iMtrx = 'pm25'
  linepointsAbPlot(1:myLen[[iMtrx]] - myShft, cor_obsdif[['win2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], cor_obsder[['win2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], 
                col = winCol, pchBot = 1, pchTop = 16, cex = 1.5)
  linepointsAbPlot(1:myLen[[iMtrx]] - myShft, cor_obsdif[['sum2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], cor_obsder[['sum2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], 
                col = sumCol, pchBot = 1, pchTop = 16, cex = 1.5)
  linepointsAbPlot(1:myLen[[iMtrx]] + myShft, cor_obsdif[['win2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], cor_obsder[['win2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], 
                col = winCol, pchBot = 0, pchTop = 15, cex = 1.5)
  linepointsAbPlot(1:myLen[[iMtrx]] + myShft, cor_obsdif[['sum2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], cor_obsder[['sum2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], 
                col = sumCol, pchBot = 0, pchTop = 15, cex = 1.5)
  mtext(expression('PM'[2.5]), 3, line = -2, at = brkLen / 2, cex = 1.5, font = 2)
  axis(1, at = 1:myLen[[iMtrx]], labels = myStations[myVals[[iMtrx]]], las = 3)
  
  iMtrx = 'pm10'
  linepointsAbPlot((maxLen - myLen[[iMtrx]] + 1):maxLen - myShft, cor_obsdif[['win2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], cor_obsder[['win2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], 
                   col = winCol, pchBot = 1, pchTop = 16, cex = 1.5)
  linepointsAbPlot((maxLen - myLen[[iMtrx]] + 1):maxLen - myShft, cor_obsdif[['sum2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], cor_obsder[['sum2013']][[iMtrx]][['go03']][myVals[[iMtrx]]], 
                   col = sumCol, pchBot = 1, pchTop = 16, cex = 1.5)
  linepointsAbPlot((maxLen - myLen[[iMtrx]] + 1):maxLen + myShft, cor_obsdif[['win2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], cor_obsder[['win2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], 
                   col = winCol, pchBot = 0, pchTop = 15, cex = 1.5)
  linepointsAbPlot((maxLen - myLen[[iMtrx]] + 1):maxLen + myShft, cor_obsdif[['sum2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], cor_obsder[['sum2013']][[iMtrx]][['ov14']][myVals[[iMtrx]]], 
                   col = sumCol, pchBot = 0, pchTop = 15, cex = 1.5)
  mtext(expression('PM'[10]), 3, line = -2, at = brkLen + (maxLen - brkLen) / 2, cex = 1.5, font = 2)
  axis(1, at = (maxLen - myLen[[iMtrx]] + 1):maxLen, labels = myStations[myVals[[iMtrx]]], las = 3)
  ## / bottom left plot
  
  
  ## bottom right legend
  screen(9)
  plot(c(0,1), c(0,1), type = 'n', ann = FALSE, xaxt = 'n', yaxt = 'n')
  legend('topleft', legend = c('winter 2013', 'summer 2013', 'GO03', 'OV14'),
         col = c(winCol, sumCol, 'black', 'black'),
         lty = c(1, 1, NA, NA), lwd = 3, seg.len = 1,
         pch = c(NA, NA, 16, 15), pt.cex = 2, 
         cex = 1.3, bty = 'n', ncol = 3, 
         text.width = 0.4, inset = 0.07, 
         title = 'Legend:', title.adj = 0)
  legend('bottomleft', legend = c(expression('ssSulf'['obs'] ~ '= measured sea salt sulfate'),
                                  expression('                     from EMEP'),
                                  '', 
                               expression('ssSulf'['p,Na'] ~ '= predicted sea salt sulfate, '),
                               expression('                     calculated from Na'^'+'),
                               '', 
                               expression('ssSulf'['p,diff'] ~ '= sea salt sulfate obs'),
                               expression('                     calculated from NoSalt case')),
         col = NA, bty = 'n', cex = 1.3, 
         text.width = 0.4, inset = c(0, 0.1))
  ## / bottom right legend
  
  
  # plot C: finalize plot ----
  close.screen(all.screens = TRUE)
  dev.off()
  
}